
clear
format long


%==========================================================================
%==========================================================================
%PROGRAM NOTES:
%
%Dependencies:  The files
%  parmVectorField.m
%  ode_1.m
%  horner.m
%must be in the working directory.
%
%Basic Use: at the command prompt type   >> parmVectorField
%
%Advanced Use: See comments in the `program parameters' section.
%              Especially see the comments for the parameter 
%              `numPoints' with reguards to runtime.
%
%Output: The output is a plot of the phase space, with the fixed points
%marked, the eigenvectors of the hyperbolic fixed points drawn, and the 
%parameteizations of the stable and unstable manifolds of the hyperbolic 
%fixed points plotted.  The endpoints of the manifold segments, as well 
%as the tangent vectors at the endpoints are output as well.  Finally
%an error estimate is given.
%
%Comments:  The code is not clean.  I have not `functionized' the 
%           code very much.  It's almost 375 lines, and this could be 
%           cut a lot.  However I wanted it to be readable in a linear 
%           fassion, and for you to not have to go digging into function 
%           calls in multiple files in order to see what's going on.
%           Because of this there is quite a bit of repetion in 
%           the loops and initializations.  Also, twhile he eigenvalues 
%           and eigenvectors are entered as algebraic expressions, in  
%           another problem they would be computed numerically.
%==========================================================================
%==========================================================================


%========================================================
%Program Parameters:
%========================================================

%Integration and System Constants:
  %Number of initial conditions in the phase space plot:
  %     Set to `numPoints = 0' to see how fast the parameterization 
  %     computations actually run.  I typically set at 50, but 
  %     then the program takes a few seconds to run.  However, most
  %     of this time is spent in computing the phase space example
  %     orbits.  If you set to zero, the program runs in about a second.
    numPoints = 8;
  %Length of time to integrate the example orbits.  Increase
  %if the example orbits are too short (needed for smaller L).
    ex_finalTime = 8.0;
    
  %Initialize the initial condition containers:
    y_1 = linspace(-1.00001, 1.00001, numPoints);
    y_2 = zeros(numPoints, 1);
    IC = [y_1;
          y_2'];
    
  %Vectorfield Parameter:
    %NOTE: L = 0.3 to L = 1.1 work well 
    %without changing other parameter 
    %values.  For other values of L the 
    %`Main Constants' need to be adjusted.
    %Also, at any particular value of L the
    %parameters can be pushed to get maximum
    %performance.
    L = 0.85;

 
    
    
%Main Constants:-----------------------------------------   
  %Parameterization Constants:
      %Truncation Order of the Parameterization:
        parmOrder = 60;
      %Parameter Domain is the interval [0, stopParm]
      %Plots of parameterizations are P([0, stopParm]):
        stopParm = 1.5;
      %Number of parameters points to plot:
        numParmPoints = 100;
%-------------------------------------------------------
        

%======================================================================
%Example Phase Space Diagram
%======================================================================
  

%Integrate and plot the vector field:
figure
hold on
for i = 1:numPoints
    tspan = [0 ex_finalTime];
    y0 = [IC(1, i); IC(2, i)];
    options=odeset('RelTol',1e-12,'AbsTol',1e-12);
    [t,y] = ode45('ode_1', tspan, y0, options, flag, L);
    plot(y(:,1), y(:,2), 'k');
end    



%=====================================================================
%Phase Space Land Marks
%=====================================================================

%Fixed Points:
  %Hyperbolic:
     p0 = [-1;
            0];
        
     p1 = [1;
           0];
           
  %Eliptic:
     q = [0;
          0];
        
%Plot the fixed points:      
plot(p0(1), p0(2), 'g*');      
plot(p1(1), p1(2), 'g*');       
plot(q(1), q(2),   'r*'); 

%Hyperbolic Eigenvectors:
   normalization = 1/(sqrt(1+2*L^2));

   xi_s = normalization*[1;
                        -sqrt(2)*L];
       
   xi_u = normalization*[1;
                         sqrt(2)*L];
       
%Plot the eigenvectors:                     
quiver(p0(1), p0(2), xi_s(1), xi_s(2), 'r')       
quiver(p0(1), p0(2), xi_u(1), xi_u(2), 'b')  
quiver(p1(1), p1(2), -xi_s(1), -xi_s(2), 'r')  
quiver(p1(1), p1(2), -xi_u(1), -xi_u(2), 'b')  
         

%=======================================================================
%MAIN PROGRAM: Parameterization Computations:
%=======================================================================


%Parameterization initializations:--------------------------------------
  %Hyperbolic Eigenvalues:
  lambda_s = -sqrt(2)*L;
  lambda_u = sqrt(2)*L;
  
  %Order of the polynomial approximation:
  N = parmOrder;
  
  %Initialize the coefficient vectors (should be put into function calls)
  %
  %Note: 0s denotes coefficients associated with the stable manifold
  %         at fixed point 0.
  %         Similarly for 0u, 1s, and 1u.
  
    %The p0 stable coefficients:
      A0s = zeros(N+1, 1);
      B0s = zeros(N+1, 1);

    %The p0 unstable coefficients:
      A0u = zeros(N+1, 1);
      B0u = zeros(N+1, 1);
         
    %The p1 stable coefficients:
      A1s = zeros(N+1, 1);
      B1s = zeros(N+1, 1);

    %The p1 unstable coefficients:
      A1u = zeros(N+1, 1);
      B1u = zeros(N+1, 1);         

   %Also need the `c_n' coefficients:
   %  These are the coefficients for 
   %  (\sum a_n x^n)^3
     c0s = zeros(N+1, 1);
     c0u = zeros(N+1, 1);
     c1s = zeros(N+1, 1);
     c1u = zeros(N+1, 1);
%--------------------------------------------------------------------------
     
%--------------------------------------------------------------------------     
  %Initialize the zero-th and first order coefficients:
  %   Initializations should be put into function calls.
  %   (Note: coefficients are offset by one, so a_0 = A(1) etc...) 
    
    %zero order:
    A0s(1) = p0(1);
    B0s(1) = p0(2);
    
    A0u(1) = p0(1);
    B0u(1) = p0(2);
    
    A1s(1) = p1(1);
    B1s(1) = p1(2);
    
    A1u(1) = p1(1);
    B1u(1) = p1(2);

    c0s(1) = A0s(1)^3;
    c0u(1) = A0u(1)^3;
    
    c1s(1) = A1s(1)^3;
    c1u(1) = A1u(1)^3;
    
    %first order:
    A0s(2) = xi_s(1);
    B0s(2) = xi_s(2);
    
    A0u(2) = xi_u(1);
    B0u(2) = xi_u(2);
    
    A1s(2) = -xi_s(1);
    B1s(2) = -xi_s(2);
    
    A1u(2) = -xi_u(1);
    B1u(2) = -xi_u(2);
   
    c0s(2) = 3*A0s(1)^2*A0s(2);
    c0u(2) = 3*A0u(1)^2*A0u(2);
    c1s(2) = 3*A1s(1)^2*A1s(2);
    c1u(2) = 3*A1u(1)^2*A1u(2);
%----------------------------------------------------------------------
  
%======================================================================           
%Main Computation
%======================================================================
        
%Solve the homological equation for the remaining coefficients:    
%Note: (should be rewriten as a function call to reduce redundency) 
for n = 1:(N-1)

  %Homological Equation Matrices:  
  M0s = [lambda_s*(n+1),                 -1;
         L^2*(1 - 3*(c0s(1)/A0s(1))),    lambda_s*(n+1)];
  
  M0u = [lambda_u*(n+1),                 -1;
         L^2*(1 - 3*(c0u(1)/A0u(1))),    lambda_u*(n+1)];
  
  M1s = [lambda_s*(n+1),                 -1;
         L^2*(1 - 3*(c1s(1)/A1s(1))),    lambda_s*(n+1)];
  
  M1u = [lambda_u*(n+1),                 -1;
         L^2*(1 - 3*(c1u(1)/A1u(1))),    lambda_u*(n+1)];
  
  %rhs if the homological equations depend
  %recurssively on the lower order coefficients:     
  h0s = 0;
  h0u = 0;
  h1s = 0;
  h1u = 0;

  for k = 0:(n-1)
    h0s = h0s + (k+1)*(c0s(k+2)*A0s(n-k+1) - 3*A0s(k+2)*c0s(n-k+1));     
    h0u = h0u + (k+1)*(c0u(k+2)*A0u(n-k+1) - 3*A0u(k+2)*c0u(n-k+1));  
    h1s = h1s + (k+1)*(c1s(k+2)*A1s(n-k+1) - 3*A1s(k+2)*c1s(n-k+1));  
    h1u = h1u + (k+1)*(c1u(k+2)*A1u(n-k+1) - 3*A1u(k+2)*c1u(n-k+1));  
  end    
  
  coef_0s = -1/((n+1)*A0s(1));
  coef_0u = -1/((n+1)*A0u(1));
  coef_1s = -1/((n+1)*A1s(1));
  coef_1u = -1/((n+1)*A1u(1));
  
  H0s = L^2*coef_0s*h0s;
  H0u = L^2*coef_0u*h0u;
  H1s = L^2*coef_1s*h1s;
  H1u = L^2*coef_1u*h1u;

  %Solve the homological equations:
  thisSol_0s = M0s\[0; H0s];
  thisSol_0u = M0u\[0; H0u];
  thisSol_1s = M1s\[0; H1s];
  thisSol_1u = M1u\[0; H1u];
  
  %Assign the coefficient values:
  A0s(n+2) = thisSol_0s(1);
  A0u(n+2) = thisSol_0u(1);
  A1s(n+2) = thisSol_1s(1);
  A1u(n+2) = thisSol_1u(1);
  
  B0s(n+2) = thisSol_0s(2);
  B0u(n+2) = thisSol_0u(2);
  B1s(n+2) = thisSol_1s(2);
  B1u(n+2) = thisSol_1u(2);
  
  c0s(n+2) = coef_0s*h0s + 3*(c0s(1)/A0s(1))*A0s(n+2);
  c0u(n+2) = coef_0u*h0u + 3*(c0u(1)/A0u(1))*A0u(n+2);
  c1s(n+2) = coef_1s*h1s + 3*(c1s(1)/A1s(1))*A1s(n+2);
  c1u(n+2) = coef_1s*h1u + 3*(c1u(1)/A1u(1))*A1u(n+2);
end    


%==========================================================================
%Analysis:
%==========================================================================
    
%Plot points:
numberOfPoints = numParmPoints;

p0_stableManifold = zeros(numberOfPoints, 2);
p0_unstableManifold = zeros(numberOfPoints, 2);
p1_stableManifold = zeros(numberOfPoints, 2);
p1_unstableManifold = zeros(numberOfPoints, 2);

stopPar = stopParm;
s0s = linspace(0, stopPar, numberOfPoints);
s0u = linspace(0, stopPar, numberOfPoints);
s1s = linspace(0, stopPar, numberOfPoints);
s1u = linspace(0, stopPar, numberOfPoints);

%compute manifold points ------------------------------------------------
for i = 1:numberOfPoints;
    x0s = horner(s0s(i), A0s, N);
    y0s = horner(s0s(i), B0s, N);
    p0_stableManifold(i, 1) = x0s;
    p0_stableManifold(i, 2) = y0s;
    
    x0u = horner(s0u(i), A0u, N);
    y0u = horner(s0u(i), B0u, N);
    p0_unstableManifold(i, 1) = x0u;
    p0_unstableManifold(i, 2) = y0u;
    
    x1s = horner(s1s(i), A1s, N);
    y1s = horner(s1s(i), B1s, N);
    p1_stableManifold(i, 1) = x1s;
    p1_stableManifold(i, 2) = y1s;
    
    x1u = horner(s1u(i), A1u, N);
    y1u = horner(s1u(i), B1u, N);
    p1_unstableManifold(i, 1) = x1u;
    p1_unstableManifold(i, 2) = y1u;
end    
%----------------------------------------------------------------


 
%Output the phase endpoints of the manifolds:
%p0_s_end = [horner(s0s(numberOfPoints), A0s, N);
%            horner(s0s(numberOfPoints), B0s, N)]

%p0_u_end = [horner(s0u(numberOfPoints), A0u, N);
%            horner(s0u(numberOfPoints), B0u, N)]
       
%p1_s_end = [horner(s1s(numberOfPoints), A1s, N);
%            horner(s1s(numberOfPoints), B1s, N)]
       
%p1_u_end = [horner(s1u(numberOfPoints), A1u, N);
%            horner(s1u(numberOfPoints), B1u, N)]
       

%Compute derivative series:
for n = 0:(N-1)
   DA0s(n+1) = (n+1)*A0s(n+2); 
   DB0s(n+1) = (n+1)*B0s(n+2);
   DA0u(n+1) = (n+1)*A0u(n+2);
   DB0u(n+1) = (n+1)*B0u(n+2);
   
   DA1s(n+1) = (n+1)*A1s(n+2);
   DB1s(n+1) = (n+1)*B1s(n+2);
   DA1u(n+1) = (n+1)*A1u(n+2);
   DB1u(n+1) = (n+1)*B1u(n+2); 
end   

numTestPoints = 10000;

theta = 0;
maxError = 0;
for k = 1:numTestPoints
    h = 2*pi/numTestPoints;
    theta = theta + k*h;
    z = stopParm*exp(j*theta);
    Pz = [horner(z, A0s, N);
          horner(z, B0s, N)];
    fPz = [Pz(2);
           L^2*(Pz(1)^3 - Pz(1))];
    DPz = [horner(z, DA0s, N-1);
           horner(z, DB0s, N-1)];  
    thisError = norm(fPz - lambda_s*z*DPz);
    if (thisError > maxError)
       maxError = thisError; 
    end
end
 
maxError

eps = 1;
while (1+eps) > 1
    eps = eps/2;
end
machineEps = eps*2        

maxErr_multMachEps = maxError/machineEps

%==========================================================================
%Aposteriori Validation:
%==========================================================================

%Compute a bound on the image of the parameterization:
temp = size(p0_stableManifold);
numPoints = temp(1,1);
normPs = 0;
for j = 1:numPoints
  thisPoint = p0_stableManifold(j, :) - [-1 0];
  thisNorm = norm(thisPoint);
  if (thisNorm > normPs)
     normPs = thisNorm;
  end
end

normPs
        
%Lets bound the vector field on a circle of radius 2.
%Then take:

dim = 1
rho = 2
rhoPrime = 1
s = rho - rhoPrime

%Then
C1 = ceil(sqrt(1+4*L^4)*rho^3)
C2 = ceil(sqrt(5*(1+L^4))*rho^2)



eps_tol = maxError

delta_star = 0.1*eps_tol
delta = 0.025*eps_tol

mu = lambda_s + 0.001
denom_1 = ((N+1)*mu + C2)

deltaCond_2 = C2/abs(mu)

deltaCond_2 = (2*eps_tol)/denom_1

%Try
term1 = (abs((N+1)*mu + C2)*s^2)/((2^(dim+2))*(2*C1 + s*C2))
term2 = sqrt((abs((N+1)*mu + C2)*s^3)/((dim^(3/2))*2^(dim+2))*(2*C1 + s*C2))
deltaCond_3 =  min(term1, term2)


delta_inMachEps = (2*deltaCond_2)/machineEps
delta

lipDom = 1.1
LipConst = (L^2)*(3*lipDom^2 + 1)
allowedDecay = 10^(-3)
stopTime = (1/LipConst)*log(allowedDecay/delta)









%Plots -----------------------------------------------------------        
plot(p0_stableManifold(:, 1), p0_stableManifold(:,2), 'r')
plot(p0_unstableManifold(:, 1), p0_unstableManifold(:,2), 'b')
plot(p1_stableManifold(:, 1), p1_stableManifold(:,2), 'r')
plot(p1_unstableManifold(:, 1), p1_unstableManifold(:,2), 'b')

plot(p0_stableManifold(:, 1), p0_stableManifold(:,2), 'r.')
plot(p0_unstableManifold(:, 1), p0_unstableManifold(:,2), 'b.')
plot(p1_stableManifold(:, 1), p1_stableManifold(:,2), 'r.')
plot(p1_unstableManifold(:, 1), p1_unstableManifold(:,2), 'b.')
%-----------------------------------------------------------------

