clear
format long

%startintlab


a = intval('1.6')
b = intval('0.1')
numDim = 50




nu = 10^(-10)


%=========================================================================
%
%=========================================================================


x_p = (b-1 + sqrt((1-b)^2 + 4*a))/(2*a);
x_m = (b-1 - sqrt((1-b)^2 + 4*a))/(2*a);

p_p = intval(zeros(numDim, 1));
p_m = intval(zeros(numDim, 1));

for i = 1:numDim
   p_p(i) = x_p;
   p_m(i) = x_m;
end


fp_test1 = sup(norm(p_p - henonLikeMap_intval(p_p, a, b, numDim)))
fp_test2 = sup(norm(p_m - henonLikeMap_intval(p_m, a, b, numDim)))


Dfp_p = henonLikeDifferential_intval(p_p, a, b, numDim);


[Q_float, Sigma] = eig(full(Dfp_p.mid));

Q = intval(zeros(numDim, numDim));
theEigs = intval(zeros(numDim, numDim-1));
Lambda = intval(zeros(numDim-1, numDim-1));

[lambda_temp, xi_temp] = ...
     verifyeig(Dfp_p, Sigma(1,1), Q_float(:,1));
Q(:,1) = real(xi_temp);
lambda_u = lambda_temp;


inLoop = 1
tic
for i = 2:numDim
    [lambda_temp, xi_temp] = verifyeig(Dfp_p, Sigma(i,i), Q_float(:,i));
    Lambda(i-1,i-1) = lambda_temp;
    theEigs(:,i-1) = xi_temp;
	Q(:,i) = xi_temp;      
end
compTime = toc


Q_inv = inv(Q);


eigsError = max(max(rad(theEigs)))
eigVectError = max(max(rad(Q)))
invQError = max(max(rad(Q_inv)))


A = Q(:, 2:numDim);


figure
hold on
plot(mid(lambda_u), 0, 'ko')

for k = 1:numDim-1
    plot(real(mid(Lambda(k,k))), imag(mid(Lambda(k,k))), 'ko');
end




stopNow = stopHere


%==========================================================================
%Validation Algorithm:
%==========================================================================

%STEP 1: ------------------------------------------------------------------
% Pick the size of the neighborhood to look at
R = intval(1)      %intval('2')/intval('1')      
                   %In the notes this is rho instead of R
                   %I'm using R here as rho is a system 
                   %parameter for Lorenz


%STEP 2: ------------------------------------------------------------------
%Compute a bound on the size of the vector field in an R neighborhood
%of the equilibria:

M = abs(2*a);
N_f = 1


%STEP 3: ------------------------------------------------------------------
%Compute the matrix norm bounds:
a_norm = norm(A, inf)
Q1 = norm(Q, inf)
Q2 = norm(Q_inv, inf)



%STEP 6: ------------------------------------------------------------------
%Choose s and compute the combinatorial constants:
%Do step 6 first so we can see how bad consts are...

s = 10^(-4)
dim = numDim    %Dimension of the manifolds
C_n = linearApproxCombConst_C(s, dim)
K_n = linearApproxCombConst_K(s, dim)


%STEP 4: ------------------------------------------------------------------

%Find the resonance bound N_0: How many conditions to check by hand in order
%to rule out resonances?

mu_subStar = intval(1);
mu_supStar = intval(0);


for k = 1:(numDim-1)
    this_up = sup(abs(Lambda(k,k)));
    this_down = inf(abs(Lambda(k,k)));
    if this_up > sup(mu_supStar)
       mu_supStar = abs(Lambda(k,k)); 
    end
    if inf(mu_subStar) > this_down
       mu_subStar = abs(Lambda(k,k));
    end
end



mu_subStar
mu_supStar



checkNum = inf(mu_subStar) - sup(mu_supStar); 
N_0 = 1;
while checkNum <= 0
    N_0 = N_0 + 1;
   checkNum = inf(mu_subStar) - sup(mu_supStar)^(N_0);
end

%Answer:
N_0


%Now find the bound on the nearest resonance in order to bound the 
%linear operator L.

maxRes = 1/abs(mu_subStar - mu_supStar^N_0)


loopNum = numDim - 1
max_res = intval(0)
counter = 1



for j = 1:loopNum
   for k = 1:loopNum
      this_c = ...
          1/abs(Lambda(k,k)^2 - Lambda(j,j));
      %resBounds(counter) = this_c;
      counter = counter + 1;
      if inf(this_c) > sup(max_res)
        max_res = this_c; 
       end
   end
end



for j = 1:loopNum-1
    lambda_j = Lambda(j,j);
    j
    for m_1 = 1:(loopNum - 1)
        for m_2 = (m_1+1):(loopNum)
            m_2;
            %indices(counter, 1:2) = [m_1, m_2];
            this_c = ...
                1/abs(Lambda(m_1, m_1)*Lambda(m_2, m_2) - lambda_j);
            %resBounds(counter) = this_c;
            counter = counter+1;
             
            if inf(this_c) > sup(max_res)
               max_res = this_c; 
            end
        end
    end
end

maxRes



%STEP 5: ------------------------------------------------------------------
%    Define the bound on the linear operator:
C_tilde = Q1*Q2*maxRes


%Step 7: ------------------------------------------------------------------
%Fix the tolerance and compute a guess for nu:

%epsilon_tol =  10^(-)
%epsilon_tol = 2.5*10^(-16)
%nu = sqrt(epsilon_tol)
%nu = 10^(-8)

%Step 8: ------------------------------------------------------------------
%find delta_min and check non-degenerate:

B = C_tilde*N_f*M*C_n


if inf((2*a_norm*B*nu - 1)^2 - 4*B^2*a_norm^2*nu^2) <= 0
    discriminant = inf((2*a_norm*B*nu - 1)^2 - 4*B^2*a_norm^2*nu^2)
    failCondition = negativeDiscriminant 
end



if sup(2*a_norm*B*nu) >= 1
   failCondition = b_not_negative 
end

%Use the quadratic formula to find the min and max delta:
delta_min  = ((1 - 2*a_norm*B*nu) - ...
    sqrt((1 - 2*a_norm*B*nu)^2 - 4*B*a_norm^2*B*nu^2))/(2*B) 




%STEP 9: ------------------------------------------------------------------
%Check that s was small enough


step_9_number = (a_norm*nu + delta_min)/R
if sup((a_norm*nu + delta_min)/R) >= s
   failCondition = s_not_small_enough 
end

%STEP 10: -----------------------------------------------------------------

step_10_number = C_tilde*N_f*M*K_n*(a_norm*nu + delta_min)

if sup(step_10_number) >= 1
   failCondition = lost_Z_bound 
end


%STEP 11: -----------------------------------------------------------------
%Return the values: if we made it here then:

dimensionIs = numDim

%validated domain radius:
nu_is = sup(nu) 

s_is = s
%validated error bound:
delta_is = sup(delta_min)

C_tilde


C_n


K_n



C_T = delta_is/nu_is^2







