  
clear
format long 


%intvalinit('FastIVmult')
intvalinit('SharpIVmult')


%=================================================================
%   PROGRAM COMPUTES BOTH ONE AND TWO DIMENSIONAL STABLE AND 
%   UNSTABLE MANIFOLDS AT BOTH FIXED POINTS, BY PARAMETERIZATION
%
%   COMPUTE HOMOCLINIC ORBIT:
%
%
%
%
%================================================================



%==========================================================================
%Non-Rigorous Computation of Approximate Solution:
%==========================================================================



%==============================================================
%Computation Controls:
%==============================================================



%Map Parameter Set: Dullin to Lomeli Meiss.
   %Dullin Meiss Parms:
      a_dm = intval(1)
      b_dm = intval('0.5')
      c_dm = intval('0.5')
            
     
      
      %The Maciej Data was with 0.25
      eps_dm = intval('0.25')    %boundary near 6.18200
      mu_dm = intval('-2.4')

      
   %Convert to Lomeli:   
      a = c_dm
      b = b_dm - 2*c_dm
      c = c_dm + a_dm - b_dm
      
      tau =    2*a_dm*(3+mu_dm)/(2*a_dm - b_dm)
      alpha =  ((9+6*mu_dm+mu_dm^2)*a_dm - 4*eps_dm*a_dm^2 + ...
               4*eps_dm*a_dm*b_dm - eps_dm*b_dm^2)/(2*a_dm - b_dm)^2




%Order of manifold computation:
  %Order of 2D manifolds:
  N = 35;
  
   
%==========================================================================
%Generate zeroth and first order problem data:
%==========================================================================

 
  %LOAD GUESSES:
  
  %Proof 1:
      load maciejData_N_Ku_Ks.txt
      load maciejData_r0u_r1s.txt
      load maciejData_uParData.txt
      load maciejData_sParData.txt
      
      Ku = maciejData_N_Ku_Ks(2)
      Ks = maciejData_N_Ku_Ks(3)
      %r0u = maciejData_r0u_r1s(1)
      %r1s = maciejData_r0u_r1s(2)
      
      r0u = 0.225
      r1s = 0.125
      
      sParSpace = maciejData_sParData(:, 1:2)
      uParSpace = maciejData_uParData(:, 1:2)
      
        
    
 figure 
 hold on  
 
 ang=0:0.01:2*pi; 
 xp=r0u*cos(ang);
 yp=r0u*sin(ang);
 plot(xp,yp);
 
 plot(uParSpace(:,1), uParSpace(:,2), 'b.')   
 plot(uParSpace(1,1), uParSpace(1,2), 'ko')

 axis([-0.5 0.5 -0.5 0.5])
  
   
 figure 
 hold on  
 
 ang=0:0.01:2*pi; 
 xp=r1s*cos(ang);
 yp=r1s*sin(ang);
 plot(xp,yp);
 
 plot(sParSpace(:,1), sParSpace(:,2), 'r.')   
 plot(sParSpace(1,1), sParSpace(1,2), 'ko')
 
 axis([-0.5 0.5 -0.5 0.5])
      
           
[p0, p1, lambda0_s, lambda0_u1, lambda0_u2, ...
         lambda1_u, lambda1_s1, lambda1_s2, ...
         xi0_s, xi0_u1, xi0_u2, ...
         xi1_u, xi1_s1, xi1_s2] ...
       = firstOrderData_intval_3DHenon(a, b, c, alpha, tau);
     
 
           
%==========================================================================
%Two-D Manifolds:
%==========================================================================


 scale_0u_intval = intval('2');
 scale_1s_intval = intval('2');
 
 tic
 
 [A0u, B0u, C0u] = twoDimManifold_intval_3DHenon(a, b, c, alpha, tau, ...
           N, p0, lambda0_u1, lambda0_u2, xi0_u1, xi0_u2, ...
           scale_0u_intval, scale_0u_intval);
   
 time_1 = toc/60


 

 tic
 
 [A1s, B1s, C1s] = twoDimManifold_intval_3DHenon(a, b, c, alpha, tau, ...
        N, p1, lambda1_s1, lambda1_s2, xi1_s1, xi1_s2, ...
        scale_1s_intval, scale_1s_intval);
   
 time_1 = toc/60
 
 
%----------------------Put in Polynomial Data Structure--------------------



[P0u1, P0u2, P0u3] = convertToPoly_2D(A0u, B0u, C0u, N);
[P1s1, P1s2, P1s3] = convertToPoly_2D(A1s, B1s, C1s, N);                                                 

N_f = 4
N_dim = 3


[delta_0u, delta_1s, delta_max_0u, delta_max_1s, ...
 P0u_norm, P1s_norm, u0_delta_good, s1_delta_good, ...
 orderTest] = validateManifolds_2D_3DHenon(...
    P0u1, P0u2, P0u3, P1s1, P1s2, P1s3, ...
    p0, p1, xi0_u1, xi0_u2, xi0_s, xi1_u, xi1_s1, xi1_s2, ...
    lambda0_u1, lambda0_u2, lambda1_s1, lambda1_s2, N_f, N_dim, ...
    r0u, r1s, N, a, b, c, alpha, tau);


numCoef = nchoosek(N+2, 2)


supCoef_realP0u1 = zeros(numCoef, 1);
infCoef_realP0u1 = zeros(numCoef, 1);
supCoef_imagP0u1 = zeros(numCoef, 1);
infCoef_imagP0u1 = zeros(numCoef, 1);

supCoef_realP0u2 = zeros(numCoef, 1);
infCoef_realP0u2 = zeros(numCoef, 1);
supCoef_imagP0u2 = zeros(numCoef, 1);
infCoef_imagP0u2 = zeros(numCoef, 1);

supCoef_realP0u3 = zeros(numCoef, 1);
infCoef_realP0u3 = zeros(numCoef, 1);
supCoef_imagP0u3 = zeros(numCoef, 1);
infCoef_imagP0u3 = zeros(numCoef, 1);

supCoef_realP1s1 = zeros(numCoef, 1);
infCoef_realP1s1 = zeros(numCoef, 1);
supCoef_imagP1s1 = zeros(numCoef, 1);
infCoef_imagP1s1 = zeros(numCoef, 1);

supCoef_realP1s2 = zeros(numCoef, 1);
infCoef_realP1s2 = zeros(numCoef, 1);
supCoef_imagP1s2 = zeros(numCoef, 1);
infCoef_imagP1s2 = zeros(numCoef, 1);

supCoef_realP1s3 = zeros(numCoef, 1);
infCoef_realP1s3 = zeros(numCoef, 1);
supCoef_imagP1s3 = zeros(numCoef, 1);
infCoef_imagP1s3 = zeros(numCoef, 1);

coefNum = 0
for n = 0:N
   for k = 0:n
       supCoef_realP0u1(coefNum + 1, 1) = sup(real(P0u1(n-k, k)));
       infCoef_realP0u1(coefNum + 1, 1) = inf(real(P0u1(n-k, k)));
       supCoef_imagP0u1(coefNum + 1, 1) = sup(imag(P0u1(n-k, k)));
       infCoef_imagP0u1(coefNum + 1, 1) = inf(imag(P0u1(n-k, k)));
       
       supCoef_realP0u2(coefNum + 1, 1) = sup(real(P0u2(n-k, k)));
       infCoef_realP0u2(coefNum + 1, 1) = inf(real(P0u2(n-k, k)));
       supCoef_imagP0u2(coefNum + 1, 1) = sup(imag(P0u2(n-k, k)));
       infCoef_imagP0u2(coefNum + 1, 1) = inf(imag(P0u2(n-k, k)));
       
       supCoef_realP0u3(coefNum + 1, 1) = sup(real(P0u3(n-k, k)));
       infCoef_realP0u3(coefNum + 1, 1) = inf(real(P0u3(n-k, k)));
       supCoef_imagP0u3(coefNum + 1, 1) = sup(imag(P0u3(n-k, k)));
       infCoef_imagP0u3(coefNum + 1, 1) = inf(imag(P0u3(n-k, k)));
       
       supCoef_realP1u1(coefNum + 1, 1) = sup(real(P1s1(n-k, k)));
       infCoef_realP1u1(coefNum + 1, 1) = inf(real(P1s1(n-k, k)));
       supCoef_imagP1u1(coefNum + 1, 1) = sup(imag(P1s1(n-k, k)));
       infCoef_imagP1u1(coefNum + 1, 1) = inf(imag(P1s1(n-k, k)));
       
       supCoef_realP1s2(coefNum + 1, 1) = sup(real(P1s2(n-k, k)));
       infCoef_realP1s2(coefNum + 1, 1) = inf(real(P1s2(n-k, k)));
       supCoef_imagP1s2(coefNum + 1, 1) = sup(imag(P1s2(n-k, k)));
       infCoef_imagP1s2(coefNum + 1, 1) = inf(imag(P1s2(n-k, k)));
       
       supCoef_realP1s3(coefNum + 1, 1) = sup(real(P1s3(n-k, k)));
       infCoef_realP1s3(coefNum + 1, 1) = inf(real(P1s3(n-k, k)));
       supCoef_imagP1s3(coefNum + 1, 1) = sup(imag(P1s3(n-k, k)));
       infCoef_imagP1s3(coefNum + 1, 1) = inf(imag(P1s3(n-k, k)));
       
       coefNum = coefNum + 1;
   end
end
  
delta_0u = sup(delta_0u)
delta_1s = sup(delta_1s)

save maciejData_delta_u_delta_s.txt delta_0u delta_1s -double -ascii
save maciejData_r0u_r1s.txt r0u r1s -double -ascii

save maciejData_supCoef_realP0u1.txt supCoef_realP0u1 -double -ascii
save maciejData_infCoef_realP0u1.txt infCoef_realP0u1 -double -ascii
save maciejData_supCoef_imagP0u1.txt supCoef_imagP0u1 -double -ascii
save maciejData_infCoef_imagP0u1.txt infCoef_imagP0u1 -double -ascii



save maciejData_supCoef_realP0u2.txt supCoef_realP0u2 -double -ascii
save maciejData_infCoef_realP0u2.txt infCoef_realP0u2 -double -ascii
save maciejData_supCoef_imagP0u2.txt supCoef_imagP0u2 -double -ascii
save maciejData_infCoef_imagP0u2.txt infCoef_imagP0u2 -double -ascii




save maciejData_supCoef_realP0u3.txt supCoef_realP0u3 -double -ascii
save maciejData_infCoef_realP0u3.txt infCoef_realP0u3 -double -ascii
save maciejData_supCoef_imagP0u3.txt supCoef_imagP0u3 -double -ascii
save maciejData_infCoef_imagP0u3.txt infCoef_imagP0u3 -double -ascii




save maciejData_supCoef_realP1s1.txt supCoef_realP1s1 -double -ascii
save maciejData_infCoef_realP1s1.txt infCoef_realP1s1 -double -ascii
save maciejData_supCoef_imagP1s1.txt supCoef_imagP1s1 -double -ascii
save maciejData_infCoef_imagP1s1.txt infCoef_imagP1s1 -double -ascii


save maciejData_supCoef_realP1s2.txt supCoef_realP1s2 -double -ascii
save maciejData_infCoef_realP1s2.txt infCoef_realP1s2 -double -ascii
save maciejData_supCoef_imagP1s2.txt supCoef_imagP1s2 -double -ascii
save maciejData_infCoef_imagP1s2.txt infCoef_imagP1s2 -double -ascii


save maciejData_supCoef_realP1s3.txt supCoef_realP1s3 -double -ascii
save maciejData_infCoef_realP1s3.txt infCoef_realP1s3 -double -ascii
save maciejData_supCoef_imagP1s3.txt supCoef_imagP1s3 -double -ascii
save maciejData_infCoef_imagP1s3.txt infCoef_imagP1s3 -double -ascii








