  
clear
format long 


%intvalinit('FastIVmult')
intvalinit('SharpIVmult')


%=================================================================
%   PROGRAM COMPUTES BOTH ONE AND TWO DIMENSIONAL STABLE AND 
%   UNSTABLE MANIFOLDS AT BOTH FIXED POINTS, BY PARAMETERIZATION
%
%   COMPUTE HOMOCLINIC ORBIT:
%
%
%
%
%================================================================



%==========================================================================
%Non-Rigorous Computation of Approximate Solution:
%==========================================================================



%==============================================================
%Computation Controls:
%==============================================================



%Map Parameter Set: Dullin to Lomeli Meiss.
   %Dullin Meiss Parms:
      a_dm = intval(1)
      b_dm = intval('0.5')
      c_dm = intval('0.5')
            
     
      
      %The Maciej Data was with 0.25
      eps_dm = intval('0.25')    %boundary near 6.18200
      mu_dm = intval('-2.4')

      
   %Convert to Lomeli:   
      a = c_dm
      b = b_dm - 2*c_dm
      c = c_dm + a_dm - b_dm
      
      tau =    2*a_dm*(3+mu_dm)/(2*a_dm - b_dm)
      alpha =  ((9+6*mu_dm+mu_dm^2)*a_dm - 4*eps_dm*a_dm^2 + ...
               4*eps_dm*a_dm*b_dm - eps_dm*b_dm^2)/(2*a_dm - b_dm)^2




%Order of manifold computation:
  %Order of 2D manifolds:
  N = 35;
  
   
%==========================================================================
%Generate zeroth and first order problem data:
%==========================================================================

 
  %LOAD GUESSES:
  
  %Proof 1:
      load maciejData_N_Ku_Ks.txt
      load maciejData_r0u_r1s.txt
      load maciejData_uParData.txt
      load maciejData_sParData.txt
      
      Ku = maciejData_N_Ku_Ks(2)
      Ks = maciejData_N_Ku_Ks(3)
      %r0u = maciejData_r0u_r1s(1)
      %r1s = maciejData_r0u_r1s(2)
      
      r0u = 0.225
      r1s = 0.125
      
      sParSpace = maciejData_sParData(:, 1:2)
      uParSpace = maciejData_uParData(:, 1:2)
      
        
    
 figure 
 hold on  
 
 ang=0:0.01:2*pi; 
 xp=r0u*cos(ang);
 yp=r0u*sin(ang);
 plot(xp,yp);
 
 plot(uParSpace(:,1), uParSpace(:,2), 'b.')   
 plot(uParSpace(1,1), uParSpace(1,2), 'ko')

 axis([-0.5 0.5 -0.5 0.5])
  
   
 figure 
 hold on  
 
 ang=0:0.01:2*pi; 
 xp=r1s*cos(ang);
 yp=r1s*sin(ang);
 plot(xp,yp);
 
 plot(sParSpace(:,1), sParSpace(:,2), 'r.')   
 plot(sParSpace(1,1), sParSpace(1,2), 'ko')
 
 axis([-0.5 0.5 -0.5 0.5])
      
           
[p0, p1, lambda0_s, lambda0_u1, lambda0_u2, ...
         lambda1_u, lambda1_s1, lambda1_s2, ...
         xi0_s, xi0_u1, xi0_u2, ...
         xi1_u, xi1_s1, xi1_s2] ...
       = firstOrderData_intval_3DHenon(a, b, c, alpha, tau);
     
 
           
%==========================================================================
%Two-D Manifolds:
%==========================================================================


 scale_0u_intval = intval('2');
 scale_1s_intval = intval('2');
 
 tic
 
 [A0u, B0u, C0u] = twoDimManifold_intval_3DHenon(a, b, c, alpha, tau, ...
           N, p0, lambda0_u1, lambda0_u2, xi0_u1, xi0_u2, ...
           scale_0u_intval, scale_0u_intval);
   
 time_1 = toc/60


 

 tic
 
 [A1s, B1s, C1s] = twoDimManifold_intval_3DHenon(a, b, c, alpha, tau, ...
        N, p1, lambda1_s1, lambda1_s2, xi1_s1, xi1_s2, ...
        scale_1s_intval, scale_1s_intval);
   
 time_1 = toc/60
 
 
%----------------------Put in Polynomial Data Structure--------------------



[P0u1, P0u2, P0u3] = convertToPoly_2D(A0u, B0u, C0u, N);
[P1s1, P1s2, P1s3] = convertToPoly_2D(A1s, B1s, C1s, N);                                                 

N_f = 4
N_dim = 3


[delta_0u, delta_1s, delta_max_0u, delta_max_1s, ...
 P0u_norm, P1s_norm, u0_zdelta_good, s1_delta_good, ...
 orderTest] = validateManifolds_2D_3DHenon(...
    P0u1, P0u2, P0u3, P1s1, P1s2, P1s3, ...
    p0, p1, xi0_u1, xi0_u2, xi0_s, xi1_u, xi1_s1, xi1_s2, ...
    lambda0_u1, lambda0_u2, lambda1_s1, lambda1_s2, N_f, N_dim, ...
    r0u, r1s, N, a, b, c, alpha, tau);



z_poly = polynom(intval([1, i]), [1 0; 0 1], {'x' 'y'})
zbar_poly = polynom(intval([1, -i]), [1 0; 0 1], {'x' 'y'})


Pbar_u1 = polynom(intval([0]), [0 0], {'x' 'y'});
Pbar_u2 = polynom(intval([0]), [0 0], {'x' 'y'});
Pbar_u3 = polynom(intval([0]), [0 0], {'x' 'y'}); 

Pbar_s1 = polynom(intval([0]), [0 0], {'x' 'y'});
Pbar_s2 = polynom(intval([0]), [0 0], {'x' 'y'});
Pbar_s3 = polynom(intval([0]), [0 0], {'x' 'y'});

for n = 0:N
    n
    N
    
    newPoly1 = polynom(intval([0]), [0 0], {'x' 'y'});
    newPoly2 = polynom(intval([0]), [0 0], {'x' 'y'});
    newPoly3 = polynom(intval([0]), [0 0], {'x' 'y'});
    newPoly4 = polynom(intval([0]), [0 0], {'x' 'y'});
    newPoly5 = polynom(intval([0]), [0 0], {'x' 'y'});
    newPoly6 = polynom(intval([0]), [0 0], {'x' 'y'});
    for k = 0:n
         z_n_k = z_poly^(n-k);
         zbar_k = zbar_poly^k;
         zzbar_pow = z_n_k*zbar_k; 
         
         newPoly1 = newPoly1 + P0u1(n-k, k)*zzbar_pow;
         newPoly2 = newPoly2 + P0u2(n-k, k)*zzbar_pow;
         newPoly3 = newPoly3 + P0u3(n-k, k)*zzbar_pow;
         newPoly4 = newPoly4 + P1s1(n-k, k)*zzbar_pow;
         newPoly5 = newPoly5 + P1s2(n-k, k)*zzbar_pow;
         newPoly6 = newPoly6 + P1s3(n-k, k)*zzbar_pow;
    end
         Pbar_u1 = Pbar_u1 + real(newPoly1); 
         Pbar_u2 = Pbar_u2 + real(newPoly2); 
         Pbar_u3 = Pbar_u3 + real(newPoly3); 
         
         Pbar_s1 = Pbar_s1 + real(newPoly4); 
         Pbar_s2 = Pbar_s2 + real(newPoly5); 
         Pbar_s3 = Pbar_s3 + real(newPoly6); 
end




numCoef = nchoosek(N+2, 2)


supCoef_P0u1 = zeros(numCoef, 1);
infCoef_P0u1 = zeros(numCoef, 1);

supCoef_P0u2 = zeros(numCoef, 1);
infCoef_P0u2 = zeros(numCoef, 1);

supCoef_P0u3 = zeros(numCoef, 1);
infCoef_P0u3 = zeros(numCoef, 1);

supCoef_P1s1 = zeros(numCoef, 1);
infCoef_P1s1 = zeros(numCoef, 1);

supCoef_P1s2 = zeros(numCoef, 1);
infCoef_P1s2 = zeros(numCoef, 1);

supCoef_P1s3 = zeros(numCoef, 1);
infCoef_P1s3 = zeros(numCoef, 1);

coefNum = 0
for n = 0:N
   for k = 0:n
       supCoef_P0u1(coefNum + 1, 1) = sup(Pbar_u1(n-k, k));
       infCoef_P0u1(coefNum + 1, 1) = inf(Pbar_u1(n-k, k));
       
       supCoef_P0u2(coefNum + 1, 1) = sup(Pbar_u2(n-k, k));
       infCoef_P0u2(coefNum + 1, 1) = inf(Pbar_u2(n-k, k));
       
       supCoef_P0u3(coefNum + 1, 1) = sup(Pbar_u3(n-k, k));
       infCoef_P0u3(coefNum + 1, 1) = inf(Pbar_u3(n-k, k));
       
       supCoef_P1s1(coefNum + 1, 1) = sup(Pbar_s1(n-k, k));
       infCoef_P1s1(coefNum + 1, 1) = inf(Pbar_s1(n-k, k));
       
       supCoef_P1s2(coefNum + 1, 1) = sup(Pbar_s2(n-k, k));
       infCoef_P1s2(coefNum + 1, 1) = inf(Pbar_s2(n-k, k));
       
       supCoef_P1s3(coefNum + 1, 1) = sup(Pbar_s3(n-k, k));
       infCoef_P1s3(coefNum + 1, 1) = inf(Pbar_s3(n-k, k));
       
       coefNum = coefNum + 1;
   end
end
  
delta_0u = sup(delta_0u)
delta_1s = sup(delta_1s)


theta_1 = 0.1
theta_2 = 0.2
polyval(P1s3, theta_1 + i*theta_2, theta_1-i*theta_2)
polyval(Pbar_s3, theta_1, theta_2)


save maciejData_supCoef_P0u1.txt supCoef_P0u1 -double -ascii
save maciejData_infCoef_P0u1.txt infCoef_P0u1 -double -ascii
save maciejData_supCoef_P0u2.txt supCoef_P0u2 -double -ascii
save maciejData_infCoef_P0u2.txt infCoef_P0u2 -double -ascii
save maciejData_supCoef_P0u3.txt supCoef_P0u3 -double -ascii
save maciejData_infCoef_P0u3.txt infCoef_P0u3 -double -ascii

save maciejData_supCoef_P1s1.txt supCoef_P1s1 -double -ascii
save maciejData_infCoef_P1s1.txt infCoef_P1s1 -double -ascii
save maciejData_supCoef_P1s2.txt supCoef_P1s2 -double -ascii
save maciejData_infCoef_P1s2.txt infCoef_P1s2 -double -ascii
save maciejData_supCoef_P1s3.txt supCoef_P1s3 -double -ascii
save maciejData_infCoef_P1s3.txt infCoef_P1s3 -double -ascii







