clear 
format long

%=====================================================================
%This code produces the results of the computatoins discussed in 
%section 4.4.  Set order_1 equal to orderComp1 for the second order 
%computation.  Use order_1 equal to orderComp2 for the order 25 
%computation.  Similarily choose domRad0s equal to domRadComp1 for
%the 2nd order computation or domRadComp2 for the 25th order comp.
%=====================================================================



%=====================================================================

%Order of 1D manifold computations
  orderComp1 = 2      %Use this value for the first computation
                      %discussed in section 4.4
  orderComp2 = 25     %Use this value for the second computation.

order_1 = orderComp1;  %Choose which computation order to
                       %work with.



%Phase Space Bounds: 1D
  domRadComp1 = 5*10^(-6)         %Use this domRad for first computation
  domRadComp2 = 0.8               %Use this domRad for second comp.

  domRad0s = domRadComp1     %Pick the value of domRad.
  domRad0s = intval(domRad0s)
%=====================================================================


%Map Parameter Set: Dullin to Lomeli Meiss.
   %Dullin Meiss Parms:
      a_dm = 1
      b_dm = 0.5
      c_dm = 0.5
            
      
      %eps_dm = 2.11    %boundary near 6.182001
      eps_dm = 0.32      %worked at 0.4
      mu_dm = -2.4
      
      
   %Convert to Lomeli:   
      a = c_dm
      b = b_dm - 2*c_dm
      c = c_dm + a_dm - b_dm
      
      tau =    2*a_dm*(3+mu_dm)/(2*a_dm - b_dm)
      alpha =  ((9+6*mu_dm+mu_dm^2)*a_dm - 4*eps_dm*a_dm^2 + ...
               4*eps_dm*a_dm*b_dm - eps_dm*b_dm^2)/(2*a_dm - b_dm)^2

     
      
   %Convert to Lomeli:   
      a = intval(a)      %c_dm
      b = intval(b)      %b_dm - 2*c_dm
      c = intval(c)      %c_dm + a_dm - b_dm
      
      tau =  intval(tau)  %2*a_dm*(3+mu_dm)/(2*a_dm - b_dm)
      alpha =  intval(alpha) %((9+6*mu_dm+mu_dm^2)*a_dm - 4*eps_dm*a_dm^2 + ...
                           %4*eps_dm*a_dm*b_dm - eps_dm*b_dm^2)/(2*a_dm - b_dm)^2

     
                                     
[p0, p1, lambda0_s, lambda0_u1, lambda0_u2, ...
         lambda1_u, lambda1_s1, lambda1_s2, ...
         xi0_s, xi0_u1, xi0_u2, ...
         xi1_u, xi1_s1, xi1_s2] ...
       = firstOrderData_intval_3DHenon(a, b, c, alpha, tau);
        
%=====================================================================
%One-D Manifolds 
%=====================================================================

%Step 1: Compute 1D Coefficients:
scale = intval('1');

[A0s, B0s, C0s] = oneDimManifold_intval_3DHenon(a, b, c, alpha, tau, ...
     order_1, p0, lambda0_s, xi0_s, scale);        
           

firstOrder = 1

%==========================================================================
%1D Validation
%==========================================================================

[P0s1, P0s2, P0s3] = convertToPoly_1D(A0s, B0s, C0s, order_1);                                                 

                                                                                            
N_f = 4
N_dim = 3

C0 = inv_henon3D_differential_intval(p0, alpha, tau, a, b, c);

norm_C0 = norm(C0, inf)

sum1 = norm_C0;

for k = 1:order_1
    term1 = -2*a*A0s(k+1) - b*B0s(k+1);
    term2 = -b*A0s(k+1) - 2*c*B0s(k+1);
    thisMatrix = [0,     0,     0; ...
                  0,     0,     0; ...
                  0, term1, term2];
    sum1 = sum1 + norm(thisMatrix, inf)*domRad0s^k; 
end

barK = sum1

%A Posteriori Errors: 1D                                               
                                                                                              
epsilon_s1D = sError_intval_1D(P0s1, P0s2, P0s3, alpha, tau, ...
                               a, b, c, lambda0_s, order_1, domRad0s)

                          
%Bounds.
max1 = polyval(mag(P0s1-p0(1)), domRad0s);
max2 = polyval(mag(P0s2-p0(2)), domRad0s);
max3 = polyval(mag(P0s3-p0(3)), domRad0s);
P0s_norm = norm([max1, max2, max3], inf)

                           
rho_prime = P0s_norm
rho= 1.1*rho_prime
                           
K_1= norm([2*abs(a), 2*abs(c), abs(b), 1], inf)

orderTest = 0;
OrderTest_0s = -log(barK)/log(lambda0_s)


if ((order_1 +1) >= OrderTest_0s)
    orderTest = 1
end



denom_0s = 2*N_dim*intval('pi')*N_f*barK*K_1
test1 = ...
    (1 - barK*(abs(lambda0_s))^(order_1 + 1))/denom_0s

test2 = ...
    ((2*barK)/(1 - barK*(abs(lambda0_s))^(order_1+1)))*epsilon_s1D

delta = (intval(exp(1))^(-1))*min(inf(test1), inf(rho-rho_prime));


if ((sup(test2) < inf(delta)) && (inf(test2) > 0))
  delta_good = 1
end

      
min_delta = test2
max_delta = delta







