
%=================================================================
%   Run after running aaa_allManifolds.
%   
%  %1D Stable Manifold Computations
%
%================================================================





%==============================================================
%Some Stable Manifold Computations:
%==============================================================



theta = linspace(0.0, 0.01, 10000);
Wu0 = zeros(num_1d_points, 3);

 for t = 1:num_1d_points
     this_x = 0;
     this_y = 0;
     this_z = 0;
     this_theta = theta(t);
     for n = 0:order_1
         this_x = this_x + A0u(n+1)*this_theta^n;
         this_y = this_y + B0u(n+1)*this_theta^n;
         this_z = this_z + C0u(n+1)*this_theta^n;
     end
     Wu0(t, 1) = this_x;
     Wu0(t, 2) = this_y;
     Wu0(t, 3) = this_z;
 end    


%--------------------------------------------------------------
%1D-fundamental manifold
%--------------------------------------------------------------


theta = linspace(lambda_1s*0.5, 0.5, num_1d_points);
Ws1_fd = zeros(num_1d_points, 3);

 for t = 1:num_1d_points
     this_x = 0;
     this_y = 0;
     this_z = 0;
     this_theta = theta(t);
     for n = 0:order_1
         this_x = this_x + A1s(n+1)*this_theta^n;
         this_y = this_y + B1s(n+1)*this_theta^n;
         this_z = this_z + C1s(n+1)*this_theta^n;
     end
     Ws1_fd(t, 1) = this_x;
     Ws1_fd(t, 2) = this_y;
     Ws1_fd(t, 3) = this_z;
 end    

Ws1_global = inverse_iterate_set_k(Ws1_fd, 110, alpha, tau, a, b, c);

figure
hold on

plot3(Wu0(:,1), Wu0(:,2), Wu0(:,3), 'b.')
plot3(Ws1_global(:,1), Ws1_global(:,2), Ws1_global(:,3), 'r.')
plot3(x0, x0, x0, 'k*')
plot3(x1, x1, x1, 'k*')

leftend = 0.67248827264963 - 0.001;
rightend = 0.67248827264963 + 0.001;

%axis([leftend rightend leftend rightend leftend rightend])
axis([-2 2 -2 2 -2 2]);


%--------------------------------------------------------------
%Error:
%--------------------------------------------------------------
for t = 1:num_1d_points
     this_x = 0;
     this_y = 0;
     this_z = 0;
     lambda_x = 0;
     lambda_y = 0;
     lambda_z = 0;
     this_theta = theta(t);
     lambda_theta = lambda_1s*this_theta;
     for n = 0:order_1
         this_x = this_x + A1s(n+1)*this_theta^n;
         this_y = this_y + B1s(n+1)*this_theta^n;
         this_z = this_z + C1s(n+1)*this_theta^n;
     end
     for n = 0:order_1
         lambda_x = lambda_x + A1s(n+1)*lambda_theta^n;
         lambda_y = lambda_y + B1s(n+1)*lambda_theta^n;
         lambda_z = lambda_z + C1s(n+1)*lambda_theta^n;
     end
     q = [this_x, this_y, this_z];
     fP = henon3D(q, alpha, tau, a, b, c);
     P_lambda = [lambda_x, lambda_y, lambda_z];
     error(t) = norm(fP - P_lambda');
end       
 
Error = max(error)

%----------------------------------------------------------------
%Lyapunov Exponent Near P_s(0.5):
%----------------------------------------------------------------

this_x = 0;
this_y = 0;
this_z = 0;

for n = 0:order_1
this_x = this_x + A1s(n+1)*(0.5)^n;
this_y = this_y + B1s(n+1)*(0.5)^n;
this_z = this_z + C1s(n+1)*(0.5)^n;
end

Ws_point = [this_x this_y this_z];

neighbors = zeros(1:1000, 1:3);
theNorms = zeros(1:1000, 1);
testIterate = 110;
numSamples = 50000

for k = 1:10000
   thisRandO = (5*10^(-15))*rand(1, 3);  
   randNeighbor = Ws_point + thisRandO;
   neighbors(k, 1:3) = inverse_iterate_point_k(randNeighbor, testIterate, alpha, tau, a, b, c);    
end    

pointIterate = inverse_iterate_point_k(Ws_point, testIterate, alpha, tau, a, b, c)';

theMax = 0;

for k = 1:10000
    theNorms(k) = norm(neighbors(k, 1:3) - pointIterate);
    if (theNorms(k) > theMax)
       theMax = theNorms(k); 
    end    
end    

theMax

%------------------------------------------------------------------
%||P||
%------------------------------------------------------------------

for t = 1:num_1d_points
  normP(t) = norm(Ws1(t,:) - p1');
end       

barsP = max(normP)
rho = barsP + 10^(-15)

%------------------------------------------------------------------
%||A^{-1}||
%------------------------------------------------------------------

Amatrix = Df1;
Amatrix_inv = inv(Df1);
[U, Sigma, V] = svd(Amatrix_inv);
normAinv = Sigma(1,1)
Order = order_1

rho_to_N = rho^Order
lambda_to_N = abs(lambda_1s)^Order



%-------------------------------------------------------------------
%view plane
%-------------------------------------------------------------------

d = (p1 - p0)'
point = p0 + 0.5*d'
[U, Sigma, V] = svd(d)

T1 = V(:, 2)
T2 = V(:, 3)

normal = cross(T1, T2)
check1 = d*T1
check2 = d*T2

r = 4.0
theta = -0.5*pi
planePoint = point + r*sin(theta)*T1 + r*cos(theta)*T2 - 0.0*d'





















