
clear
format long 

%=================================================================
%   COMPUTE HETEROCLINIC ARC BY NEWTON METHOD
%   
%
%   Program caries out all three steps of computation discussed in   
%   the paper.  Plots results.
%   
%
%================================================================

%==============================================================
%Computation Controls:
%==============================================================

%Map Parameters:
a = 0.44;
b = 0.21;
c = 0.35;

alpha = -0.25; 
tau = -0.3;   

  
%Order of manifold computation:
N = 65;

%Number of times to globalize best disks:
k = 9
 
%number of times to globalize heteroclinic connections:
K_hetero = 35

%Domain parameters:
  R = 0.7
  allowed_error = 5*10^(-15)      %10^-14 definitely works.
  num_theta = 100
  num_r = 40
  
%path parameters
  pathStep = 2.5*10^(-3);
  epsilon_path = 10^(-14);
  maxTry = 10000;
  
%Set Parameters:
  intersectionTol = 2.5*10^(-2) 

  %Arc Taylor Order
  K = 50;
  
%==============================================================
%Compute Best Domain:
%==============================================================


[Au, Bu, Cu, As, Bs, Cs, Du, Ds, Wu_loc, Ws_loc, x0, x1, p0, p1, lambda_s, lambda_u] = ...
    bestDomainFunction(a, b, c, alpha, ...
              tau, N, R, allowed_error, num_theta, num_r);

%Plane Data
  
%Point q is a point on the plane:
q0 = p0 + 0.5*(p1-p0);

%eta one and two are the direction vectors of the plane:        
eta1 = [1; 0; -1]
eta2 = [0; 1; -1]
  
%Iterate Disks:        

Wu = iterate_set_k(Wu_loc, k, alpha, tau, a, b, c);
Ws = inverse_iterate_set_k(Ws_loc, k, alpha, tau, a, b, c);

%=========================================================================
%Set Operations:
%=========================================================================

intersectPlane_u = setIntersectPlane(Wu, eta1, eta2, q0, intersectionTol);
intersectPlane_s = setIntersectPlane(Ws, eta1, eta2, q0, intersectionTol);

[intersectionGuesses_u, intersectionGuesses_s] = ... 
    stableUnstableIntersection(intersectPlane_u, intersectPlane_s, 2*intersectionTol)

%=======================================================================
%Find Guesses:
%=======================================================================

[guesses_uParameters, guesses_uIterates, k_u, rawParameters_u] = findUnstableParameters(intersectionGuesses_u, ...
    Du, Wu_loc, lambda_u, alpha, tau, a, b, c);



[guesses_sParameters, guesses_sIterates, k_s, rawParameters_s] = findStableParameters(intersectionGuesses_s, ...
    Ds, Ws_loc, lambda_s, alpha, tau, a, b, c);



k_u = guesses_uIterates(1)
k_s = guesses_sIterates(1)



temp = size(guesses_uParameters);
numGuesses = temp(1)
epsilon_ic = 0.5*10^(-15)
maxTry = 1000;


[new_guesses_uParameters(1, 1:2), new_guesses_sParameters(1, 1:2)] = ...
    initialGuessNewton(N, k_u, k_s, epsilon_ic, ...
    rawParameters_u(1,1:2), rawParameters_s(1, 1:2), Au, Bu, Cu, As, Bs, Cs, ...
    alpha, tau, a, b, c, maxTry);


  Pu = evaluate_complexCase(new_guesses_uParameters(1,1), new_guesses_uParameters(1, 2), Au, Bu, Cu, N)';
  Ps = evaluate_complexCase(new_guesses_sParameters(1,1), new_guesses_sParameters(1, 2), As, Bs, Cs, N)';
  F1 = iterate_point_k(Pu, k_u, alpha, tau, a, b, c) 
  F2 = inverse_iterate_point_k(Ps, k_s, alpha, tau, a, b, c)      
  intersectionError = norm(F1 - F2)



  
[fkAu, fkBu, fkCu] = iterate_k_compose_f_P(k_u, N, Au, Bu, Cu, alpha, tau, a, b, c);


[fkAs, fkBs, fkCs] = iterate_k_compose_invf_P(k_s, N, As, Bs, Cs, alpha, tau, a, b, c);

%This is where you pick which point to use:

s = new_guesses_uParameters(1,1)
t = new_guesses_uParameters(1,2)

sbar = new_guesses_sParameters(1,1) 
tbar = new_guesses_sParameters(1, 2)

%Unstable  Tests:
p_1 = evaluate_complexCase(s, t, Au, Bu, Cu, N)
fkp_1 = iterate_point_k(p_1, k_u, alpha, tau, a, b, c)'

fkq_1 = evaluate_complexCase(s, t, fkAu, fkBu, fkCu, N)
          
error = norm(fkp_1 - fkq_1)

%Stable Tests:
p_1 = evaluate_complexCase(sbar, tbar, As, Bs, Cs, N)
invfkp_1 = inverse_iterate_point_k(p_1, k_s, alpha, tau, a, b, c)'
invfkq_1 = evaluate_complexCase(sbar, tbar, fkAs, fkBs, fkCs, N)          
error = norm(invfkp_1 - invfkq_1)
intersectionError = norm(invfkq_1 - fkq_1)


tic

%Parameterize Heteroclinic Segment:
Pu = evaluate_complexCase(s, t, Au, Bu, Cu, N)'
Ps = evaluate_complexCase(sbar, tbar, As, Bs, Cs, N)'
F1 = iterate_point_k(Pu, k_u, alpha, tau, a, b, c) 
F2 = inverse_iterate_point_k(Ps, k_s, alpha, tau, a, b, c)
F = F1 - F2

DF1 = Dfk_circ_P(k_u, s, t, Au, Bu, Cu, N, alpha, tau, a, b, c);
DF2 = Dfinvk_circ_P(k_s, sbar, tbar, As, Bs, Cs, N, alpha, tau, a, b, c); 
DF = [DF1, -DF2];
DF_0 = DF;

DF_inv = pinv(DF);
DF_inv_0 = DF_inv;


[U, S, V] = svd(DF);
direction_0 = 0.5*V(1:4, 4);

if ([s, t]*[V(1, 4); V(2, 4)] > 0)
  direction_0 = -1.0*direction_0;
end

ds = direction_0(1)
dt = direction_0(2)
dsbar = direction_0(3)
dtbar = direction_0(4)




alpha_0 = zeros(K+1, 1);
beta_0 = zeros(K+1, 1);
alphaBar_0 = zeros(K+1, 1);
betaBar_0 = zeros(K+1, 1);

alpha_0(1) = s;
beta_0(1) = t;
alphaBar_0(1) = sbar;
betaBar_0(1) = tbar;

alpha_0(2) = ds;
beta_0(2) = dt;
alphaBar_0(2) = dsbar;
betaBar_0(2) = dtbar;

[Pu_1, Pu_2, Pu_3] = P_composition_gamma(N, K, fkAu, fkBu, fkCu, alpha_0, beta_0);
[Ps_1, Ps_2, Ps_3] = P_composition_gamma(N, K, fkAs, fkBs, fkCs, alphaBar_0, betaBar_0);


theta_1 = -0.3;
gamma_1 = [horner(theta_1, alpha_0, K), horner(theta_1, beta_0, K)];
point_0 = evaluate_complexCase(s, t, fkAu, fkBu, fkCu, N)';
point_1 = evaluate_complexCase(gamma_1(1), gamma_1(2), fkAu, fkBu, fkCu, N)';
point_2 = [ horner(theta_1, Pu_1, K);
            horner(theta_1, Pu_2, K);
            horner(theta_1, Pu_3, K)];

        
deltaSpace_u = norm(point_0 - point_2)        
error_F_u = norm(point_1 - point_2)

theta_1 = 0.3;
gamma_1s = [horner(theta_1, alphaBar_0, K), horner(theta_1, betaBar_0, K)];
point_0s = evaluate_complexCase(sbar, tbar, fkAs, fkBs, fkCs, N)';
point_1s = evaluate_complexCase(gamma_1s(1), gamma_1s(2), fkAs, fkBs, fkCs, N)';
point_2s = [horner(theta_1, Ps_1, K);
            horner(theta_1, Ps_2, K);
            horner(theta_1, Ps_3, K)];

        
deltaSpace_s = norm(point_0s - point_2s)        
error_F_s = norm(point_1s - point_2s)


alpha_j = alpha_0;
beta_j = beta_0;
alphaBar_j = alphaBar_0;
betaBar_j = betaBar_0;

for (j = 1:5)

hetero_NewtonStep = j    
    
[alpha_correction, beta_correction, alphaBar_correction, betaBar_correction] = ...
    inverseDF_heteroclinic(N, K, alpha_j, beta_j, alphaBar_j, betaBar_j, fkAu, fkBu, fkCu, fkAs, fkBs, fkCs);

 delta_alpha = norm(alpha_correction(1:10));
 delta_beta = norm(beta_correction(1:10));
 delta_alphaBar = norm(alphaBar_correction(1:10));
 delta_betaBar = norm(betaBar_correction(1:10));
 thisDelta = sqrt(delta_alpha^2 + delta_beta^2 + delta_alphaBar^2 + delta_betaBar^2)

alpha_j = alpha_j - alpha_correction;
beta_j = beta_j - beta_correction;
alphaBar_j = alphaBar_j - alphaBar_correction;
betaBar_j = betaBar_j - betaBar_correction;

end


heteroArcTime = toc
%==========================================================================
%Parameter Space Work:
%==========================================================================

parmPath = zeros(100, 2);
parmPath_1 = zeros(100, 2);
parmPath_2 = zeros(100, 2);
parmPath_3 = zeros(100, 2);
parmPath_4 = zeros(100, 2);

arcParameters = linspace(-0.5, 0.5, 100);
circParameters = linspace(0.0, 2*pi, 1000);
circ = [0.65*cos(circParameters)', 0.65*sin(circParameters)'];

for (k = 1:100)
    parmPath(k, 1:2) = [horner(arcParameters(k), alpha_j, K), 
                    horner(arcParameters(k), beta_j, K)];

    complexifyPath(k) = parmPath(k, 1) + i*parmPath(k, 2);
end    

complexifyPath_1 = lambda_u^(-1)*complexifyPath;
complexifyPath_2 = lambda_u^(-1)*complexifyPath_1;
complexifyPath_3 = lambda_u^(-1)*complexifyPath_2;
complexifyPath_4 = lambda_u^(-1)*complexifyPath_3;

parmPath_1 = [real(complexifyPath_1)', imag(complexifyPath_1)'];
parmPath_2 = [real(complexifyPath_2)', imag(complexifyPath_2)'];
parmPath_3 = [real(complexifyPath_3)', imag(complexifyPath_3)'];
parmPath_4 = [real(complexifyPath_4)', imag(complexifyPath_4)'];

figure 
hold on

plot(circ(:,1), circ(:,2), 'b');
%plot(parmPath(:, 1), parmPath(:, 2), 'g')
plot(parmPath_1(:, 1), parmPath_1(:, 2), 'k')
plot(parmPath_2(:, 1), parmPath_2(:, 2), 'g')
plot(parmPath_3(:, 1), parmPath_3(:, 2), 'g')
plot(parmPath_4(:, 1), parmPath_4(:, 2), 'g')

%==========================================================================
%Phase Spcae Work:
%==========================================================================


outOfNewton = 1

[gamma_u1, gamma_u2, gamma_u3] = P_composition_gamma(N, K, fkAu, fkBu, fkCu, alpha_j, beta_j);
[gamma_s1, gamma_s2, gamma_s3] = P_composition_gamma(N, K, fkAs, fkBs, fkCs, alphaBar_j, betaBar_j);

theta_0 = 0.0;
theta_1 = 0.5;

testPoint_0 = [horner(theta_0, gamma_u1, K);
               horner(theta_0, gamma_u2, K);
               horner(theta_0, gamma_u3, K)];

testPoint_u = [horner(theta_1, gamma_u1, K);
               horner(theta_1, gamma_u2, K);
               horner(theta_1, gamma_u3, K)];

testPoint_s = [horner(theta_1, gamma_s1, K);
               horner(theta_1, gamma_s2, K);
               horner(theta_1, gamma_s3, K)];
           
heteroclinic_delta = norm(testPoint_0 - testPoint_u)                   
heteroclinic_error = norm(testPoint_s - testPoint_u)



theta_parameters = linspace(-0.5, 0.5, 100);

path = zeros(100, 3);
for (k = 1:100)
    path(k, 1:3) = [horner(theta_parameters(k), gamma_u1, K), 
                    horner(theta_parameters(k), gamma_u2, K), 
                    horner(theta_parameters(k), gamma_u3, K)];
end    

pathIterate = iterate_set_k(path, 50, alpha, tau, a, b, c);
pathInverseIterate = inverse_iterate_set_k(path, 50, alpha, tau, a, b, c);
continuePath = setUnion(pathIterate, pathInverseIterate);

figure
hold on

plot3(continuePath(:,1), continuePath(:,2), continuePath(:,3), 'g.');
%plot3(path(:, 1), path(:, 2), path(:, 3), 'k*')
plot3(Wu_loc(:,1), Wu_loc(:,2), Wu_loc(:,3), 'b.')
plot3(Ws_loc(:,1), Ws_loc(:,2), Ws_loc(:,3), 'r.')

%-----------------------------------------------------------------------


path_1 = iterate_set(path, alpha, tau, a, b, c);
path_minus1 = inverseIterate_set(path, alpha, tau, a, b, c);
path_2 = iterate_set(path_1, alpha, tau, a, b, c);
path_minus2 = inverseIterate_set(path_minus1, alpha, tau, a, b, c);
path_3 = iterate_set(path_2, alpha, tau, a, b, c);
path_minus3 = inverseIterate_set(path_minus2, alpha, tau, a, b, c);
path_4 = iterate_set(path_3, alpha, tau, a, b, c);
path_minus4 = inverseIterate_set(path_minus3, alpha, tau, a, b, c);



figure
hold on

%view(2)
plot3(Wu_loc(:,1), Wu_loc(:,2), Wu_loc(:,3), 'b.')
plot3(Ws_loc(:,1), Ws_loc(:,2), Ws_loc(:,3), 'r.')
plot3(path(:, 1), path(:, 2), path(:, 3), 'k*')
axis([-2 2 -2 2 -2 2])


figure
hold on

%plot3(Wu_loc(:,1), Wu_loc(:,2), Wu_loc(:,3), 'b.')
%plot3(Ws_loc(:,1), Ws_loc(:,2), Ws_loc(:,3), 'r.')
plot3(path(:, 1), path(:, 2), path(:, 3), 'k*')
axis([-2 2 -2 2 -2 2])

figure
hold on

%plot3(Wu_loc(:,1), Wu_loc(:,2), Wu_loc(:,3), 'b.')
%plot3(Ws_loc(:,1), Ws_loc(:,2), Ws_loc(:,3), 'r.')
plot3(path(:, 1), path(:, 2), path(:, 3), 'g*')
plot3(path_1(:, 1), path_1(:, 2), path_1(:, 3), 'k*')
axis([-2 2 -2 2 -2 2])

figure
hold on

%plot3(Wu_loc(:,1), Wu_loc(:,2), Wu_loc(:,3), 'b.')
%plot3(Ws_loc(:,1), Ws_loc(:,2), Ws_loc(:,3), 'r.')
plot3(path(:, 1), path(:, 2), path(:, 3), 'g*')
plot3(path_1(:, 1), path_1(:, 2), path_1(:, 3), 'g*')
plot3(path_2(:, 1), path_2(:, 2), path_2(:, 3), 'k*')
axis([-2 2 -2 2 -2 2])

figure
hold on

%plot3(Wu_loc(:,1), Wu_loc(:,2), Wu_loc(:,3), 'b.')
%plot3(Ws_loc(:,1), Ws_loc(:,2), Ws_loc(:,3), 'r.')
plot3(path(:, 1), path(:, 2), path(:, 3), 'g*')
plot3(path_1(:, 1), path_1(:, 2), path_1(:, 3), 'g*')
plot3(path_2(:, 1), path_2(:, 2), path_2(:, 3), 'g*')
plot3(path_3(:, 1), path_3(:, 2), path_3(:, 3), 'k*')
axis([-2 2 -2 2 -2 2])



figure
hold on

%plot3(Wu_loc(:,1), Wu_loc(:,2), Wu_loc(:,3), 'b.')
%plot3(Ws_loc(:,1), Ws_loc(:,2), Ws_loc(:,3), 'r.')
plot3(path_1(:, 1), path_1(:, 2), path_1(:, 3), 'g*')
plot3(path_2(:, 1), path_2(:, 2), path_2(:, 3), 'g*')
plot3(path_3(:, 1), path_3(:, 2), path_3(:, 3), 'g*')

plot3(path(:, 1), path(:, 2), path(:, 3), 'g*')
plot3(path_minus1(:, 1), path_minus1(:, 2), path_minus1(:, 3), 'k*')
axis([-2 2 -2 2 -2 2])

figure
hold on

%plot3(Wu_loc(:,1), Wu_loc(:,2), Wu_loc(:,3), 'b.')
%plot3(Ws_loc(:,1), Ws_loc(:,2), Ws_loc(:,3), 'r.')
plot3(path(:, 1), path(:, 2), path(:, 3), 'g*')
plot3(path_1(:, 1), path_1(:, 2), path_1(:, 3), 'g*')
plot3(path_2(:, 1), path_2(:, 2), path_2(:, 3), 'g*')
plot3(path_3(:, 1), path_3(:, 2), path_3(:, 3), 'g*')

plot3(path_minus1(:, 1), path_minus1(:, 2), path_minus1(:, 3), 'g*')
plot3(path_minus2(:, 1), path_minus2(:, 2), path_minus2(:, 3), 'k*')
axis([-2 2 -2 2 -2 2])

figure
hold on

%plot3(Wu_loc(:,1), Wu_loc(:,2), Wu_loc(:,3), 'b.')
%plot3(Ws_loc(:,1), Ws_loc(:,2), Ws_loc(:,3), 'r.')
plot3(path(:, 1), path(:, 2), path(:, 3), 'g*')
plot3(path_1(:, 1), path_1(:, 2), path_1(:, 3), 'g*')
plot3(path_2(:, 1), path_2(:, 2), path_2(:, 3), 'g*')
plot3(path_3(:, 1), path_3(:, 2), path_3(:, 3), 'g*')

plot3(path_minus1(:, 1), path_minus1(:, 2), path_minus1(:, 3), 'g*')
plot3(path_minus2(:, 1), path_minus2(:, 2), path_minus2(:, 3), 'g*')
plot3(path_minus3(:, 1), path_minus3(:, 2), path_minus3(:, 3), 'k*')
axis([-2 2 -2 2 -2 2])


figure
hold on

plot3(continuePath(:,1), continuePath(:,2), continuePath(:,3), 'g.');
%plot3(path(:, 1), path(:, 2), path(:, 3), 'k*')
axis([-2 2 -2 2 -2 2])


figure
hold on

plot3(continuePath(:,1), continuePath(:,2), continuePath(:,3), 'g.');
%plot3(path(:, 1), path(:, 2), path(:, 3), 'k*')
plot3(Wu_loc(:,1), Wu_loc(:,2), Wu_loc(:,3), 'b.')
plot3(Ws_loc(:,1), Ws_loc(:,2), Ws_loc(:,3), 'r.')
axis([-2 2 -2 2 -2 2])



heteroArcTime
% 
% 
% disk_u = unique(Wu_loc, 'rows');
% disk_s = unique(Ws_loc, 'rows');
% 
% 
% 
% %% Run  program
% [tu]=Delaunay2_5D(disk_u);
% [ts]=Delaunay2_5D(disk_s);
% 
% figure
% hold on
% plot3(x0, x0, x0, 'k*')
% plot3(x1, x1, x1, 'k*')
% 
% ax_lim = 2.0;
% 
% axis([-ax_lim ax_lim -ax_lim ax_lim -ax_lim ax_lim])
% trisurf(tu, disk_u(:,1), disk_u(:,2), disk_u(:,3), 'SpecularColorReflectance', 0.99, ...
%                                     'FaceAlpha', 1.0, 'BackFaceLighting', 'reverselit',...
%                                     'FaceLighting', 'gouraud', 'facecolor','b','edgecolor','none')
% 
% trisurf(ts, disk_s(:,1), disk_s(:,2), disk_s(:,3), 'SpecularColorReflectance', 0.99, ...
%                                     'FaceAlpha', 0.75, 'BackFaceLighting', 'reverselit',...
%                                     'FaceLighting', 'gouraud', 'facecolor','r','edgecolor','none')
% 
%                                 
% %plot3(path(:,1), path(:,2), path(:,3), 'g*');                                
% %plot3(continuePath(:,1), continuePath(:,2), continuePath(:,3), 'g.');
% 
% camlight headlight
                                