clear
format long 

%=================================================================
%   PROGRAM COMPUTES BOTH ONE AND TWO DIMENSIONAL STABLE AND 
%   UNSTABLE MANIFOLDS AT BOTH FIXED POINTS, BY PARAMETERIZATION
%
%   COMPUTE HOMOCLINIC ORBIT:
%
%   Plot 1 and 2 D manifolds for Dullin-Meiss parameters.
%       Steps 1, 2, and 3 of paper codes.
%
%
%  Development Nots:  The code for globalizing the stable and 
%  unstable manifolds are in aaa_barycenterWorkII.m and 
%  aaa_barycenterWu.m respectively.
%
%
%
%================================================================

%==============================================================
%Computation Controls:
%==============================================================



%Map Parameter Set: Dullin to Lomeli Meiss.
   %Dullin Meiss Parms:
      a_dm = 1
      b_dm = 0.5
      c_dm = 0.5
            
      
      
      eps_dm = 2.11    %boundary near 6.182001
      mu_dm = -2.4
      
      
   %Convert to Lomeli:   
      a = c_dm
      b = b_dm - 2*c_dm
      c = c_dm + a_dm - b_dm
      
      tau =    2*a_dm*(3+mu_dm)/(2*a_dm - b_dm)
      alpha =  ((9+6*mu_dm+mu_dm^2)*a_dm - 4*eps_dm*a_dm^2 + ...
               4*eps_dm*a_dm*b_dm - eps_dm*b_dm^2)/(2*a_dm - b_dm)^2



%---for epsilon > 3.5-ish guess_tol ~ 10^(-1) is good.
%---for epsilon ~ 0.2 - 2 neew  guess_tol ~ 2.5 X 10^(-2)
%---what to do close to eps ~ 0.1?
guess_tol = 5*10^(-2) 


l = 20

numRad = 30;
numTheta = 250;

%Order of manifold computation:
  %Order of 2D manifolds:
  N = 65;
  %Order of 1D manifold computations 
  order_1 = 150;
   
  n_u = 1;
  ns_0 = 1
  
%==========================================================================
%Generate zeroth and first order problem data:
%==========================================================================

[p0, p1, lambda0_s, lambda0_u1, lambda0_u2, ...
         lambda1_u, lambda1_s1, lambda1_s2, ...
         xi0_s, xi0_u1, xi0_u2, ...
         xi1_u, xi1_s1, xi1_s2] ...
     = firstOrderData_3DHenon(a, b, c, alpha, tau) 
  
%=====================================================================
%One-D Manifolds 
%=====================================================================

%Step 1: Compute 1D Coefficients:

scale = 1;

[A0s, B0s, C0s] = oneDimManifold_3DHenon(a, b, c, alpha, tau, ...
     order_1, p0, lambda0_s, xi0_s, scale);

[A1u, B1u, C1u] = oneDimManifold_3DHenon(a, b, c, alpha, tau, ...
    order_1, p1, lambda1_u, xi1_u, scale);

%Step 2: Choose Parameter domain radius:
r0 = 1.5;
step = 0.05;
eps_tol = 5*10^(-15);

r1_u = oneD_parmInterval(A1u, B1u, C1u, order_1, lambda1_u, r0, step, eps_tol, ...
                         alpha, tau, a, b, c);


r0_s = oneD_parmInterval(A0s, B0s, C0s, order_1, lambda0_s, r0, step, eps_tol, ...
                         alpha, tau, a, b, c);

r0_s
r1_u

%Step 3: Compute fundamental domain point clouds:
numPoints = 1000;

[I1u_left, I1u_right, W1u_fd_left, W1u_fd_right, ...
    I1u, W1u_loc] = ...
    oneDim_fd(A1u, B1u, C1u, order_1, lambda1_u, r1_u, numPoints);

[I0s_left, I0s_right, W0s_fd_left, W0s_fd_right, ...
    I0s, W0s_loc] = ...
    oneDim_fd(A0s, B0s, C0s, order_1, lambda0_s, r0_s, numPoints);


%[I1u_left, I1u_right, W1u_fd_left, W1u_fd_right] = ...
%    oneDim_fd(A1u, B1u, C1u, order_1, lambda1_u, r1_u, numPoints);

%[I0s_left, I0s_right, W0s_fd_left, W0s_fd_right] = ...
%    oneDim_fd(A0s, B0s, C0s, order_1, lambda0_s, r0_s, numPoints);

%==========================================================================
%Two-D Manifolds:
%==========================================================================

%Step 1: Compute Coefficients:
scale1 = 2;
scale2 = 2;

[A0u, B0u, C0u] = twoDimManifold_3DHenon(a, b, c, alpha, tau, ...
    N, p0, lambda0_u1, lambda0_u2, xi0_u1, xi0_u2, scale1, scale2);

[A1s, B1s, C1s] = twoDimManifold_3DHenon(a, b, c, alpha, tau, ...
    N, p1, lambda1_s1, lambda1_s2, xi1_s1, xi1_s2, scale1, scale2);

%Step 2: Choose Parameter Domain Radius:
r0 = 1.2;
step = 0.1;
eps_tol = 5*10^(-5);

r0_u = twoD_parmDomain(A0u, B0u, C0u, N, lambda0_u1, r0, step, eps_tol, ...
                      alpha, tau, a, b, c);

r1_s = twoD_parmDomain(A1s, B1s, C1s, N, lambda1_s1, r0, step, eps_tol, ...
                      alpha, tau, a, b, c);                  

r0_u
r1_s    

%Step 3: Compute 2D fundamental domains:

[D0u_fd, W0u_fd] = twoDim_fd(A0u, B0u, C0u, N, ...
                             lambda0_u1, r0_u, numRad, numTheta);
[D1s_fd, W1s_fd] = twoDim_fd(A1s, B1s, C1s, N, ...
                             lambda1_s1, r1_s, numRad, numTheta);

%==========================================================================
%Step 5: Compute Homoclinic Orbit/Tangle.
%        (Here we will just compute at p0)
%==========================================================================


%----------------Compute Homoclinic Equation Parameters:-------------------

  %Iterate the 2D fundamental domain:
 
  W0u_global = ...
        k_image_set(W0u_fd, n_u, alpha, tau, a, b, c);
 
   globalize_fd = inverse_iterate_set_k(W0s_fd_right, 10, ...
                                        alpha, tau, a, b, c); 
    
   figure
   hold on
   plot3(W0u_global(:,1), W0u_global(:,2), W0u_global(:,3), 'b.'); 
   %plot3(W0s_fd_right(:,1), W0s_fd_right(:,2), W0s_fd_right(:,3), 'r.'); 
   plot3(globalize_fd(:,1), globalize_fd(:,2), globalize_fd(:,3), 'r.');
   plot3(p0(1), p0(2), p0(3), 'k*'); 
   axis([-l l -l l -l l]);
   
   
  %Iterate the 1D fundamental domain until an approximate
  %intersection with the 2D domain is found.  Return the 
  %iterate numbers, and the approximate parameters.
  
  tol = guess_tol;
  %tol = 2.5*10^(-2)
  
  
  [theta_0, tau_0, n_s] = homoclinicInitialGuess(...
      D0u_fd, I0s_left, I0s_right, ...
      W0u_global, W0s_fd_left, W0s_fd_right, ...
      alpha, tau, a, b, c, p0, tol, ns_0);
  


%---------------Solve Homoclinic Equation by Newton Method:----------------



[thetaBar, tauBar, error] = homoclinicPoint_p0(theta_0, tau_0, n_u, n_s, ...
    A0u, B0u, C0u, A0s, B0s, C0s, N, order_1, alpha, tau, a, b, c)



%----------------Plot Orbit:-----------------------------------------------

homPoint_s = [horner(tauBar, A0s, order_1), ...
              horner(tauBar, B0s, order_1), ...
              horner(tauBar, C0s, order_1)];

homPoint_u = evaluate_complexCase(thetaBar(1), thetaBar(2), ...
                                  A0u, B0u, C0u, N);
      
timesToIterate = 30                              
homOrbit_s1 = inverse_iterate_set_k(homPoint_s, n_s, alpha, tau, a, b, c);      
homOrbit_s2 = iterate_set_k(homPoint_s, timesToIterate, alpha, tau, a, b, c); 
homOrbit_u1 = iterate_set_k(homPoint_u, n_u, alpha, tau, a, b, c); 
homOrbit_u2 = inverse_iterate_set_k(homPoint_u, timesToIterate, alpha, tau, a, b, c); 

figure
hold on
plot3(homOrbit_s1(:,1), homOrbit_s1(:,2), homOrbit_s1(:,3), 'g.');
plot3(homOrbit_s2(:,1), homOrbit_s2(:,2), homOrbit_s2(:,3), 'g.');
plot3(homOrbit_u1(:,1), homOrbit_u1(:,2), homOrbit_u1(:,3), 'g.');
plot3(homOrbit_u2(:,1), homOrbit_u2(:,2), homOrbit_u2(:,3), 'g.');

time1 = linspace(-timesToIterate, 0, timesToIterate+1);
time2 = linspace(0, n_u, n_u+1);
time3 = linspace(n_u, n_s + n_u, n_s + 1);
time4 = linspace(n_s+n_u, n_s+n_u+timesToIterate, timesToIterate+1);
figure
hold on
plot(time1(:), flipud(homOrbit_u2(:, 1)), 'g');
plot(time2(:), homOrbit_u1(:,1), 'g');
plot(time3(:), flipud(homOrbit_s1(:,1)), 'g');
plot(time4(:), homOrbit_s2(:,1), 'g');
plot(time1(:), flipud(homOrbit_u2(:, 1)), 'k.');
plot(time2(:), homOrbit_u1(:,1), 'k.');
plot(time3(:), flipud(homOrbit_s1(:,1)), 'k.');
plot(time4(:), homOrbit_s2(:,1), 'k.');



%q0 = [horner(tau, A0s, order_1), ...
%      horner(tau, B0s, order_1), ...
%      horner(tau, C0s, order_1)]; 

%q0 = evaluate_complexCase(thetaBar(1), thetaBar(2), A0u, B0u, C0u, N);
  

%forward_orbit = point_orbit_k(q0, 90, alpha, tau, a, b, c); 
%backward_orbit = point_inverse_orbit_k(q0, 90, alpha, tau, a, b, c);
%homoclinicOrbit = setUnion(forward_orbit, backward_orbit);

%figure
%hold on
%plot3(homoclinicOrbit(:,1), homoclinicOrbit(:,2), homoclinicOrbit(:,3), 'k.')
%plot3(W0u_global(:,1), W0u_global(:,2), W0u_global(:,3), 'b.');   
  
  





