
clear
format long 

%=================================================================
%   Compute Two Dimensional Unstable Manifolds by
%   Parametrization Method.
%
%
%  (This is the correct code...)
%
%================================================================



%==============================================================
%Computation Controls:
%==============================================================

%Map Parameters:
a = 0.5;
b = 0.0;
c = 0.5;

alpha = -0.1;  %0.0
tau = -0.1;   %-0.17

%NOTE:  For alpha = -0.5 and tau = 3.0 p0 is stable
%and p1 is unstable.  In the stable case the first eigenvalue
%is real and the second two are complex.  For the unstable case
%the the first two eigenvalues are complex and the third is real.

%polynomial approximation of W^s(p1):

  %Order of manifold computation
  N = 60;
    
  %scale the eigenvectors (determins paramatrization uniquely):
  scale_u1 = 2;
  scale_u2 = 2;  
  scale_s1 = 2;
  scale_s2 = 2;  
  
  
  %Domain parameters:
  R = 0.6
  bound = 1.8;
  num_theta = 200;
  num_r = 200;
  
  %number of poin
  %num_t = 100;
  %num_s = 100;
     
  %s0 = -0.02;
  %t0 = -0.02;
   
  %t_length = abs(2*t0);
  %s_length = abs(2*s0);
   
  %h2 = s_length/num_s;
  %h1 = t_length/num_t;
  
  
%==============================================================
%Program:
%==============================================================

%Fixed Points, Eigenvalues, Eigenvectors, ect:  ================

x0 = (1/2)*(-tau + sqrt( tau^2 - 4*alpha));
x1 = (1/2)*(-tau - sqrt( tau^2 - 4*alpha));

p0 = [x0; x0; x0];
p1 = [x1; x1; x1];

fp0 = henon3D(p0, alpha, tau, a, b, c);
fp1 = henon3D(p1, alpha, tau, a, b, c);

%Differential at p0;

A_0 = (2*a + b)*x0 + tau;
B_0 = (2*c + b)*x0;

Df0 = [ A_0, B_0, 1;
          1,   0, 0;
          0,   1, 0];
    
%Differential of p1;

A_1 = (2*a + b)*x1 + tau;
B_1 = (2*c + b)*x1;

Df1 = [ A_1, B_1, 1;
          1,   0, 0;
          0,   1, 0];
    
%Eigenvalues and vectors of Df0:

[E0, eigenvalues_0] = eigs(Df0);

E0
eigenvalues_0

%Eigenvalues and vectors of Df1:

[E1, eigenvalues_1] = eigs(Df1);

E1
eigenvalues_1

%Check linear stablilty of the fixed points

stability_0 = norm(eigenvalues_0(2,2))
stability_1 = norm(eigenvalues_1(1,1))



%==============================================================
%    Computations:
%==============================================================



%=======================================================
%Compute unstable manifold of p1:
%=======================================================

lambda = eigenvalues_1(1,1);
xi = scale_u1*E1(:, 1);
xi_bar = scale_u2*E1(:, 2);

[Au, Bu, Cu] = parametrizationCoefficents_complexCase(N, ...
    p1, xi, xi_bar, lambda, tau, a, b, c);


computed_Wu = 1

%===========================================================
%Compute stable manifold of p0:
%===========================================================

lambda = eigenvalues_0(2,2);
nu = scale_s1*E0(:, 2);
nu_bar = scale_s2*E0(:, 3);

[As, Bs, Cs] = parametrizationCoefficents_complexCase(N, ... 
    p0, nu, nu_bar, lambda, tau, a, b, c);

computed_Ws = 1

%==========================================================================
%Evaluate the polynomial to compute the manifolds:
%==========================================================================

%Prepare Domain:
[Du, Ds, Wu, Ws] = makeBestDomain(R, bound, num_theta, num_r,  Au, Bu, Cu, As, Bs, Cs, N);

figure
hold on
plot(Du(:,1), Du(:,2), 'b.');

figure
hold on
plot(Ds(:,1), Ds(:,2), 'r.');

%Wu = evaluate_manifold_on_set(Du, N, Au, Bu, Cu);
%Wu = real(Wu);

%evaluated_Wu = 1

%Ws = evaluate_manifold_on_set(Ds, N, As, Bs, Cs);
%Ws = real(Ws);

%evaluated_Ws = 1

%iterate:
%fWs = inverse_iterate_set(Ws, alpha, tau, a, b, c);
%f2Ws = inverse_iterate_set(fWs, alpha, tau, a, b, c);
%f3Ws = inverse_iterate_set(f2Ws, alpha, tau, a, b, c);
%f4Ws = inverse_iterate_set(f3Ws, alpha, tau, a, b, c);
%f5Ws = inverse_iterate_set(f4Ws, alpha, tau, a, b, c);
%f6Ws = inverse_iterate_set(f5Ws, alpha, tau, a, b, c);

%fWu = iterate_set(Wu, alpha, tau, a, b, c);
%f2Wu = iterate_set(fWu, alpha, tau, a, b, c);
%f3Wu = iterate_set(f2Wu, alpha, tau, a, b, c);
%f4Wu = iterate_set(f3Wu, alpha, tau, a, b, c);
%f5Wu = iterate_set(f4Wu, alpha, tau, a, b, c);
%f6Wu = iterate_set(f5Wu, alpha, tau, a, b, c);

%================================================================
%Results:
%================================================================


%Phase space plots near zero:
figure
hold on
plot3(x0, x0, x0, 'k*')
plot3(x1, x1, x1, 'k*')

plot3(Wu(:,1), Wu(:,2), Wu(:,3), 'r.'); 
%plot3(fWu(:,1), fWu(:,2), fWu(:,3), 'r.'); 
%plot3(f2Wu(:,1), f2Wu(:,2), f2Wu(:,3), 'r.'); 
%plot3(f3Wu(:,1), f3Wu(:,2), f3Wu(:,3), 'r.'); 
%plot3(f4Wu(:,1), f4Wu(:,2), f4Wu(:,3), 'r.'); 
%plot3(f5Wu(:,1), f5Wu(:,2), f5Wu(:,3), 'r.'); 
%plot3(f6Wu(:,1), f6Wu(:,2), f6Wu(:,3), 'r.'); 


plot3(Ws(:,1), Ws(:,2), Ws(:,3), 'b.'); 
%plot3(fWs(:,1), fWs(:,2), fWs(:,3), 'b.'); 
%plot3(f2Ws(:,1), f2Ws(:,2), f2Ws(:,3), 'b.'); 
%plot3(f3Ws(:,1), f3Ws(:,2), f3Ws(:,3), 'b.'); 
%plot3(f4Ws(:,1), f4Ws(:,2), f4Ws(:,3), 'b.'); 
%plot3(f5Ws(:,1), f5Ws(:,2), f5Ws(:,3), 'b.'); 
%plot3(f6Ws(:,1), f6Ws(:,2), f6Ws(:,3), 'b.'); 

axis([-5 5 -5 5 -5 5])

%Phase space plot (global):
figure
hold on
plot3(x0, x0, x0, 'k*')
plot3(x1, x1, x1, 'k*')

plot3(Wu(:,1), Wu(:,2), Wu(:,3), 'r.'); 
%plot3(fWu(:,1), fWu(:,2), fWu(:,3), 'r.'); 
%plot3(f2Wu(:,1), f2Wu(:,2), f2Wu(:,3), 'r.'); 
%plot3(f3Wu(:,1), f3Wu(:,2), f3Wu(:,3), 'r.'); 
%plot3(f4Wu(:,1), f4Wu(:,2), f4Wu(:,3), 'r.'); 
%plot3(f5Wu(:,1), f5Wu(:,2), f5Wu(:,3), 'r.'); 
%plot3(f6Wu(:,1), f6Wu(:,2), f6Wu(:,3), 'r.'); 


plot3(Ws(:,1), Ws(:,2), Ws(:,3), 'b.'); 
%plot3(fWs(:,1), fWs(:,2), fWs(:,3), 'b.'); 
%plot3(f2Ws(:,1), f2Ws(:,2), f2Ws(:,3), 'b.'); 
%plot3(f3Ws(:,1), f3Ws(:,2), f3Ws(:,3), 'b.'); 
%plot3(f4Ws(:,1), f4Ws(:,2), f4Ws(:,3), 'b.'); 
%plot3(f5Ws(:,1), f5Ws(:,2), f5Ws(:,3), 'b.'); 
%plot3(f6Ws(:,1), f6Ws(:,2), f6Ws(:,3), 'b.'); 



x_coor = -R:0.01:R;
y_coor = x_coor;
[X, Y] = meshgrid(x_coor,y_coor);
[Zu1, Zu2, Zu3] = mesh_eval(X, Y, Au, Bu, Cu, N);
[Zs1, Zs2, Zs3] = mesh_eval(X, Y, As, Bs, Cs, N);

figure
hold on
surf(X, Y, Zu1);
axis([-R R -R R -bound bound])
title('Unstable coordinate 1')

figure
hold on
surf(X, Y, Zu2);
axis([-R R -R R -bound bound])
title('Unstable coordinate 2')

figure
hold on
surf(X, Y, Zu3);
axis([-R R -R R -bound bound])
title('Unstable coordinate 3')

figure
hold on
surf(X, Y, Zs1);
axis([-R R -R R -bound bound])
title('Stable coordinate 1')

figure
hold on
surf(X, Y, Zs2);
axis([-R R -R R -bound bound])
title('Stable coordinate 2')

figure
hold on
surf(X, Y, Zs3);
axis([-R R -R R -bound bound])
title('Stable coordinate 3')






