function varargout=bestDomainFunction(a, b, c, alpha, ...
    tau, N, R, allowed_error, num_theta, num_r);


  %scale the eigenvectors (determins paramatrization uniquely):
  scale_u1 = 2;
  scale_u2 = 2;  
  scale_s1 = 2;
  scale_s2 = 2;  
 
  bound = 2.0
  
    
%==============================================================
%Program:
%==============================================================

%Fixed Points, Eigenvalues, Eigenvectors, ect:  ================

x0 = (1/2)*(-tau + sqrt( tau^2 - 4*alpha));
x1 = (1/2)*(-tau - sqrt( tau^2 - 4*alpha));

p0 = [x0; x0; x0];
p1 = [x1; x1; x1];

fp0 = henon3D(p0, alpha, tau, a, b, c);
fp1 = henon3D(p1, alpha, tau, a, b, c);

%Differential at p0;

A_0 = (2*a + b)*x0 + tau;
B_0 = (2*c + b)*x0;

Df0 = [ A_0, B_0, 1;
          1,   0, 0;
          0,   1, 0];
    
%Differential of p1;

A_1 = (2*a + b)*x1 + tau;
B_1 = (2*c + b)*x1;

Df1 = [ A_1, B_1, 1;
          1,   0, 0;
          0,   1, 0];
    
%Eigenvalues and vectors of Df0:

[E0, eigenvalues_0] = eigs(Df0);

E0
eigenvalues_0

%Eigenvalues and vectors of Df1:

[E1, eigenvalues_1] = eigs(Df1);

E1
eigenvalues_1

%Check linear stablilty of the fixed points

stability_0 = norm(eigenvalues_0(2,2))
stability_1 = norm(eigenvalues_1(1,1))




%==============================================================
%    Computations:
%==============================================================



%=======================================================
%Compute coefficents for unstable manifold of p1:
%=======================================================

lambda_u = eigenvalues_1(1,1);
xi = scale_u1*E1(:, 1);
xi_bar = scale_u2*E1(:, 2);

[Au, Bu, Cu] = parametrizationCoefficents_complexCase(N, ...
    p1, xi, xi_bar, lambda_u, tau, a, b, c);


computed_Wu = 1

%===========================================================
%Compute coefficents for stable manifold of p0:
%===========================================================

lambda_s = eigenvalues_0(2,2);
nu = scale_s1*E0(:, 2);
nu_bar = scale_s2*E0(:, 3);

[As, Bs, Cs] = parametrizationCoefficents_complexCase(N, ... 
    p0, nu, nu_bar, lambda_s, tau, a, b, c);

computed_Ws = 1

%==========================================================================
%Evaluate the polynomial to compute the manifolds:  Finds largest domain
%in the parameter space so that error tolerance is satisfied.  Returns 
%domains (jparameters) and their images (stable and unstable manifolds 
%in phase space).
%==========================================================================

%Prepare Domain:
[Du, Ds, Wu, Ws] = makeBestErrorDomain_vectorized(R, allowed_error, ...
    num_theta, num_r, ...
    lambda_u, lambda_s, Au, Bu, Cu, ...
    As, Bs, Cs, N, alpha, tau, a, b, c);


%================================================================
%Results:
%================================================================


%Return:
varargout(1,1)={Au};
varargout(2,1)={Bu};
varargout(3,1)={Cu};
varargout(4,1)={As};
varargout(5,1)={Bs};
varargout(6,1)={Cs};
varargout(7,1)={Du};
varargout(8,1)={Ds};
varargout(9,1)={Wu};
varargout(10,1)={Ws};
varargout(11,1)={x0};
varargout(12,1)={x1};
varargout(13,1)={p0};
varargout(14,1)={p1};
varargout(15,1)={lambda_s};
varargout(16,1)={lambda_u};



