function varargout=continue_heteroclinic(N, K, intersectionPoints, epsilon, ...
    guess_u, guess_s, Au, Bu, Cu, As, Bs, Cs, ...
    alpha, tau, a, b, c, tolerance, maxTry);


%N = order
%K = iterate number

%This could insert some erronious zeros into the 
%result if Newton method fails. Fix at end.
heteroclinicConnection = zeros(2*intersectionPoints+1, 3);
connection_uParSpace = zeros(2*intersectionPoints+1, 2);
connection_sParSpace = zeros(2*intersectionPoints+1, 2);

%Set up first Newton Iteration

Pu = evaluate_complexCase(guess_u(1), guess_u(2), Au, Bu, Cu, N)'
Ps = evaluate_complexCase(guess_s(1), guess_s(2), As, Bs, Cs, N)'
F1 = iterate_point_k(Pu, K, alpha, tau, a, b, c) 
F2 = inverse_iterate_point_k(Ps, K, alpha, tau, a, b, c)
F = F1 - F2

DF1 = Dfk_circ_P(K, guess_u(1), guess_u(2), Au, Bu, Cu, N, alpha, tau, a, b, c)
DF2 = Dfinvk_circ_P(K, guess_s(1), guess_s(2), As, Bs, Cs, N, alpha, tau, a, b, c) 
DF = [DF1, -DF2]

DF_inv = pinv(DF)
[U, S, V] = svd(DF);


%initialize loop variables:
Try = 0;
z_n = [guess_u(1); guess_u(2); guess_s(1); guess_s(2)];
%F = 1;

while ((norm(F) > tolerance) & (Try <= maxTry))
   z_n = z_n - DF_inv*F;
   Try = Try + 1;
   
   Pu = evaluate_complexCase(z_n(1), z_n(2), Au, Bu, Cu, N)';
   Ps = evaluate_complexCase(z_n(3), z_n(4), As, Bs, Cs, N)';
   F1 = iterate_point_k(Pu, K, alpha, tau, a, b, c); 
   F2 = inverse_iterate_point_k(Ps, K, alpha, tau, a, b, c);
   F = F1 - F2;
   
   DF1 = Dfk_circ_P(K, z_n(1), z_n(2), Au, Bu, Cu, N, alpha, tau, a, b, c);
   DF2 = Dfinvk_circ_P(K, z_n(3), z_n(4), As, Bs, Cs, N, alpha, tau, a, b, c); 
   DF = [DF1, -DF2];
   DF_inv = pinv(DF);   
    
   %F = evaluate_complexCase(z_n(1), z_n(2), A, B, C, N)' - L(z_n(3), z_n(4), eta1, eta2, q0); 
    %DP = evaluate_jacob_complexCase(z_n(1), z_n(2), A, B, C, N);
    %DF(1:3, 1:2) = DP;
    %DF(1:3, 3) = -eta1;
    %DF(1:3, 4) = -eta2; 
    %DF_inv = pinv(DF);    
    
    %z_n = z_n - DF_inv*F; 
    %Try = Try + 1;
end 

%F = evaluate_complexCase(z_n(1), z_n(2), A, B, C, N)' - L(z_n(3), z_n(4), eta1, eta2, q0); 
%DP = evaluate_jacob_complexCase(z_n(1), z_n(2), A, B, C, N);
%DF(1:3, 1:2) = DP;
%DF(1:3, 3) = -eta1;
%DF(1:3, 4) = -eta2; 

z_0 = z_n;
DF_0 = DF;
DF_inv_0 = DF_inv;

%Pz_n = evaluate_complexCase(z_n(1), z_n(2), A, B, C, N)';
%Lz_n = L(z_n(3), z_n(4), eta1, eta2, q0);

tolerance_check = tolerance
check_success_1st_NewtonStep = norm(F)

if (norm(F) <= tolerance)
    heteroclinic_Path(1, 1:3) = F1;
    heteroclinic_uParSpace(1, 1:2) = z_n(1:2); 
    heteroclinic_sParSpace(1, 1:2) = z_n(3:4);
end

[U, S, V] = svd(DF);
direction_0 = V(1:4, 4);

pointsFound = 1;
newtonFailed = 0;
F = 1;

%Test Code:
%Vout = V
%Sigma = S
%hp = [0; 0; 0; 1]
%h = V*hp
%DFh = DF*h

%DF
%ProdTest1 = U*S*V
%ProdTest2 = U*S*(V')
testOrientation = 1;
oldDirection = V(1:4, 4);

%==========================================================================
%First Direction:
%==========================================================================

for n = 1:intersectionPoints
  Try = 0;
  %Make sure orientation of kernel basis
  %has not changed since last computation:
  newDirection = V(1:4, 4);
  testOrientation = oldDirection'*newDirection;
  sign = testOrientation/norm(testOrientation);
  newDirection = sign*newDirection;

  %addaptively refine next guess:
  scale = epsilon;
  z_test = z_n + scale*newDirection;
  Pu_test = evaluate_complexCase(z_test(1), z_test(2), Au, Bu, Cu, N)';
  Ps_test = evaluate_complexCase(z_test(3), z_test(4), As, Bs, Cs, N)';
  F1_test = iterate_point_k(Pu_test, K, alpha, tau, a, b, c);
  F2_test = inverse_iterate_point_k(Ps_test, K, alpha, tau, a, b, c);
  F_test = F1_test - F2_test; 
  normTest = norm(F_test);   
  
  whileCounter = 0;
  scaleMin = 0.1*10^(-4);
  while ((norm(F_test) > 0.1*10^(-2)) && (scale >= scaleMin))
      whileCounter = whileCounter + 1;
      whileCounter;
      scale = scale*(0.75);
      z_test = z_n + scale*newDirection;
      Pu_test = evaluate_complexCase(z_test(1), z_test(2), Au, Bu, Cu, N)';
      Ps_test = evaluate_complexCase(z_test(3), z_test(4), As, Bs, Cs, N)';
      F1_test = iterate_point_k(Pu_test, K, alpha, tau, a, b, c);
      F2_test = inverse_iterate_point_k(Ps_test, K, alpha, tau, a, b, c);
      F_test = F1_test - F2_test; 
      normTest = norm(F_test);  
  end    
  
  epsilon;
  scale;
  
  z_n = z_n + scale*newDirection;
  
  Pu = evaluate_complexCase(z_n(1), z_n(2), Au, Bu, Cu, N)';
  Ps = evaluate_complexCase(z_n(3), z_n(4), As, Bs, Cs, N)';
  F1 = iterate_point_k(Pu, K, alpha, tau, a, b, c); 
  F2 = inverse_iterate_point_k(Ps, K, alpha, tau, a, b, c);
  F = F1 - F2;
  DF1 = Dfk_circ_P(K, z_n(1), z_n(2), Au, Bu, Cu, N, alpha, tau, a, b, c);
  DF2 = Dfinvk_circ_P(K, z_n(3), z_n(4), As, Bs, Cs, N, alpha, tau, a, b, c); 
  DF = [DF1, -DF2];
  DF_inv = pinv(DF);      
  
  while ((norm(F) > tolerance) & (Try <= maxTry) & (newtonFailed == 0))
    z_n = z_n - DF_inv*F;
    Try = Try + 1;
   
    Pu = evaluate_complexCase(z_n(1), z_n(2), Au, Bu, Cu, N)';
    Ps = evaluate_complexCase(z_n(3), z_n(4), As, Bs, Cs, N)';
    F1 = iterate_point_k(Pu, K, alpha, tau, a, b, c); 
    F2 = inverse_iterate_point_k(Ps, K, alpha, tau, a, b, c);
    F = F1 - F2;
     
    DF1 = Dfk_circ_P(K, z_n(1), z_n(2), Au, Bu, Cu, N, alpha, tau, a, b, c);
    DF2 = Dfinvk_circ_P(K, z_n(3), z_n(4), As, Bs, Cs, N, alpha, tau, a, b, c); 
    DF = [DF1, -DF2];
    DF_inv = pinv(DF);   
      
  end

  oldDirection = newDirection;   
  [U, S, V] = svd(DF);

  if (norm(F) <= tolerance)
    pointsFound = pointsFound+1;
    heteroclinic_Path(pointsFound, 1:3) = F1;
    heteroclinic_uParSpace(pointsFound, 1:2) = z_n(1:2); 
    heteroclinic_sParSpace(pointsFound, 1:2) = z_n(3:4);
  else
    newtonFailed = 1;
  end
end

direction_1_points = pointsFound

%==========================================================================
%Second Direction:
%==========================================================================


z_n = z_0;
DF = DF_0;
DF_inv = DF_inv_0;

testOrientation = 1;
oldDirection = -direction_0;
V(1:4, 4) = -direction_0;

for n = 1:intersectionPoints
  Try = 0;
  %Make sure orientation of kernel basis
  %has not changed since last computation:
  newDirection = V(1:4, 4);
  testOrientation = oldDirection'*newDirection;
  sign = testOrientation/norm(testOrientation);
  newDirection = sign*newDirection;

  %addaptively refine next guess:
  scale = epsilon;
  z_test = z_n + scale*newDirection;
  Pu_test = evaluate_complexCase(z_test(1), z_test(2), Au, Bu, Cu, N)';
  Ps_test = evaluate_complexCase(z_test(3), z_test(4), As, Bs, Cs, N)';
  F1_test = iterate_point_k(Pu_test, K, alpha, tau, a, b, c);
  F2_test = inverse_iterate_point_k(Ps_test, K, alpha, tau, a, b, c);
  F_test = F1_test - F2_test; 
  normTest = norm(F_test);   
  
  whileCounter = 0;
  scaleMin = 0.1*10^(-4);
  while ((norm(F_test) > 0.1*10^(-2)) && (scale >= scaleMin))
      whileCounter = whileCounter + 1;
      whileCounter;
      scale = scale*(0.75);
      z_test = z_n + scale*newDirection;
      Pu_test = evaluate_complexCase(z_test(1), z_test(2), Au, Bu, Cu, N)';
      Ps_test = evaluate_complexCase(z_test(3), z_test(4), As, Bs, Cs, N)';
      F1_test = iterate_point_k(Pu_test, K, alpha, tau, a, b, c);
      F2_test = inverse_iterate_point_k(Ps_test, K, alpha, tau, a, b, c);
      F_test = F1_test - F2_test; 
      normTest = norm(F_test);  
  end    
  
  epsilon;
  scale;
  
  z_n = z_n + scale*newDirection;
  
  Pu = evaluate_complexCase(z_n(1), z_n(2), Au, Bu, Cu, N)';
  Ps = evaluate_complexCase(z_n(3), z_n(4), As, Bs, Cs, N)';
  F1 = iterate_point_k(Pu, K, alpha, tau, a, b, c); 
  F2 = inverse_iterate_point_k(Ps, K, alpha, tau, a, b, c);
  F = F1 - F2;
  DF1 = Dfk_circ_P(K, z_n(1), z_n(2), Au, Bu, Cu, N, alpha, tau, a, b, c);
  DF2 = Dfinvk_circ_P(K, z_n(3), z_n(4), As, Bs, Cs, N, alpha, tau, a, b, c); 
  DF = [DF1, -DF2];
  DF_inv = pinv(DF);      
  
  while ((norm(F) > tolerance) & (Try <= maxTry) & (newtonFailed == 0))
    z_n = z_n - DF_inv*F;
    Try = Try + 1;
   
    Pu = evaluate_complexCase(z_n(1), z_n(2), Au, Bu, Cu, N)';
    Ps = evaluate_complexCase(z_n(3), z_n(4), As, Bs, Cs, N)';
    F1 = iterate_point_k(Pu, K, alpha, tau, a, b, c); 
    F2 = inverse_iterate_point_k(Ps, K, alpha, tau, a, b, c);
    F = F1 - F2;
     
    DF1 = Dfk_circ_P(K, z_n(1), z_n(2), Au, Bu, Cu, N, alpha, tau, a, b, c);
    DF2 = Dfinvk_circ_P(K, z_n(3), z_n(4), As, Bs, Cs, N, alpha, tau, a, b, c); 
    DF = [DF1, -DF2];
    DF_inv = pinv(DF);   
      
  end

  oldDirection = newDirection;   
  [U, S, V] = svd(DF);

  if (norm(F) <= tolerance)
    pointsFound = pointsFound+1;
    heteroclinic_Path(pointsFound, 1:3) = F1;
    heteroclinic_uParSpace(pointsFound, 1:2) = z_n(1:2); 
    heteroclinic_sParSpace(pointsFound, 1:2) = z_n(3:4);
  else
    newtonFailed = 1;
  end
end


%==========================================================================
%Return Results:
%==========================================================================


pointsFound

%Keep only points found and not trailing zeros:
Path = heteroclinic_Path(1:pointsFound, 1:3);
uParSpace = heteroclinic_uParSpace(1:pointsFound, 1:2);
sParSpace = heteroclinic_sParSpace(1:pointsFound, 1:2);

%Return:
varargout(1,1)={Path};
varargout(2,1)={uParSpace};
varargout(3,1)={sParSpace};




