function varargout=findStableParameters(stableGuesses, ...
    Ds, Ws_loc, lambda_s, alpha, tau, a, b, c);


temp = size(stableGuesses);
numGuesses = temp(1);
dim = temp(2);

epsilon = 10^(-12);

for k = 1:numGuesses
  numIterates = 0;  
  success = 0;
  x = stableGuesses(k, 1:dim);
  while (success < 0.5)
    [minDist, minPoint, minPointIndex] = distPointToSet(x, Ws_loc);
    if (minDist < epsilon)
      stableIterates(k) = numIterates;        
      stableParameters(k, 1:2) = Ds(minPointIndex, 1:2);      
      success = 1;
    else
       x = henon3D(x, alpha, tau, a, b, c)'; 
       numIterates = numIterates + 1;
    end
  end
end


stableIterates = stableIterates + 1;

for k = 1:numGuesses
   s = stableParameters(k, 1);
   t = stableParameters(k, 2);
   z = s+i*t;
   z_prime = lambda_s*z;
   s_prime = real(z_prime);
   t_prime = imag(z_prime);
   stableParameters(k, 1) = s_prime;
   stableParameters(k, 2) = t_prime;
end

rawParameters = stableParameters;

maxIteration = max(stableIterates);

for k = 1:numGuesses
   thisIterate = stableIterates(k);
   fixIterate = maxIteration - thisIterate;
   
   s = stableParameters(k, 1);
   t = stableParameters(k, 2);
   z = s+i*t;
   z_prime = (lambda_s)^(fixIterate)*z;
   s_prime = real(z_prime);
   t_prime = imag(z_prime);
   stableParameters(k, 1) = s_prime;
   stableParameters(k, 2) = t_prime;
end    

       
varargout(1,1)={stableParameters};
varargout(2,1)={stableIterates};
varargout(3,1)= {maxIteration};
varargout(4,1)= {rawParameters};

