function varargout=findUnstableParameters(unstableGuesses, ...
    Du, Wu_loc, lambda_u, alpha, tau, a, b, c);


temp = size(unstableGuesses);
numGuesses = temp(1);
dim = temp(2);

epsilon = 10^(-12);

for k = 1:numGuesses
  numIterates = 0;  
  success = 0;
  x = unstableGuesses(k, 1:dim);
  while (success < 0.5)
    [minDist, minPoint, minPointIndex] = distPointToSet(x, Wu_loc);
    if (minDist < epsilon)
      unstableIterates(k) = numIterates;        
      unstableParameters(k, 1:2) = Du(minPointIndex, 1:2);      
      success = 1;    
    else
       x = inv_henon3D(x, alpha, tau, a, b, c)'; 
       numIterates = numIterates + 1;
    end
  end
end

unstableIterates = unstableIterates + 1;

for k = 1:numGuesses
   s = unstableParameters(k, 1);
   t = unstableParameters(k, 2);
   z = s+i*t;
   z_prime = (lambda_u)^(-1)*z;
   s_prime = real(z_prime);
   t_prime = imag(z_prime);
   unstableParameters(k, 1) = s_prime;
   unstableParameters(k, 2) = t_prime;
end    

rawParameters = unstableParameters;

maxIteration = max(unstableIterates);

for k = 1:numGuesses
   thisIterate = unstableIterates(k);
   fixIterate = maxIteration - thisIterate;
   
   s = unstableParameters(k, 1);
   t = unstableParameters(k, 2);
   z = s+i*t;
   z_prime = (lambda_u)^(-fixIterate)*z;
   s_prime = real(z_prime);
   t_prime = imag(z_prime);
   unstableParameters(k, 1) = s_prime;
   unstableParameters(k, 2) = t_prime;
end    

varargout(1,1)={unstableParameters};
varargout(2,1)={unstableIterates};
varargout(3,1)= {maxIteration};
varargout(4,1)={rawParameters};

