function varargout = firstOrderData_3DHenon(a, b, c, alpha, tau);


x0 = (1/2)*(-tau + sqrt( tau^2 - 4*alpha));
x1 = (1/2)*(-tau - sqrt( tau^2 - 4*alpha));

p_plus = [x0; x0; x0];
p_minus = [x1; x1; x1];


%Differential at p_plus;

A_0 = (2*a + b)*x0 + tau;
B_0 = (2*c + b)*x0;

Df0 = [ A_0, B_0, 1;
          1,   0, 0;
          0,   1, 0];
    
%Differential of p_minus;

A_1 = (2*a + b)*x1 + tau;
B_1 = (2*c + b)*x1;

Df1 = [ A_1, B_1, 1;
          1,   0, 0;
          0,   1, 0];
    
%Eigenvalues and vectors of Df_plus:

[E_plus, eigenvalues_plus] = eigs(Df0);

%Eigenvalues and vectors of Df_minus:

[E_minus, eigenvalues_minus] = eigs(Df1);


%==========================================================================
%Assign Correct Names:
%==========================================================================


%fprintf(fid, '%s\n', 'union {');
%  for n = 1:numPoints
% 		outstr = [ '    sphere{ <', num2str(dataCloud(n,1)), ',', num2str(dataCloud(n,2)), ',', ...
%                     num2str(dataCloud(n,3)), '> ' 'RAD' '}' ];
% 		fprintf(fid, '%s \n', outstr);
% 		if mod(n,1000) == 0; disp([num2str(n) '	'	outstr]), end
%  end




%stopHere = stopNow


if ((isreal(eigenvalues_plus(1,1))) & (isreal(eigenvalues_minus(3,3))) & ... 
      (~isreal(eigenvalues_plus(2,2))) & (~isreal(eigenvalues_plus(3,3))) &...
      (~isreal(eigenvalues_minus(1,1))) & (~isreal(eigenvalues_minus(2,2))))
      p0 = p_minus;
      lambda0_s = eigenvalues_minus(3,3);
      lambda0_u1 = eigenvalues_minus(1,1);
      lambda0_u2 = eigenvalues_minus(2,2);
      xi0_s = E_minus(:, 3);
      xi0_u1 = E_minus(:, 1);
      xi0_u2 = E_minus(:, 2);
      
      p1 = p_plus;
      lambda1_u = eigenvalues_plus(1,1);
      lambda1_s1 = eigenvalues_plus(2,2);
      lambda1_s2 = eigenvalues_plus(3,3);
      xi1_u = E_plus(:, 1);
      xi1_s1 = E_plus(:, 2);
      xi1_s2 = E_plus(:, 3);
      
elseif ((isreal(eigenvalues_plus(3,3))) & (isreal(eigenvalues_minus(1,1))) & ... 
      (~isreal(eigenvalues_plus(1,1))) & (~isreal(eigenvalues_plus(2,2))) & ...
      (~isreal(eigenvalues_minus(2,2))) & (~isreal(eigenvalues_minus(3,3))))
      p0 = p_plus;
      lambda0_s = eigenvalues_plus(3,3);
      lambda0_u1 = eigenvalues_plus(1,1);
      lambda0_u2 = eigenvalues_plus(2,2);
      xi0_s = E_plus(:, 3);
      xi0_u1 = E_plus(:, 1);
      xi0_u2 = E_plus(:, 2);
      
      p1 = p_minus;
      lambda1_u = eigenvalues_minus(1,1);
      lambda1_s1 = eigenvalues_minus(2,2);
      lambda1_s2 = eigenvalues_minus(3,3);
      xi1_u = E_minus(:, 1);
      xi1_s1 = E_minus(:, 2);
      xi1_s2 = E_minus(:, 3);
else
    outstr = [  'problem with first order data: not conforming to naming convention...' ];
    disp([outstr]);
end


%==========================================================================
%Return Values:
%==========================================================================

varargout(1,1)={p0};
varargout(2,1)={p1};
varargout(3,1)={lambda0_s};
varargout(4,1)={lambda0_u1};
varargout(5,1)={lambda0_u2};
varargout(6,1)={lambda1_u};
varargout(7,1)={lambda1_s1};
varargout(8,1)={lambda1_s2};
varargout(9,1)={xi0_s};
varargout(10,1)={xi0_u1};
varargout(11,1)={xi0_u2};
varargout(12,1)={xi1_u};
varargout(13,1)={xi1_s1};
varargout(14,1)={xi1_s2};





