function varargout = firstOrderData_6DHenon(a1, b1, c1, alpha1, tau1, ...
                                            a2, b2, c2, alpha2, tau2, ...
                                            epsilon);

                                            
x1_plus =  (1/2)*(-tau1 + sqrt( tau1^2 - 4*alpha1));
x1_minus = (1/2)*(-tau1 - sqrt( tau1^2 - 4*alpha1));

x2_plus =  (1/2)*(-tau2 + sqrt( tau2^2 - 4*alpha2));
x2_minus = (1/2)*(-tau2 - sqrt( tau2^2 - 4*alpha2));
      


p_plus1 =  [x1_plus; x1_plus; x1_plus];
p_minus1 = [x1_minus; x1_minus; x1_minus];

p_plus2 =  [x2_plus; x2_plus; x2_plus];
p_minus2 = [x2_minus; x2_minus; x2_minus];

      
p_plus = [p_plus1; p_plus2];
p_minus = [p_minus1; p_minus2];

 
DF_plus = henon6D_differential(p_plus, alpha1, tau1, a1, b1, c1, ...
                                       alpha2, tau2, a2, b2, c2, epsilon);   
   
DF_minus = henon6D_differential(p_minus, alpha1, tau1, a1, b1, c1, ...
                                         alpha2, tau2, a2, b2, c2, epsilon);   
   
   
%Eigenvalues and vectors of Df_plus:

[E_plus, eigenvalues_plus] = eigs(DF_plus);

%Eigenvalues and vectors of Df_minus:

[E_minus, eigenvalues_minus] = eigs(DF_minus);




%==========================================================================
%Assign Correct Names:
%==========================================================================




if ((isreal(eigenvalues_plus(1,1))) & (isreal(eigenvalues_minus(6,6))))
      p0 = p_minus;
      lambda0_s1 = eigenvalues_minus(5,5);
      lambda0_s2 = eigenvalues_minus(6,6);
      lambda0_u1 = eigenvalues_minus(1,1);
      lambda0_u2 = eigenvalues_minus(2,2);
      lambda0_u3 = eigenvalues_minus(3,3);
      lambda0_u4 = eigenvalues_minus(4,4);
      xi0_s1 = E_minus(:, 5);
      xi0_s2 = E_minus(:, 6);
      xi0_u1 = E_minus(:, 1);
      xi0_u2 = E_minus(:, 2);
      xi0_u3 = E_minus(:, 3);
      xi0_u4 = E_minus(:, 4);
      
      p1 = p_plus;
      lambda1_u1 = eigenvalues_plus(1,1);
      lambda1_u2 = eigenvalues_plus(2,2);
      lambda1_s1 = eigenvalues_plus(3,3);
      lambda1_s2 = eigenvalues_plus(4,4);
      lambda1_s3 = eigenvalues_plus(5,5);
      lambda1_s4 = eigenvalues_plus(6,6);
      xi1_u1 = E_plus(:, 1);
      xi1_u2 = E_plus(:, 2);
      xi1_s1 = E_plus(:, 3);
      xi1_s2 = E_plus(:, 4);
      xi1_s3 = E_plus(:, 5);
      xi1_s4 = E_plus(:, 6);
      
elseif ((isreal(eigenvalues_plus(6,6))) & (isreal(eigenvalues_minus(1,1))))
      p0 = p_plus;
      lambda0_s1 = eigenvalues_plus(5,5);
      lambda0_s2 = eigenvalues_plus(6,6);
      lambda0_u1 = eigenvalues_plus(1,1);
      lambda0_u2 = eigenvalues_plus(2,2);
      lambda0_u3 = eigenvalues_plus(3,3);
      lambda0_u4 = eigenvalues_plus(4,4);
      xi0_s1 = E_plus(:, 5);
      xi0_s2 = E_plus(:, 6);
      xi0_u1 = E_plus(:, 1);
      xi0_u2 = E_plus(:, 2);
      xi0_u3 = E_plus(:, 3);
      xi0_u4 = E_plus(:, 4);
      
      p1 = p_minus;
      lambda1_u1 = eigenvalues_minus(1,1);
      lambda1_u2 = eigenvalues_minus(2,2);
      lambda1_s1 = eigenvalues_minus(3,3);
      lambda1_s2 = eigenvalues_minus(4,4);
      lambda1_s3 = eigenvalues_minus(5,5);
      lambda1_s4 = eigenvalues_minus(6,6);
      xi1_u1 = E_minus(:, 1);
      xi1_u2 = E_minus(:, 2);
      xi1_s1 = E_minus(:, 3);
      xi1_s2 = E_minus(:, 4);
      xi1_s3 = E_minus(:, 5);
      xi1_s4 = E_minus(:, 6);
else
    outstr = [  'problem with first order data: not conforming to naming convention...' ];
    disp([outstr]);
end


%==========================================================================
%Return Values:
%==========================================================================

varargout(1,1)={p0};
varargout(2,1)={p1};
varargout(3,1)={lambda0_s1};
varargout(4,1)={lambda0_s2}
varargout(5,1)={lambda0_u1};
varargout(6,1)={lambda0_u2};
varargout(7,1)={lambda0_u3};
varargout(8,1)={lambda0_u4};
varargout(9,1)={lambda1_u1};
varargout(10,1)={lambda1_u2};
varargout(11,1)={lambda1_s1};
varargout(12,1)={lambda1_s2};
varargout(13,1)={lambda1_s3};
varargout(14,1)={lambda1_s4};
varargout(15,1)={xi0_s1};
varargout(16,1)={xi0_s2}
varargout(17,1)={xi0_u1};
varargout(18,1)={xi0_u2};
varargout(19,1)={xi0_u3};
varargout(20,1)={xi0_u4};
varargout(21,1)={xi1_u1};
varargout(22,1)={xi1_u2};
varargout(23,1)={xi1_s1};
varargout(24,1)={xi1_s2};
varargout(25,1)={xi1_s3};
varargout(26,1)={xi1_s4};


