function varargout = firstOrderData_intval_3DHenon(a, b, c, alpha, tau);


x0 = (1/2)*(-tau + sqrt( tau^2 - 4*alpha));
x1 = (1/2)*(-tau - sqrt( tau^2 - 4*alpha));

p_plus = [x0; x0; x0];
p_minus = [x1; x1; x1];


%Differential at p_plus;

A_0 = (2*a + b)*x0 + tau;
B_0 = (2*c + b)*x0;

Df0 = [ A_0, B_0, 1;
          1,   0, 0;
          0,   1, 0];
    
%Differential of p_minus;

A_1 = (2*a + b)*x1 + tau;
B_1 = (2*c + b)*x1;

Df1 = [ A_1, B_1, 1;
          1,   0, 0;
          0,   1, 0];
    
%Eigenvalues and vectors of Df_plus:

[E_plus_temp, eigenvalues_plus_temp] = eigs(Df0.mid);

%Verify eigen data plus:
[lambda1_p, xi1_p] = verifyeig(Df0, eigenvalues_plus_temp(1,1), E_plus_temp(:,1));
[lambda2_p, xi2_p] = verifyeig(Df0, eigenvalues_plus_temp(2,2), E_plus_temp(:,2));
[lambda3_p, xi3_p] = verifyeig(Df0, eigenvalues_plus_temp(3,3), E_plus_temp(:,3));

             
%Eigenvalues and vectors of Df_minus:

[E_minus_temp, eigenvalues_minus_temp] = eigs(Df1.mid);

%Verify eigen data minus:
[lambda1_m, xi1_m] = verifyeig(Df1, eigenvalues_minus_temp(1,1), E_minus_temp(:,1));
[lambda2_m, xi2_m] = verifyeig(Df1, eigenvalues_minus_temp(2,2), E_minus_temp(:,2));
[lambda3_m, xi3_m] = verifyeig(Df1, eigenvalues_minus_temp(3,3), E_minus_temp(:,3));


%==========================================================================
%Assign Correct Names:
%==========================================================================


if ((isreal(lambda1_p)) & (isreal(lambda3_m)) & ... 
      (~isreal(lambda2_p)) & (~isreal(lambda3_p)) &...
      (~isreal(lambda1_m)) & (~isreal(lambda2_m)))
      p0 = p_minus;
      lambda0_s = lambda3_m;
      lambda0_u1 = lambda1_m;
      lambda0_u2 = lambda2_m;
      xi0_s = xi3_m;
      xi0_u1 = xi1_m;
      xi0_u2 = xi2_m;
      
      p1 = p_plus;
      lambda1_u = lambda1_p;
      lambda1_s1 = lambda2_p;
      lambda1_s2 = lambda3_p;
      xi1_u = xi1_p;
      xi1_s1 = xi2_p;
      xi1_s2 = xi3_p;
      
elseif ((isreal(lambda3_p)) & (isreal(lambda1_m)) & ... 
      (~isreal(lambda1_p)) & (~isreal(lambda2_p)) & ...
      (~isreal(lambda2_m)) & (~isreal(lambda3_m)))
      p0 = p_plus;
      lambda0_s = lambda3_p;
      lambda0_u1 = lambda1_p;
      lambda0_u2 = lambda2_p;
      xi0_s = xi3_p;
      xi0_u1 = xi1_p;
      xi0_u2 = xi2_p;
      
      p1 = p_minus;
      lambda1_u = lambda1_m;
      lambda1_s1 = lambda2_m;
      lambda1_s2 = lambda3_m;
      xi1_u = xi1_m;
      xi1_s1 = xi2_m;
      xi1_s2 = xi3_m;
else
    outstr = [  'problem with first order data: not conforming to naming convention...' ];
    disp([outstr]);
end


%==========================================================================
%Return Values:
%==========================================================================

varargout(1,1)={p0};
varargout(2,1)={p1};
varargout(3,1)={lambda0_s};
varargout(4,1)={lambda0_u1};
varargout(5,1)={lambda0_u2};
varargout(6,1)={lambda1_u};
varargout(7,1)={lambda1_s1};
varargout(8,1)={lambda1_s2};
varargout(9,1)={xi0_s};
varargout(10,1)={xi0_u1};
varargout(11,1)={xi0_u2};
varargout(12,1)={xi1_u};
varargout(13,1)={xi1_s1};
varargout(14,1)={xi1_s2};





