function varargout = homoclinicPoint6D_p0_prodSpace(...
    theta_0, tau_0, n_u, n_s, ...
    A0u, B0u, C0u, D0u, E0u, F0u, ...
    A0s, B0s, C0s, D0s, E0s, F0s, ...
    N, order_1, ...
    alpha1, tau1, a1, b1, c1, ...
    alpha2, tau2, a2, b2, c2, ...
    epsilon)



K = n_u + n_s;
Kmid = ceil(K/2);

xn = zeros(6*(K-1), 1);
this_x = evaluate4D_complexCase(theta_0(1), theta_0(2), ...
                                theta_0(3), theta_0(4), ...
                                A0u, B0u, C0u, D0u, E0u, F0u, N);
                            
this_y = evaluate_2D(tau_0(1), tau_0(2), ...
                     A0s, B0s, C0s, D0s, E0s, F0s, order_1);
                 
      
for j = 1:Kmid 
   this_orbit = henon6D(this_x, alpha1, tau1, a1, b1, c1, ...
                                alpha2, tau2, a2, b2, c2, ...
                                epsilon);
                            
   xn((6*j-5):(6*j)) = this_orbit;
   this_x = this_orbit;
end

thisIndex = 0;
for j = (Kmid+1):(K-1)
   this_orbit = inv_henon6D(this_y, alpha1, tau1, a1, b1, c1, ...
                                    alpha2, tau2, a2, b2, c2, epsilon);
                                
   xn((6*(K-1) - 6*thisIndex - 5):(6*(K-1) - 6*thisIndex)) = this_orbit;
   this_y = this_orbit;
   thisIndex = thisIndex + 1;
end

theta_n = [theta_0(1);
           theta_0(2);
           theta_0(3);
           theta_0(4)];
       
tau_n = [tau_0(1);
         tau_0(2)];
     
x = zeros(6*K, 1);
Fx = x;
DFx = zeros(6*K, 6*K);

thisError = inf;
numLoops = 0;   

%Base error calculation


while ((thisError > 10^(-13)) & numLoops < 200)
 
    
 numLoops = numLoops + 1
 
 %Image of theta_n under Ps:
 %Ps_xn = [horner(tau_n, A0s, order_1);
 %         horner(tau_n, B0s, order_1);
 %         horner(tau_n, C0s, order_1)];
 
 Ps_xn = evaluate_2D(tau_n(1), tau_n(2), ...
                     A0s, B0s, C0s, D0s, E0s, F0s, order_1);
 
 
 %Image of z_n = (s_n, t_n) Pu:
 %Pu_zn = evaluate_complexCase(theta_n(1), theta_n(2), A0u, B0u, C0u, N);
 
 Pu_zn = evaluate4D_complexCase(theta_n(1), theta_n(2), ...
                                theta_n(3), theta_n(4), ...
                                A0u, B0u, C0u, D0u, E0u, F0u, N);
 
                            
 %Differential of Ps(theta_n):
 %DPs_tau = evaluate_differential_1D(tau_n, A0s, B0s, C0s, order_1);
 
 DPs_tau = evaluate_differential_2D(tau_n, A0s, B0s, C0s, ...
                                           D0s, E0s, F0s, order_1);
 
 %Differential of Pu(z_n):
 %DPu_theta = evaluate_jacob_complexCase(theta_n(1), theta_n(2), A0u, B0u, C0u, N)
 
 DPu_theta = evaluate_jacob_complexCase4D(theta_n, A0u, B0u, C0u, ...
                                                   D0u, E0u, F0u, N)
                                              
 %stopHere = stopNow                                             
                                               
 x(1) = theta_n(1);
 x(2) = theta_n(2);
 x(3) = theta_n(3);
 x(4) = theta_n(4);
 x(5:(6*K-2)) = xn;
 x(6*K-1) = tau_n(1);
 x(6*K) = tau_n(2);
 
 Fx(1:6) = inv_henon6D(xn(1:6), ...
                       alpha1, tau1, a1, b1, c1, ...
                       alpha2, tau2, a2, b2, c2, epsilon) - Pu_zn';
 
 DFx = zeros(6*K, 6*K);
 DFx(1:6, 1:4) = -DPu_theta;
 DFx(1:6, 5:10) = inv_henon6D_differential(xn(1:6), ...
                                           alpha1, tau1, a1, b1, c1, ...
                                           alpha2, tau2, a2, b2, c2, ...
                                           epsilon);
 
 
 Id = eye(6);
 
 for j = 2:Kmid
    Fx((6*j - 5):(6*j)) = ...
          inv_henon6D(xn((6*j-5):(6*j)), ...
                      alpha1, tau1, a1, b1, c1, ...
                      alpha2, tau2, a2, b2, c1, ...
                      epsilon) ...
                      - xn((6*j-11):(6*j-6));
                  
    DFx((6*j - 5):(6*j), (6*j-7):(6*j-2)) = -Id;
    
    DFx((6*j - 5):(6*j), (6*j-1):(6*j+4)) = ...
        inv_henon6D_differential(xn((6*j-5):(6*j)), ...
                                 alpha1, tau1, a1, b1, c1, ...
                                 alpha2, tau2, a2, b2, c2, ...
                                 epsilon);
 end
 for j = (Kmid+1):(K-1)
    Fx((6*j - 5):(6*j)) = ...
        henon6D(xn((6*(j-1)-5):(6*(j-1))), ...
                alpha1, tau1, a1, b1, c1, ...
                alpha2, tau2, a2, b2, c2, ...
                epsilon) ...
                - xn((6*j - 5):(6*j));
            
    DFx((6*j-5):(6*j), (6*j-7):(6*j-2)) = ...
        henon6D_differential(xn((6*(j-1)-5):(6*(j-1))), ...
                             alpha1, tau1, a1, b1, c1, ...
                             alpha2, tau2, a2, b2, c2, ...
                             epsilon);
    
    DFx((6*j-5):(6*j), (6*j-1):(6*j+4)) = -Id;
 end
 
 
 Fx((6*K - 5):(6*K)) = henon6D(xn((6*(K-1) - 5):(6*(K-1))), ...
                               alpha1, tau1, a1, b1, c1, ...
                               alpha2, tau2, a2, b2, c2, ...
                               epsilon) ...
                               -  Ps_xn';
                                                   
 DFx((6*K - 5):(6*K), (6*K-7):(6*K-2)) = ...
     henon6D_differential(xn((6*(K-1) - 5):(6*(K-1))), ...
                          alpha1, tau1, a1, b1, c1, ...
                          alpha2, tau2, a2, b2, c2, ...
                          epsilon);
                      
 DFx((6*K - 5):(6*K), (6*K-1):(6*K)) = -DPs_tau;       
 
 

 %Compute the Newton Step:
 xnew = x - inv(DFx)*Fx;
 
 theta_n = xnew(1:4);
 xn = xnew(5:(6*K-2));
 tau_n = xnew((6*K-1):(6*K));
 
 
                     
%Check Error:

Ps_xn = evaluate_2D(tau_n(1), tau_n(2), ...
                     A0s, B0s, C0s, D0s, E0s, F0s, order_1);
        
  
 %Image of z_n = (s_n, t_n) Pu:
 Pu_zn = evaluate4D_complexCase(theta_n(1), theta_n(2), ...
                                theta_n(3), theta_n(4), ...
                                A0u, B0u, C0u, D0u, E0u, F0u, N);
                           
                            
                            
                            
 Fx_new(1:6) = inv_henon6D(xn(1:6), alpha1, tau1, a1, b1, c1, ...
                                    alpha2, tau2, a2, b2, c2, ...
                                    epsilon) - Pu_zn';

                                
for j = 2:Kmid
    Fx_new((6*j - 5):(6*j)) = ...
          inv_henon6D(xn((6*j-5):(6*j)), alpha1, tau1, a1, b1, c1, ...
                                         alpha2, tau2, a2, b2, c2, ...
                                         epsilon) - xn((6*j-11):(6*j-6)); 
 end
 for j = (Kmid+1):(K-1)
    Fx_new((6*j - 5):(6*j)) = ...
        henon6D(xn((6*(j-1)-5):(6*(j-1))), alpha1, tau1, a1, b1, c1, ...
                                           alpha2, tau2, a2, b2, c2, ...
                                           epsilon) - xn((6*j - 5):(6*j)); ;
 end
 Fx_new((6*K - 5):(6*K)) = henon6D(xn((6*(K-1) - 5):(6*(K-1))), ...
                                   alpha1, tau1, a1, b1, c1, ...
                                   alpha2, tau2, a2, b2, c2, ...
                                   epsilon) -  Ps_xn';
 
 
 %lastError = thisError
 thisError = norm(Fx_new, inf)
end
 

varargout(1,1)={theta_n};
varargout(2,1)= {xn};
varargout(3,1)={tau_n};
varargout(4,1) = {thisError};
