function varargout = homoclinicPoint_p0_prodSpace(theta_0, tau_0, n_u, n_s, ...
    A0u, B0u, C0u, A0s, B0s, C0s, N, order_1, alpha, tau, a, b, c)


%TEST CODE:
%[Pu1, Pu2, Pu3, Ps1g, Ps2g, Ps3g] = convertToPoly_2D(A0u, B0u, C0u, ...
%                                                     A0u, B0u, C0u, N);



%[Ps1, Ps2, Ps3, Pu1g, Pu2g, Pu3g] = convertToPoly_1D(A0s, B0s, C0s, ...
%                                                   A0s, B0s, C0s, order_1);




K = n_u + n_s;
Kmid = ceil(K/2);

xn = zeros(3*(K-1), 1);
this_x = evaluate_complexCase(theta_0(1), theta_0(2), A0u, B0u, C0u, N); 
this_y = [horner(tau_0, A0s, order_1);
          horner(tau_0, B0s, order_1);
          horner(tau_0, C0s, order_1)];
 
      

for j = 1:Kmid 
   this_orbit = henon3D(this_x, alpha, tau, a, b, c);
   xn((3*j-2):(3*j)) = this_orbit;
   this_x = this_orbit;
end

thisIndex = 0;
for j = (Kmid+1):(K-1)
   this_orbit = inv_henon3D(this_y, alpha, tau, a, b, c);
   xn((3*(K-1) - 3*thisIndex - 2):(3*(K-1) - 3*thisIndex)) = this_orbit;
   this_y = this_orbit;
   thisIndex = thisIndex + 1;
end

theta_n = [theta_0(1);
           theta_0(2)];
tau_n = tau_0;
x = zeros(3*K, 1);
Fx = x;
DFx = zeros(3*K, 3*K);

thisError = inf;
numLoops = 0;   

%Base error calculation




while ((thisError > 10^(-13)) & numLoops < 25)
 
    
 numLoops = numLoops + 1
 
 %Image of theta_n under Ps:
 Ps_xn = [horner(tau_n, A0s, order_1);
          horner(tau_n, B0s, order_1);
          horner(tau_n, C0s, order_1)];
 %Image of z_n = (s_n, t_n) Pu:
 Pu_zn = evaluate_complexCase(theta_n(1), theta_n(2), A0u, B0u, C0u, N);
 
 
 %Differential of Ps(theta_n):
 DPs_tau = evaluate_differential_1D(tau_n, A0s, B0s, C0s, order_1);
 %Differential of Pu(z_n):
 DPu_theta = evaluate_jacob_complexCase(theta_n(1), theta_n(2), A0u, B0u, C0u, N)
 
 
 x(1) = theta_n(1);
 x(2) = theta_n(2);
 x(3:(3*K-1)) = xn;
 x(3*K) = tau_n; 
 
 Fx(1:3) = inv_henon3D(xn(1:3), alpha, tau, a, b, c) - Pu_zn';
 
 DFx = zeros(3*K, 3*K);
 DFx(1:3, 1:2) = -DPu_theta;
 DFx(1:3, 3:5) = inv_henon3D_differential(xn(1:3), alpha, tau, a, b, c);
 
 
 Id = eye(3);
 for j = 2:Kmid
    Fx((3*j - 2):(3*j)) = ...
          inv_henon3D(xn((3*j-2):(3*j)), alpha, tau, a, b, c) - xn((3*j-5):(3*j-3)); 
    DFx((3*j - 2):(3*j), (3*j-3):(3*j-1)) = -Id;
    DFx((3*j - 2):(3*j), (3*j):(3*j+2)) = ...
        inv_henon3D_differential(xn((3*j-2):(3*j)), alpha, tau, a, b, c);
 end
 for j = (Kmid+1):(K-1)
    Fx((3*j - 2):(3*j)) = ...
        henon3D(xn((3*(j-1)-2):(3*(j-1))), alpha, tau, a, b, c) - xn((3*j - 2):(3*j));
    DFx((3*j-2):(3*j), (3*j-3):(3*j-1)) = ...
        henon3D_differential(xn((3*(j-1)-2):(3*(j-1))) , alpha, tau, a, b, c);
    DFx((3*j-2):(3*j), (3*j):(3*j+2)) = -Id;
 end
 
 Fx((3*K - 2):(3*K)) = henon3D(xn((3*(K-1) - 2):(3*(K-1))), alpha, tau, a, b, c) -  Ps_xn;
 
 DFx((3*K - 2):(3*K), (3*K-3):(3*K-1)) = ...
     henon3D_differential(xn((3*(K-1) - 2):(3*(K-1))) , alpha, tau, a, b, c);
 DFx((3*K - 2):(3*K), 3*K) = -DPs_tau;       
 
 

 %Compute the Newton Step:
 xnew = x - inv(DFx)*Fx;
 
 theta_n = xnew(1:2);
 xn = xnew(3:(3*K-1));
 tau_n = xnew(3*K);
 
 
 %Check Error:
 Ps_xn = [horner(tau_n, A0s, order_1);
          horner(tau_n, B0s, order_1);
          horner(tau_n, C0s, order_1)];
 %Image of z_n = (s_n, t_n) Pu:
 Pu_zn = evaluate_complexCase(theta_n(1), theta_n(2), A0u, B0u, C0u, N);
 Fx_new(1:3) = inv_henon3D(xn(1:3), alpha, tau, a, b, c) - Pu_zn';
 for j = 2:Kmid
    Fx_new((3*j - 2):(3*j)) = ...
          inv_henon3D(xn((3*j-2):(3*j)), alpha, tau, a, b, c) - xn((3*j-5):(3*j-3)); 
 end
 for j = (Kmid+1):(K-1)
    Fx_new((3*j - 2):(3*j)) = ...
        henon3D(xn((3*(j-1)-2):(3*(j-1))), alpha, tau, a, b, c) - xn((3*j - 2):(3*j)); ;
 end
 Fx_new((3*K - 2):(3*K)) = henon3D(xn((3*(K-1) - 2):(3*(K-1))), alpha, tau, a, b, c) -  Ps_xn;
 
 
 %lastError = thisError
 thisError = norm(Fx_new, inf)
end
 

varargout(1,1)={theta_n};
varargout(2,1)= {xn};
varargout(3,1)={tau_n};
varargout(4,1) = {thisError};













