function varargout=inverseDF_heteroclinic(N, K, alpha, beta, alphaBar, betaBar, Pu1, Pu2, Pu3, Ps1, Ps2, Ps3);


%initialize return variables
alpha_inv = zeros(K+1, 1);
beta_inv = zeros(K+1, 1);
alphaBar_inv = zeros(K+1, 1);
betaBar_inv = zeros(K+1, 1);

%Partials:
[Pu1_s] = partial_in_s_complexVersion(N, Pu1);
[Pu2_s] = partial_in_s_complexVersion(N, Pu2);
[Pu3_s] = partial_in_s_complexVersion(N, Pu3);
[Ps1_s] = partial_in_s_complexVersion(N, Ps1);
[Ps2_s] = partial_in_s_complexVersion(N, Ps2);
[Ps3_s] = partial_in_s_complexVersion(N, Ps3);

[Pu1_t] = partial_in_t_complexVersion(N, Pu1);
[Pu2_t] = partial_in_t_complexVersion(N, Pu2);
[Pu3_t] = partial_in_t_complexVersion(N, Pu3);
[Ps1_t] = partial_in_t_complexVersion(N, Ps1);
[Ps2_t] = partial_in_t_complexVersion(N, Ps2);
[Ps3_t] = partial_in_t_complexVersion(N, Ps3);


%Composition of Partials With gamma:
[Pu1_s_at_gamma, Pu2_s_at_gamma, Pu3_s_at_gamma] = P_composition_gamma(N, K, Pu1_s, Pu2_s, Pu3_s, alpha, beta);
[Ps1_s_at_gamma, Ps2_s_at_gamma, Ps3_s_at_gamma] = P_composition_gamma(N, K, Ps1_s, Ps2_s, Ps3_s, alphaBar, betaBar);
[Pu1_t_at_gamma, Pu2_t_at_gamma, Pu3_t_at_gamma] = P_composition_gamma(N, K, Pu1_t, Pu2_t, Pu3_t, alpha, beta);
[Ps1_t_at_gamma, Ps2_t_at_gamma, Ps3_t_at_gamma] = P_composition_gamma(N, K, Ps1_t, Ps2_t, Ps3_t, alphaBar, betaBar);

%Composition of F with gamma:
[Pu1_at_gamma, Pu2_at_gamma, Pu3_at_gamma] = P_composition_gamma(N, K, Pu1, Pu2, Pu3, alpha, beta);
[Ps1_at_gamma, Ps2_at_gamma, Ps3_at_gamma] = P_composition_gamma(N, K, Ps1, Ps2, Ps3, alphaBar, betaBar);

%fill g == rhs. of homological equations.
g1 = zeros(K+1, 1);
g2 = zeros(K+1, 2);
g3 = zeros(K+1, 3);

for (j = 0:K)
   g1(j+1) = Pu1_at_gamma(j+1) - Ps1_at_gamma(j+1);
   g2(j+1) = Pu2_at_gamma(j+1) - Ps2_at_gamma(j+1);
   g3(j+1) = Pu3_at_gamma(j+1) - Ps3_at_gamma(j+1);
end    

%Homological Matrix:
A = [Pu1_s_at_gamma(1), Pu1_t_at_gamma(1), -Ps1_s_at_gamma(1), -Ps1_t_at_gamma(1);
     Pu2_s_at_gamma(1), Pu2_t_at_gamma(1), -Ps2_s_at_gamma(1), -Ps2_t_at_gamma(1);
     Pu3_s_at_gamma(1), Pu3_t_at_gamma(1), -Ps3_s_at_gamma(1), -Ps3_t_at_gamma(1)];
 
 %Moore Penrose Inverse of A:
 A_psudo_inv = pinv(A);
 
 %g_0 = [g1(1);
 %      g2(1);
 %      g3(1)];
 %  
 %x_0 = A_psudo_inv*g_0;
 
%alpha_inv(1) = x_0(1);
%beta_inv(1) = x_0(2);
%alphaBar_inv(1) = x_0(3);
%betaBar_inv(1) = x_0(4);


for (j = 0:K)
    
   s1_j = 0;
   s2_j = 0;
   s3_j = 0;
   for (k = 1:j)
      s1_j = s1_j + Pu1_s_at_gamma(k+1)*alpha_inv(j-k+1) + Pu1_t_at_gamma(k+1)*beta_inv(j-k+1) - ...
                    Ps1_s_at_gamma(k+1)*alphaBar_inv(j-k+1) - Ps1_t_at_gamma(k+1)*betaBar_inv(j-k+1);
                
      s2_j = s2_j + Pu2_s_at_gamma(k+1)*alpha_inv(j-k+1) + Pu2_t_at_gamma(k+1)*beta_inv(j-k+1) - ...
                    Ps2_s_at_gamma(k+1)*alphaBar_inv(j-k+1) - Ps2_t_at_gamma(k+1)*betaBar_inv(j-k+1);
                
      s3_j = s3_j + Pu3_s_at_gamma(k+1)*alpha_inv(j-k+1) + Pu3_t_at_gamma(k+1)*beta_inv(j-k+1) - ...
                    Ps3_s_at_gamma(k+1)*alphaBar_inv(j-k+1) - Ps3_t_at_gamma(k+1)*betaBar_inv(j-k+1);
   end    
   
   g_j = [g1(j+1) - s1_j;
          g2(j+1) - s2_j;
          g3(j+1) - s3_j];
   
   x_j = A_psudo_inv*g_j;
   
   alpha_inv(j+1) = x_j(1);
   beta_inv(j+1) = x_j(2);
   alphaBar_inv(j+1) = x_j(3);
   betaBar_inv(j+1) = x_j(4);

end    

%Return:
varargout(1,1)={alpha_inv};
varargout(1,2)={beta_inv};
varargout(1,3)={alphaBar_inv};
varargout(1,4)={betaBar_inv};

