
clear
format long 

load('bestData3_Ds.dat');
load('bestData3_Du.dat');
load('bestData3_Ws.dat');
load('bestData3_Wu.dat');

Ds = bestData3_Ds;
Du = bestData3_Du;
Ws = bestData3_Ws;
Wu = bestData3_Wu;

%=================================================================
%   Compute Two Dimensional Unstable Manifolds by
%   Parametrization Method.
%
%
%  (This is the correct code...)
%
%================================================================

%==============================================================
%Computation Controls:
%==============================================================


%Map Parameters:
a = 0.55;
b = 0.01;
c = 0.44;

alpha = -0.4;  %0.0
tau = -0.5;   %-0.17

%Number of iterates forward to look for intersection:
  k = 5;
  %k = 5 works.
  
  %Order of manifold computation
  N = 60;

%Set Tollerances:
tolerance = 10^(-2);
max_try = 35;

%number of guesses.  The guesses are equi distributed 
%around circles of radi r1, r2, r3:
   numGuesses = 20
   numDepth = 1
   %guess_r1 = 0.35;
   %guess_r2 = 0.35;
   %guess_r3 = 0.8;

%NOTE:  For alpha = -0.5 and tau = 3.0 p0 is stable
%and p1 is unstable.  In the stable case the first eigenvalue
%is real and the second two are complex.  For the unstable case
%the the first two eigenvalues are complex and the third is real.
       
  %scale the eigenvectors (determins paramatrization uniquely):
  scale_u1 = 2.0;
  scale_u2 = 2.0;  
  scale_s1 = 2.0;
  scale_s2 = 2.0;  
    
%==============================================================
%Program:
%==============================================================

%Fixed Points, Eigenvalues, Eigenvectors, ect:  ================

x0 = (1/2)*(-tau + sqrt( tau^2 - 4*alpha));
x1 = (1/2)*(-tau - sqrt( tau^2 - 4*alpha));

p0 = [x0; x0; x0];
p1 = [x1; x1; x1];

fp0 = henon3D(p0, alpha, tau, a, b, c);
fp1 = henon3D(p1, alpha, tau, a, b, c);

%Differential at p0;

A_0 = (2*a + b)*x0 + tau;
B_0 = (2*c + b)*x0;

Df0 = [ A_0, B_0, 1;
          1,   0, 0;
          0,   1, 0];
    
%Differential of p1;

A_1 = (2*a + b)*x1 + tau;
B_1 = (2*c + b)*x1;

Df1 = [ A_1, B_1, 1;
          1,   0, 0;
          0,   1, 0];
    
%Eigenvalues and vectors of Df0:

[E0, eigenvalues_0] = eigs(Df0);

E0
eigenvalues_0

%Eigenvalues and vectors of Df1:

[E1, eigenvalues_1] = eigs(Df1);

E1
eigenvalues_1

%Check linear stablilty of the fixed points

stability_0 = norm(eigenvalues_0(2,2))
stability_1 = norm(eigenvalues_1(1,1))




%==============================================================
%    Computations:
%==============================================================


%===========================================================
%Compute unstable manifold of p0:
%===========================================================

lambda_s = eigenvalues_0(2,2);
nu = scale_s1*E0(:, 2);
nu_bar = scale_s2*E0(:, 3);

[As, Bs, Cs] = parametrizationCoefficents_complexCase(N, ...
    p0, nu, nu_bar, lambda_s, tau, a, b, c);

%=======================================================
%Compute unstable manifold of p1:
%=======================================================

lambda_u = eigenvalues_1(1,1);
xi = scale_u1*E1(:, 1);
xi_bar = scale_u2*E1(:, 2);

[Au, Bu, Cu] = parametrizationCoefficents_complexCase(N, ...
    p1, xi, xi_bar, lambda_u, tau, a, b, c);



%==========================================================================
%Compute this intersections of Wu and Ws with the plane by a Newton Method: 
%==========================================================================


%Point q is a point on the plane:
q0 = p0 + (1/2)*(p1-p0);

%eta one and two are the direction vectors of the plane:        
eta1 = [1; 0; -1];
eta2 = [0; 1; -1];

%initialize "guesses"
%guesses = initializeGuesses(numGuesses, guess_r1, guess_r2, guess_r3);
guesses = initializeGuesses_withDepth(numGuesses, ...
    numDepth, 0.85, 0.7, 0.85, 0.7, 4.0, 0.5);

figure
plot(guesses(:,1), guesses(:,2), '.');
figure
plot(guesses(:,3), guesses(:,4), '.');
figure
plot(guesses(:,5), guesses(:,6), '.');

%Newton Method:
[intersection_W, intersection_uParSpace, intersection_sParSpace, intersect_plane] = ...
    iterateIntersectPlane_complexCase(N, q0, k, ...
    guesses, Au, Bu, Cu, As, Bs, Cs, tolerance, max_try, eta1, eta2, ...
    alpha, tau, a, b, c);

newtonDone = 1

%================================================================
%Results:
%================================================================

%Unstable parameter space plots:
figure
hold on


plot(Du(:,1), Du(:,2), 'b.')
plot(intersection_uParSpace(:,1), intersection_uParSpace(:,2), 'k*')

%Stable parameter space plots:
figure
hold on

plot(Ds(:,1), Ds(:,2), 'r*')
plot(intersection_sParSpace(:,1), intersection_sParSpace(:,2), 'k*')

%Stable parameter space plots:
figure
hold on

plot(intersect_plane(:,1), intersect_plane(:,2), 'g*')

%Phase space plots:
figure
hold on
   
plot3(x0, x0, x0, 'ko')
plot3(x1, x1, x1, 'ko')
plot3(q0(1), q0(2), q0(3), 'ro')
plot3(Wu(:,1), Wu(:,2), Wu(:,3), 'r.'); 
plot3(Ws(:,1), Ws(:,2), Ws(:,3), 'b.'); 
plot3(intersection_W(:,1), intersection_W(:,2), intersection_W(:,3), 'k*');

%Phase space plots:
figure
hold on
   
plot3(x0, x0, x0, 'ko')
plot3(x1, x1, x1, 'ko')
plot3(q0(1), q0(2), q0(3), 'ro')
plot3(Wu(:,1), Wu(:,2), Wu(:,3), 'r.'); 
plot3(Ws(:,1), Ws(:,2), Ws(:,3), 'b.'); 
plot3(intersection_W(:,1), intersection_W(:,2), intersection_W(:,3), 'k*');


fkWu = iterate_set_k(Wu, k, alpha, tau, a, b, c);
fkWs = inverse_iterate_set_k(Ws, k, alpha, tau, a, b, c);


plot3(fkWu(:,1), fkWu(:,2), fkWu(:,3), 'g.'); 
plot3(fkWs(:,1), fkWs(:,2), fkWs(:,3), 'm.'); 


axis([-3.0 3.0 -3.0 3.0 -3.0 3.0]);

