function varargout = oneDimManifold_3DHenon(a, b, c, alpha, tau, ...
    N, p, lambda, xi, scale);


%Initialize first order data:
x0 = p(1);

A_0 = (2*a + b)*x0 + tau;
B_0 = (2*c + b)*x0;

Df0 = [ A_0, B_0, 1;
          1,   0, 0;
          0,   1, 0];
    

     
      
%Initialize Coefficient Containers:      
order_1 = N;


A = zeros(1, order_1+1);
B = zeros(1, order_1+1);
C = zeros(1, order_1+1);

%Assign first order data:
A(1) = x0;
B(1) = x0;
C(1) = x0;

A(2) = scale*xi(1);
B(2) = scale*xi(2);
C(2) = scale*xi(3);


%--------------------------------------------------------------------------
%Solve the Homological Equations and Determine the coefficients
%--------------------------------------------------------------------------


for (n = 2:order_1)
    HomMatrix = [tau + 2*a*x0 + b*x0 - lambda^n,      b*x0 + 2*c*x0,             1;
                                              1,          -lambda^n,             0;
                                              0,                  1,     -lambda^n];
    S_n = 0;
    for (k = 1:(n-1))
       S_n = S_n + a*A(k+1)*A(n-k+1) + b*A(k+1)*B(n-k+1) + c*B(k+1)*B(n-k+1);
    end

    thisRHS = [-S_n;
               0;
               0];
           
    thisCoeff = HomMatrix\thisRHS;
    A(n+1) = thisCoeff(1);
    B(n+1) = thisCoeff(2);
    C(n+1) = thisCoeff(3);
end    
      

varargout(1,1)={A};
varargout(2,1)={B};
varargout(3,1)={C};



