function varargout=parameterizeHeteroclinic(K, N,  s_0, t_0, ds_0, dt_0, sbar_0, tbar_0, dsbar_0, dtbar_0, Au, Bu, Cu, As, Bs, Cs);


K
N

%initialize arrays:
alpha = zeros(1, K+1);
beta = zeros(1, K+1);

alphaBar = zeros(1, K+1);
betaBar = zeros(1, K+1);

a = zeros(K+1, N+1);
b = zeros(K+1, N+1);
c = zeros(K+1, N+1);
d = zeros(K+1, N+1);

aBar = zeros(K+1, N+1);
bBar = zeros(K+1, N+1);
cBar = zeros(K+1, N+1);
dBar = zeros(K+1, N+1);

%assign alpha_0, beta_0, alpha_1, beta_1 barred and unbarred: 
alpha(1) = s_0;
beta(1) = t_0;
alphaBar(1) = sbar_0;
betaBar(1) = tbar_0;

alpha(2) = ds_0;
beta(2) = dt_0;
alphaBar(2) = dsbar_0;
betaBar(2) = dtbar_0;


r_0 = sqrt(alpha(1)^2 + beta(1)^2);
rBar_0 = sqrt(alphaBar(1)^2 + betaBar(1)^2);

theta_0 = atan2(beta(1), alpha(1));
phi_0 = atan2(-beta(1), alpha(1));

thetaBar_0 = atan2(betaBar(1), alphaBar(1));
phiBar_0 = atan2(-betaBar(1), alphaBar(1));



a(1,1) = 1;
b(1,1) = 0;
c(1,1) = 1;
d(1,1) = 0;

aBar(1,1) = 1;
bBar(1,1) = 0;
cBar(1,1) = 1;
dBar(1,1) = 0;

for (k = 1:K)
    a(1,1) = 0;
    b(1,1) = 0;
    c(1,1) = 0;
    d(1,1) = 0;

    aBar(1,1) = 0;
    bBar(1,1) = 0;
    cBar(1,1) = 0;
    dBar(1,1) = 0;  
end    



for (m = 2:N) 
   a(1, m+1) = (r_0^m)*cos(m*theta_0); 
   b(1, m+1) = (r_0^m)*sin(m*theta_0);
   c(1, m+1) = (r_0^m)*cos(m*phi_0);
   d(1, m+1) = (r_0^m)*sin(m*phi_0);
   
   aBar(1, m+1) = (rBar_0^m)*cos(m*thetaBar_0);
   bBar(1, m+1) = (rBar_0^m)*sin(m*thetaBar_0);
   cBar(1, m+1) = (rBar_0^m)*cos(m*phiBar_0);
   dBar(1, m+1) = (rBar_0^m)*sin(m*phiBar_0);
   
   a(2, m+1) = (m*r_0^(m-1))*(alpha(2)*cos((m-1)*theta_0) - beta(2)*sin((m-1)*theta_0)); 
   b(2, m+1) = (m*r_0^(m-1))*(alpha(2)*sin((m-1)*theta_0) + beta(2)*cos((m-1)*theta_0));
   c(2, m+1) = (m*r_0^(m-1))*(alpha(2)*cos((m-1)*phi_0) - beta(2)*sin((m-1)*phi_0));
   d(2, m+1) = (m*r_0^(m-1))*(alpha(2)*sin((m-1)*phi_0) + beta(2)*cos((m-1)*phi_0));
   
   aBar(2, m+1) = (m*rBar_0^(m-1))*(alphaBar(2)*cos((m-1)*thetaBar_0) - betaBar(2)*sin((m-1)*thetaBar_0));
   bBar(2, m+1) = (m*rBar_0^(m-1))*(alphaBar(2)*sin((m-1)*thetaBar_0) + betaBar(2)*cos((m-1)*thetaBar_0));
   cBar(2, m+1) = (m*rBar_0^(m-1))*(alphaBar(2)*cos((m-1)*phiBar_0) - betaBar(2)*sin((m-1)*phiBar_0));
   dBar(2, m+1) = (m*rBar_0^(m-1))*(alphaBar(2)*sin((m-1)*phiBar_0) + betaBar(2)*cos((m-1)*phiBar_0));
end    


[A1_c, B1_c] = coefficients_c(N, Au, alpha, beta, a, b, c, d);
[A2_c, B2_c] = coefficients_c(N, Bu, alpha, beta, a, b, c, d);
[A3_c, B3_c] = coefficients_c(N, Cu, alpha, beta, a, b, c, d);
[Abar1_c, Bbar1_c] = coefficients_c(N, As, alphaBar, betaBar, aBar, bBar, cBar, dBar);
[Abar2_c, Bbar2_c] = coefficients_c(N, Bs, alphaBar, betaBar, aBar, bBar, cBar, dBar);
[Abar3_c, Bbar3_c] = coefficients_c(N, Cs, alphaBar, betaBar, aBar, bBar, cBar, dBar);


[A1_0n, B1_0n] = coefficients_0n(N, Au, alpha, beta, a, b, c, d);
[A2_0n, B2_0n] = coefficients_0n(N, Bu, alpha, beta, a, b, c, d);
[A3_0n, B3_0n] = coefficients_0n(N, Cu, alpha, beta, a, b, c, d);
[Abar1_0n, Bbar1_0n] = coefficients_0n(N, As, alphaBar, betaBar, aBar, bBar, cBar, dBar);
[Abar2_0n, Bbar2_0n] = coefficients_0n(N, Bs, alphaBar, betaBar, aBar, bBar, cBar, dBar);
[Abar3_0n, Bbar3_0n] = coefficients_0n(N, Cs, alphaBar, betaBar, aBar, bBar, cBar, dBar);


[A1_m0, B1_m0] = coefficients_m0(N, Au, alpha, beta, a, b, c, d);
[A2_m0, B2_m0] = coefficients_m0(N, Bu, alpha, beta, a, b, c, d);
[A3_m0, B3_m0] = coefficients_m0(N, Cu, alpha, beta, a, b, c, d);
[Abar1_m0, Bbar1_m0] = coefficients_m0(N, As, alphaBar, betaBar, aBar, bBar, cBar, dBar);
[Abar2_m0, Bbar2_m0] = coefficients_m0(N, Bs, alphaBar, betaBar, aBar, bBar, cBar, dBar);
[Abar3_m0, Bbar3_m0] = coefficients_m0(N, Cs, alphaBar, betaBar, aBar, bBar, cBar, dBar);


[A1_1n, B1_1n] = coefficients_1n(N, Au, alpha, beta, a, b, c, d);
[A2_1n, B2_1n] = coefficients_1n(N, Bu, alpha, beta, a, b, c, d);
[A3_1n, B3_1n] = coefficients_1n(N, Cu, alpha, beta, a, b, c, d);
[Abar1_1n, Bbar1_1n] = coefficients_1n(N, As, alphaBar, betaBar, aBar, bBar, cBar, dBar);
[Abar2_1n, Bbar2_1n] = coefficients_1n(N, Bs, alphaBar, betaBar, aBar, bBar, cBar, dBar);
[Abar3_1n, Bbar3_1n] = coefficients_1n(N, Cs, alphaBar, betaBar, aBar, bBar, cBar, dBar);


[A1_m1, B1_m1] = coefficients_m1(N, Au, alpha, beta, a, b, c, d);
[A2_m1, B2_m1] = coefficients_m1(N, Bu, alpha, beta, a, b, c, d);
[A3_m1, B3_m1] = coefficients_m1(N, Cu, alpha, beta, a, b, c, d);
[Abar1_m1, Bbar1_m1] = coefficients_m1(N, As, alphaBar, betaBar, aBar, bBar, cBar, dBar);
[Abar2_m1, Bbar2_m1] = coefficients_m1(N, Bs, alphaBar, betaBar, aBar, bBar, cBar, dBar);
[Abar3_m1, Bbar3_m1] = coefficients_m1(N, Cs, alphaBar, betaBar, aBar, bBar, cBar, dBar);

[A1_mn, B1_mn] = coefficients_mn(N, Au, alpha, beta, a, b, c, d);
[A2_mn, B2_mn] = coefficients_mn(N, Bu, alpha, beta, a, b, c, d);
[A3_mn, B3_mn] = coefficients_mn(N, Cu, alpha, beta, a, b, c, d);
[Abar1_mn, Bbar1_mn] = coefficients_mn(N, As, alphaBar, betaBar, aBar, bBar, cBar, dBar);
[Abar2_mn, Bbar2_mn] = coefficients_mn(N, Bs, alphaBar, betaBar, aBar, bBar, cBar, dBar);
[Abar3_mn, Bbar3_mn] = coefficients_mn(N, Cs, alphaBar, betaBar, aBar, bBar, cBar, dBar);


A1 = A1_c + A1_0n + A1_m0 + A1_1n + A1_m1 + A1_mn;
A2 = A2_c + A2_0n + A2_m0 + A2_1n + A2_m1 + A2_mn;
A3 = A3_c + A3_0n + A3_m0 + A3_1n + A3_m1 + A3_mn;

B1 = B1_c + B1_0n + B1_m0 + B1_1n + B1_m1 + B1_mn;
B2 = B2_c + B2_0n + B2_m0 + B2_1n + B2_m1 + B2_mn;
B3 = B3_c + B3_0n + B3_m0 + B3_1n + B3_m1 + B3_mn;

Abar1 = Abar1_c + Abar1_0n + Abar1_m0 + Abar1_1n + Abar1_m1 + Abar1_mn;
Abar2 = Abar2_c + Abar2_0n + Abar2_m0 + Abar2_1n + Abar2_m1 + Abar2_mn;
Abar3 = Abar3_c + Abar3_0n + Abar3_m0 + Abar3_1n + Abar3_m1 + Abar3_mn;

Bbar1 = Bbar1_c + Bbar1_0n + Bbar1_m0 + Bbar1_1n + Bbar1_m1 + Bbar1_mn;
Bbar2 = Bbar2_c + Bbar2_0n + Bbar2_m0 + Bbar2_1n + Bbar2_m1 + Bbar2_mn;
Bbar3 = Bbar3_c + Bbar3_0n + Bbar3_m0 + Bbar3_1n + Bbar3_m1 + Bbar3_mn;

homologicalMatrix = [A1, B1, -Abar1, -Bbar1;
                     A2, B2, -Abar2, -Bbar2;
                     A3, B3, -Abar3, -Bbar3];

inv_homologicalMatrix = pinv(homologicalMatrix);                  
                 
for (j = 2:K)
   check_j = j 
   %Compute components of RHS: 
   Q1j_c = call_Q_jc(j, N, Au, alpha, beta, a, b, c, d);
   Q1j_0n = call_Q_j0n(j, N, Au, alpha, beta, a, b, c, d);
   Q1j_m0 = call_Q_jm0(j, N, Au, alpha, beta, a, b, c, d);
   Q1j_1n = call_Q_j1n(j, N, Au, alpha, beta, a, b, c, d);
   Q1j_m1 = call_Q_jm1(j, N, Au, alpha, beta, a, b, c, d);
   Q1j_mn = call_Q_jmn(j, N, Au, alpha, beta, a, b, c, d);
   
   Q2j_c = call_Q_jc(j, N, Bu, alpha, beta, a, b, c, d);
   Q2j_0n = call_Q_j0n(j, N, Bu, alpha, beta, a, b, c, d);
   Q2j_m0 = call_Q_jm0(j, N, Bu, alpha, beta, a, b, c, d);
   Q2j_1n = call_Q_j1n(j, N, Bu, alpha, beta, a, b, c, d);
   Q2j_m1 = call_Q_jm1(j, N, Bu, alpha, beta, a, b, c, d);
   Q2j_mn = call_Q_jmn(j, N, Bu, alpha, beta, a, b, c, d);
   
   Q3j_c = call_Q_jc(j, N, Cu, alpha, beta, a, b, c, d);
   Q3j_0n = call_Q_j0n(j, N, Cu, alpha, beta, a, b, c, d);
   Q3j_m0 = call_Q_jm0(j, N, Cu, alpha, beta, a, b, c, d);
   Q3j_1n = call_Q_j1n(j, N, Cu, alpha, beta, a, b, c, d);
   Q3j_m1 = call_Q_jm1(j, N, Cu, alpha, beta, a, b, c, d);
   Q3j_mn = call_Q_jmn(j, N, Cu, alpha, beta, a, b, c, d);
   
   Q1barj_c = call_Q_jc(j, N, As, alphaBar, betaBar, aBar, bBar, cBar, dBar);
   Q1barj_0n = call_Q_j0n(j, N, As, alphaBar, betaBar, aBar, bBar, cBar, dBar);
   Q1barj_m0 = call_Q_jm0(j, N, As, alphaBar, betaBar, aBar, bBar, cBar, dBar);
   Q1barj_1n = call_Q_j1n(j, N, As, alphaBar, betaBar, aBar, bBar, cBar, dBar);
   Q1barj_m1 = call_Q_jm1(j, N, As, alphaBar, betaBar, aBar, bBar, cBar, dBar);
   Q1barj_mn = call_Q_jmn(j, N, As, alphaBar, betaBar, aBar, bBar, cBar, dBar);
   
   Q2barj_c = call_Q_jc(j, N, Bs, alphaBar, betaBar, aBar, bBar, cBar, dBar);
   Q2barj_0n = call_Q_j0n(j, N, Bs, alphaBar, betaBar, aBar, bBar, cBar, dBar);
   Q2barj_m0 = call_Q_jm0(j, N, Bs, alphaBar, betaBar, aBar, bBar, cBar, dBar);
   Q2barj_1n = call_Q_j1n(j, N, Bs, alphaBar, betaBar, aBar, bBar, cBar, dBar);
   Q2barj_m1 = call_Q_jm1(j, N, Bs, alphaBar, betaBar, aBar, bBar, cBar, dBar);
   Q2barj_mn = call_Q_jmn(j, N, Bs, alphaBar, betaBar, aBar, bBar, cBar, dBar);
   
   Q3barj_c = call_Q_jc(j, N, Cs, alphaBar, betaBar, aBar, bBar, cBar, dBar);
   Q3barj_0n = call_Q_j0n(j, N, Cs, alphaBar, betaBar, aBar, bBar, cBar, dBar);
   Q3barj_m0 = call_Q_jm0(j, N, Cs, alphaBar, betaBar, aBar, bBar, cBar, dBar);
   Q3barj_1n = call_Q_j1n(j, N, Cs, alphaBar, betaBar, aBar, bBar, cBar, dBar);
   Q3barj_m1 = call_Q_jm1(j, N, Cs, alphaBar, betaBar, aBar, bBar, cBar, dBar);
   Q3barj_mn = call_Q_jmn(j, N, Cs, alphaBar, betaBar, aBar, bBar, cBar, dBar);
   
   Q1j = Q1j_c + Q1j_0n + Q1j_m0 + Q1j_1n + Q1j_m1 + Q1j_mn;
   Q2j = Q2j_c + Q2j_0n + Q2j_m0 + Q2j_1n + Q2j_m1 + Q2j_mn;
   Q3j = Q3j_c + Q3j_0n + Q3j_m0 + Q3j_1n + Q3j_m1 + Q3j_mn;
   Q1barj = Q1barj_c + Q1barj_0n + Q1barj_m0 + Q1barj_1n + Q1barj_m1 + Q1barj_mn;
   Q2barj = Q2barj_c + Q2barj_0n + Q2barj_m0 + Q2barj_1n + Q2barj_m1 + Q2barj_mn;
   Q3barj = Q3barj_c + Q3barj_0n + Q3barj_m0 + Q3barj_1n + Q3barj_m1 + Q3barj_mn;
   
   
   %RHS:
   b_rhs = [Q1barj - Q1j; 
            Q2barj - Q2j; 
            Q3barj - Q3j];
   
   %Solve for the j-th components:
   x = inv_homologicalMatrix*b_rhs;
   
   %Assign the j-th components:
   alpha(j+1) = x(1);
   beta(j+1) = x(2);
   alphaBar(j+1) = x(3);
   betaBar(j+1) = x(4);

   %Solve for next a, b, c, d, aBar, bBar, cBar, dBar:
   ab_matrix = [alpha(1), -beta(1);
               beta(1), alpha(1)];
   
           
   cd_matrix = [alpha(1), beta(1);
               -beta(1), alpha(1)];
     
   abBar_matrix = [alphaBar(1), -betaBar(1);
                   betaBar(1), alphaBar(1)];
   
   cdBar_matrix = [alphaBar(1), betaBar(1);
                   -betaBar(1), alphaBar(1)];
           
   
   for (m = 2:N)
      
      sum1 = 0;
      sum2 = 0;
      sum3 = 0;
      sum4 = 0;
      
      sumBar1 = 0;
      sumBar2 = 0;
      sumBar3 = 0;
      sumBar4 = 0;
      
      
      for (k = 0:(j-2)) 
         sum1 = sum1 + (k+1)*(a(k+2,m+1)*alpha(j-k) - b(k+2,m+1)*beta(j-k) - m*(a(j-k,m+1)*alpha(k+2) - b(j-k,m+1)*beta(k+2)));
         sum2 = sum2 + (k+1)*(a(k+2,m+1)*beta(j-k) + b(k+2,m+1)*alpha(j-k) - m*(a(j-k,m+1)*beta(k+2) + b(j-k,m+1)*alpha(k+2)));
         sum3 = sum3 + (k+1)*(c(k+2,m+1)*alpha(j-k) + d(k+2,m+1)*beta(j-k) - m*(c(j-k,m+1)*alpha(k+2) + d(j-k,m+1)*beta(k+2)));
         sum4 = sum4 + (k+1)*(-c(k+2,m+1)*beta(j-k) + d(k+2,m+1)*alpha(j-k) - m*(-c(j-k,m+1)*beta(k+2) + d(j-k,m+1)*alpha(k+2)));
         
         sumBar1 = sumBar1 + (k+1)*(aBar(k+2,m+1)*alphaBar(j-k) - bBar(k+2,m+1)*betaBar(j-k) - m*(aBar(j-k,m+1)*alphaBar(k+2) - bBar(j-k,m+1)*betaBar(k+2)));
         sumBar2 = sumBar2 + (k+1)*(aBar(k+2,m+1)*betaBar(j-k) + bBar(k+2,m+1)*alphaBar(j-k) - m*(aBar(j-k,m+1)*betaBar(k+2) + bBar(j-k,m+1)*alphaBar(k+2)));
         sumBar3 = sumBar3 + (k+1)*(cBar(k+2,m+1)*alphaBar(j-k) + dBar(k+2,m+1)*betaBar(j-k) - m*(cBar(j-k,m+1)*alphaBar(k+2) + dBar(j-k,m+1)*betaBar(k+2)));
         sumBar4 = sumBar4 + (k+1)*(-cBar(k+2,m+1)*betaBar(j-k) + dBar(k+2,m+1)*alphaBar(j-k) - m*(-cBar(j-k,m+1)*betaBar(k+2) + dBar(j-k,m+1)*alphaBar(k+2)));
      end    
      
      const = -1/j;
      
      s_j1n = const*sum1 + m*(a(1,m+1)*alpha(j+1) - b(1,m+1)*beta(j+1));
      s_j2n = const*sum2 + m*(a(1,m+1)*beta(j+1) + b(1,m+1)*alpha(j+1));
      
      r_j1n = const*sum3 + m*(c(1,m+1)*alpha(j+1) + d(1,m+1)*beta(j+1));
      r_j2n = const*sum4 + m*(-c(1,m+1)*beta(j+1) + d(1,m+1)*alpha(j+1));
      
      sBar_j1n = const*sumBar1 + m*(aBar(1,m+1)*alphaBar(j+1) - bBar(1,m+1)*betaBar(j+1));
      sBar_j2n = const*sumBar2 + m*(aBar(1,m+1)*betaBar(j+1) + bBar(1,m+1)*alphaBar(j+1));
      
      rBar_j1n = const*sumBar3 + m*(cBar(1,m+1)*alphaBar(j+1) + dBar(1,m+1)*betaBar(j+1));
      rBar_j2n = const*sumBar4 + m*(-cBar(1,m+1)*betaBar(j+1) + dBar(1,m+1)*alphaBar(j+1));
      
      s_ab = [s_j1n;
              s_j2n];
          
      s_cd = [r_j1n;
              r_j2n];
          
      s_abBar = [sBar_j1n;
                 sBar_j2n];
             
      s_cdBar = [rBar_j1n;
                 rBar_j2n];
             
      next_ab = ab_matrix\s_ab;
      next_cd = cd_matrix\s_cd;
      next_abBar = abBar_matrix\s_abBar;
      next_cdBar = cdBar_matrix\s_cdBar;
      
      a(j+1, m+1) = next_ab(1);
      b(j+1, m+1) = next_ab(2);
      
      c(j+1, m+1) = next_cd(1);
      d(j+1, m+1) = next_cd(2);
      
      aBar(j+1, m+1) = next_abBar(1);
      bBar(j+1, m+1) = next_abBar(2);
      
      cBar(j+1, m+1) = next_cdBar(1);
      dBar(j+1, m+1) = next_cdBar(2);
      
   end    
   
   
   
end    

%================================================================
%Results:
%================================================================


%Return:
varargout(1,1)={alpha};
varargout(2,1)={beta};
varargout(3,1)={alphaBar};
varargout(4,1)={betaBar};















