
clear
format long 

%=================================================================
%   Compute Two Dimensional Stable and Unstable Manifolds by
%   Parametrization Method.
%
%
%  (This is the correct code...)
%
%================================================================



%==============================================================
%Computation Controls:
%==============================================================

%Map Parameters:
a = 0.5;
b = 0.0;
c = 0.5;

alpha = -0.5;  %0.0
tau = -0.5;   %-0.17

%NOTE:  For alpha = -0.5 and tau = 3.0 p0 is stable
%and p1 is unstable.  In the stable case the first eigenvalue
%is real and the second two are complex.  For the unstable case
%the the first two eigenvalues are complex and the third is real.


%polynomial approximation of W^s(p1):

  %Order of manifold computation
  N = 50;

  %scale the eigenvectors (determins paramatrization uniquely):
  scale_u1 = 0.5;
  scale_u2 = 0.5;  
  scale_s1 = 0.5;
  scale_s2 = 0.5;  
  
  %number of poin
  num_t = 45;
  num_s = 45;

  s0 = -2.5;
  t0 = -2.5;

  t_length = abs(2*t0);
  s_length = abs(2*s0);

  h2 = s_length/num_s;
  h1 = t_length/num_t;

  
%==============================================================
%Program:
%==============================================================



%Fixed Points, Eigenvalues, Eigenvectors, ect:  ================

x0 = (1/2)*(-tau + sqrt( tau^2 - 4*alpha));
x1 = (1/2)*(-tau - sqrt( tau^2 - 4*alpha));

p0 = [x0; x0; x0];
p1 = [x1; x1; x1];

fp0 = henon3D(p0, alpha, tau, a, b, c);
fp1 = henon3D(p1, alpha, tau, a, b, c);

%Differential at p0;

A_0 = (2*a + b)*x0 + tau;
B_0 = (2*c + b)*x0;

Df0 = [ A_0, B_0, 1;
          1,   0, 0;
          0,   1, 0];
    
%Differential of p1;

A_1 = (2*a + b)*x1 + tau;
B_1 = (2*c + b)*x1;

Df1 = [ A_1, B_1, 1;
          1,   0, 0;
          0,   1, 0];
    
%Eigenvalues and vectors of Df0:

[E0, eigenvalues_0] = eigs(Df0);

E0
eigenvalues_0

%Eigenvalues and vectors of Df1:

[E1, eigenvalues_1] = eigs(Df1);

E1
eigenvalues_1

%Check linear stablilty of the fixed points

stability_0 = norm(eigenvalues_0(2,2))
stability_1 = norm(eigenvalues_1(1,1))



%==============================================================
%    Computations:
%==============================================================


%===========================================================
%Compute unstable manifold of p0:
%===========================================================

lambda = eigenvalues_0(2,2);
nu = scale_s1*E0(:, 2);
nu_bar = scale_s2*E0(:, 3);

[As, Bs, Cs] = parametrizationCoefficents_complexCase(N, p0, nu, nu_bar, lambda, tau, a, b, c);

%=======================================================
%Compute unstable manifold of p1:
%=======================================================

lambda = eigenvalues_1(1,1);
xi = scale_u1*E1(:, 1);
xi_bar = scale_u2*E1(:, 2);

[Au, Bu, Cu] = parametrizationCoefficents_complexCase(N, p1, xi, xi_bar, lambda, tau, a, b, c);




%==========================================================================
%Evaluate the polynomial to compute the manifolds:
%==========================================================================


Ws = evaluate_manifold(N, As, Bs, Cs, num_t, num_s, s0, t0, h1, h2);
Wu = evaluate_manifold(N, Au, Bu, Cu, num_t, num_s, s0, t0, h1, h2);

Ws = real(Ws);
Wu = real(Wu);


%Round off estimation:
roundOffMatrix = imag(Wu);
roundOffNorm_u = norm(roundOffMatrix);

roundOffMatrix = imag(Ws);
roundOffNorm_s = norm(roundOffMatrix);


%===========================================================
%Output:
%===========================================================
outRound_u = roundOffNorm_u
outRound_s = roundOffNorm_s

fixPointDist = norm(p0-p1)

%===========================================================
%Plotting:
%===========================================================

figure
hold on
   
plot3(Wu(:,1), Wu(:,2), Wu(:,3), 'b.')  
plot3(Ws(:,1), Ws(:,2), Ws(:,3), 'r.')
plot3(x0, x0, x0, 'k*')
plot3(x1, x1, x1, 'y*')


axis([-2 2 -2 2 -2 2])
