function varargout = twoDimManifold_6DHenon(a1, b1, c1, alpha1, tau1, ...
    a2, b2, c2, alpha2, tau2, epsilon, N, p, ...
    lambda_1, lambda_2, xi_1, xi_2, scale_1, scale_2);



Dfp = henon6D_differential(p, alpha1, tau1, a1, b1, c1, ...
                              alpha2, tau2, a2, b2, c2, epsilon);

Id = eye(6);
      
%Initialize Coefficient Containers:      


A = zeros(N+1, N+1);
B = zeros(N+1, N+1);
C = zeros(N+1, N+1);
D = zeros(N+1, N+1);
E = zeros(N+1, N+1);
F = zeros(N+1, N+1);

%Assign first order data:
A(1) = p(1);
B(1) = p(2);
C(1) = p(3);
D(1) = p(4);
E(1) = p(5);
F(1) = p(6);


A(2,1) = scale_1*xi_1(1);
A(1,2) = scale_2*xi_2(1);

B(2,1) = scale_1*xi_1(2);
B(1,2) = scale_2*xi_2(2);

C(2,1) = scale_1*xi_1(3); 
C(1,2) = scale_2*xi_2(3);

D(2,1) = scale_1*xi_1(4);
D(1,2) = scale_2*xi_2(4);

E(2,1) = scale_1*xi_1(5);
E(1,2) = scale_2*xi_2(5);

F(2,1) = scale_1*xi_1(6); 
F(1,2) = scale_2*xi_2(6);



%--------------------------------------------------------------------------
%Solve the Homological Equations and Determine the coefficients
%--------------------------------------------------------------------------



for n = 0:N
    for m = 0:N
        if (m+n >= 2)

            Amn = Dfp - (lambda_1^m * lambda_2^n)*Id;            

            smn = -compute_smn_6D(m, n, A, B, C, D, E, F, ...
                                  alpha1, tau1, a1, b1, c1, ...
                                  alpha2, tau2, a2, b2, c2, ...
                                  epsilon, N);

            vmn = Amn\smn;

            A(m+1, n+1) = vmn(1);
            B(m+1, n+1) = vmn(2);
            C(m+1, n+1) = vmn(3);
            D(m+1, n+1) = vmn(4);
            E(m+1, n+1) = vmn(5);
            F(m+1, n+1) = vmn(6);
        end
    end
end





varargout(1,1)={A};
varargout(2,1)={B};
varargout(3,1)={C};
varargout(4,1)={D};
varargout(5,1)={E};
varargout(6,1)={F};








