function varargout = twoDimManifold_intval_6DHenon(a1, b1, c1, alpha1, tau1, ...
    a2, b2, c2, alpha2, tau2, epsilon, N, p, ...
    lambda_1, lambda_2, xi_1, xi_2, scale_1, scale_2);



Dfp = henon6D_differential_intval(p, alpha1, tau1, a1, b1, c1, ...
                              alpha2, tau2, a2, b2, c2, epsilon);

Id = intval(eye(6));
      
%Initialize Coefficient Containers:      


A = intval(zeros(N+1, N+1));
B = intval(zeros(N+1, N+1));
C = intval(zeros(N+1, N+1));
D = intval(zeros(N+1, N+1));
E = intval(zeros(N+1, N+1));
F = intval(zeros(N+1, N+1));

%Assign first order data:
A(1) = p(1);
B(1) = p(2);
C(1) = p(3);
D(1) = p(4);
E(1) = p(5);
F(1) = p(6);


A(2,1) = scale_1*xi_1(1);
A(1,2) = scale_2*xi_2(1);

B(2,1) = scale_1*xi_1(2);
B(1,2) = scale_2*xi_2(2);

C(2,1) = scale_1*xi_1(3); 
C(1,2) = scale_2*xi_2(3);

D(2,1) = scale_1*xi_1(4);
D(1,2) = scale_2*xi_2(4);

E(2,1) = scale_1*xi_1(5);
E(1,2) = scale_2*xi_2(5);

F(2,1) = scale_1*xi_1(6); 
F(1,2) = scale_2*xi_2(6);



%--------------------------------------------------------------------------
%Solve the Homological Equations and Determine the coefficients
%--------------------------------------------------------------------------



for n1 = 2:N
    n1
    N
    for n2 = 0:n1
        m = n1 - n2;
        n = n2;
        
        Amn = Dfp - (lambda_1^m * lambda_2^n)*Id;            

        
Q1 = reshape(A(1:m+1, 1:n+1), (m+1)*(n+1), 1);
Q1 = Q1(2:(m+1)*(n+1)-1);
Q1_flip = flipud(Q1);
Q1_T = transpose(Q1);

Q2 = reshape(B(1:m+1, 1:n+1), (m+1)*(n+1), 1);
Q2 = Q2(2:(m+1)*(n+1)-1);
Q2_flip= flipud(Q2);
Q2_T = transpose(Q2);

Q4 = reshape(D(1:m+1, 1:n+1), (m+1)*(n+1), 1);
Q4 = Q4(2:(m+1)*(n+1)-1);
Q4_flip= flipud(Q4);
Q4_T = transpose(Q4);

Q5 = reshape(E(1:m+1, 1:n+1), (m+1)*(n+1), 1);
Q5 = Q5(2:(m+1)*(n+1)-1);
Q5_flip= flipud(Q5);
Q5_T = transpose(Q5);

Y2_term = Q2_T*Q2_flip;
Y5_term = Q5_T*Q5_flip;

term1 = a1*Q1_T*Q1_flip + b1*Q1_T*Q2_flip + c1*Y2_term + epsilon*Y5_term;
term2 = a2*Q4_T*Q4_flip + b2*Q4_T*Q5_flip + c2*Y5_term + epsilon*Y2_term;

	  smn = -[term1;
		  0;
                  0;
		  term2;
		  0;
		  0];
    
        %smn_test = -compute_smn_6D_intval(m, n, A, B, C, D, E, F, ...
        %                                 alpha1, tau1, a1, b1, c1, ...
        %                                 alpha2, tau2, a2, b2, c2, ...
        %                                 epsilon, N)
        %

        vmn = verifylss(Amn, smn);                  
    
        A(m+1, n+1) = vmn(1);
        B(m+1, n+1) = vmn(2);
        C(m+1, n+1) = vmn(3);
        D(m+1, n+1) = vmn(4);
        E(m+1, n+1) = vmn(5);
        F(m+1, n+1) = vmn(6);
      
    end
end





varargout(1,1)={A};
varargout(2,1)={B};
varargout(3,1)={C};
varargout(4,1)={D};
varargout(5,1)={E};
varargout(6,1)={F};








