function varargout = validateChaosProof(validationParameters, ...
    n_s, n_u, N, order_1, scale_0s, scale_0u, ...
    domRad0s, domRad0u, a, b, c, alpha, tau);




      
   %Convert to Lomeli:   
      a = intval(a)      %c_dm
      b = intval(b)      %b_dm - 2*c_dm
      c = intval(c)      %c_dm + a_dm - b_dm
      
      tau =  intval(tau)  %2*a_dm*(3+mu_dm)/(2*a_dm - b_dm)
      alpha =  intval(alpha) %((9+6*mu_dm+mu_dm^2)*a_dm - 4*eps_dm*a_dm^2 + ...
                           %4*eps_dm*a_dm*b_dm - eps_dm*b_dm^2)/(2*a_dm - b_dm)^2
                    
           
[p0, p1, lambda0_s, lambda0_u1, lambda0_u2, ...
         lambda1_u, lambda1_s1, lambda1_s2, ...
         xi0_s, xi0_u1, xi0_u2, ...
         xi1_u, xi1_s1, xi1_s2] ...
       = firstOrderData_intval_3DHenon(a, b, c, alpha, tau);
     
   
   


           
%=====================================================================
%One-D Manifolds 
%=====================================================================

%Step 1: Compute 1D Coefficients:


scale = intval(scale_0s);


tic  

[A0s, B0s, C0s] = oneDimManifold_intval_3DHenon(a, b, c, alpha, tau, ...
     order_1, p0, lambda0_s, xi0_s, scale);

time_compCoeff1D = toc
 
 
 
firstOrder = 1

           
%==========================================================================
%Two-D Manifolds:
%==========================================================================

%Step 1: Compute Coefficients:
 scale_0u_intval = intval(scale_0u);
 
 tic
 
 [A0u, B0u, C0u] = twoDimManifold_intval_3DHenon(a, b, c, alpha, tau, ...
 N, p0, lambda0_u1, lambda0_u2, xi0_u1, xi0_u2, scale_0u, scale_0u_intval);
 
time_compCoeff2D = toc




%----------------------Put in Polynomial Data Structure--------------------



[P0u1, P0u2, P0u3] = convertToPoly_2D(A0u, B0u, C0u, N);


[P0s1, P0s2, P0s3] = convertToPoly_1D(A0s, B0s, C0s, order_1);                                                 



                                               
%Step 2: Validate the Manifolds                                               
                                               



%domRad0s = intval('0.02')
%domRad1u = intval('0.01')
%domRad1s = intval('0.01');              
%domRad0u = intval('0.01');
 

domRad0s = intval(domRad0s)
domRad0u = intval(domRad0u)


N_f = 4
N_dim = 3


tic

[delta_0s, delta_max_0s, P0s_norm, s0_delta_good, OrderTest]...
        = validateStableManifold_1D_3DHenon(...
    P0s1, P0s2, P0s3, p0, xi0_u1, xi0_u2, xi0_s, lambda0_s, ...
    N_f, N_dim, domRad0s, order_1, a, b, c, alpha, tau);

time_validate1D = toc



P0s_norm

delta_0s




                                                                                  
%Phase Space Bounds: 2D


                     
%A-Posteriori Errors: 2D                           


tic

[delta_0u, delta_max_0u, P0u_norm, u0_delta_good, orderTest] ...
      = validateUnstableManifold_2D_3DHenon(...
    P0u1, P0u2, P0u3, p0, xi0_u1, xi0_u2, xi0_s, ...
    lambda0_u1, lambda0_u2, N_f, N_dim, ...
    domRad0u, N, a, b, c, alpha, tau);

time_validate2D = toc


P0s_norm

delta_0s

P0u_norm

delta_0u


temp_1 = size(validationParameters);
numOrbits = temp_1(1,1);


tic

for proofCount = 1:numOrbits

theta_hat = validationParameters(proofCount, 1:2)
xn_hat = validationParameters(proofCount, 3:(3*(n_s + n_u)-1))';
tau_hat = validationParameters(proofCount, 3*(n_s + n_u));

[goodOrbit, goodChaos, r_NK] = ...
    validateOrbit(theta_hat, xn_hat, tau_hat, n_u, n_s, ...
        P0u1, P0u2, P0u3, P0s1, P0s2, P0s3, ...
        domRad0s, domRad0u, delta_0u, delta_0s, ...
        N, order_1, alpha, tau, a, b, c)


  
orbitsValidated(proofCount) = goodOrbit;
chaosValidation(proofCount) = goodChaos;
radii(proofCount) = r_NK;


end


time_orbitsProof = toc




varargout(1,1)={orbitsValidated};
varargout(2,1)={chaosValidation};
varargout(3,1)={radii};





