function varargout = validateOrbit(theta_hat, xn_hat, tau_hat, n_u, n_s, ...
    P0u1, P0u2, P0u3, P0s1, P0s2, P0s3, ...
    domRad0s, domRad0u, delta_0u, delta_0s, ...
    N, order_1, alpha, tau, a, b, c);


N_dim = 3;
N_f = 4;

  

%Interval version of approximate orbit:  (It is not necessary that the 
%                                         interval guess include the
%                                         floating point guess.  We only
%                                         need some guess.  So we opt for a
%                                         guess with zero raduis i.e. a
%                                         degenerate interval.

thetaBar_intval = intval(theta_hat);
xn_bar_intval = intval(xn_hat);
tauBar_intval = intval(tau_hat);



%check parmeter domain conditions:
theta_norm = sqrt(thetaBar_intval(1)^2 + thetaBar_intval(2)^2);
thetaTest = sup(theta_norm)
testDom = inf(domRad0u)
if ( thetaTest >= inf(testDom))
   crashNow = crashHere 
end

tauTest = sup(abs(tauBar_intval))
testDom = domRad0s
if (tauTest >= inf(testDom))
   crashNow = crashHere 
end


sigma_s_hat = -log((sup(abs(tauBar_intval)))/(domRad0s))
sigma_u_hat = -log((sup(theta_norm))/(domRad0u))

if (sup(sigma_s_hat) > 1)
    sigma_s_hat = intval(1)
end

if (sup(sigma_u_hat) > 1)
    sigma_u_hat = intval(1)
end


%The Newton Vector:
[Fx1_intval, Fx2_intval, Fx3_intval] = ...
    F_prod_intval(thetaBar_intval, xn_bar_intval, tauBar_intval, n_u, n_s, ...                          
    P0u1, P0u2, P0u3, P0s1, P0s2, P0s3, alpha, tau, a, b, c);

Fx_intval = [Fx1_intval, Fx2_intval, Fx3_intval];


%Interval evaluation of Df(x_0):
A = DF_prod_intval(thetaBar_intval, xn_bar_intval, tauBar_intval, n_u, n_s, ...
    P0u1, P0u2, P0u3, P0s1, P0s2, P0s3, alpha, tau, a, b, c);



Ainv = inv(A);
Ainv_norm = norm(Ainv, inf)
Ainv_1 = Ainv(:, 1);
Ainv_2 = Ainv(:, 2);
Ainv_nk = Ainv(:, N_dim*(n_s + n_u));

%Step 3: Validate the Newton Computation.                           


%Note the factor of 2 in fron of delta_0u.  This is because the 
%unstable manfiold is two dimensional with complex conjugate 
%eigenvalues.
M_N = 2*intval('pi')*N_dim*...
      (norm(Ainv_1 + Ainv_2, inf)*(2*delta_0u/(domRad0u*sigma_u_hat)) + ...
      norm(Ainv_nk, inf)*(delta_0s/(domRad0s*sigma_s_hat)))


  
if (sup(M_N) >=1)  
  crashNow = crashHere
  M_N
end
 

epsilon_hat = norm( Ainv*Fx_intval', inf)


delta_hat = norm(Ainv_1 + Ainv_2, inf)*delta_0u + ...
            norm(Ainv_nk, inf)*delta_0s
epsilon_NK = (1/(1 - M_N))*(epsilon_hat + delta_hat)

%Note that since r_NewtonKantorovich is equal to 2*epsilon_NK
%we always have r_NewtonKantorovich < 4*epsilon_NK as needed
%for the transversality proof.
r_NewtonKantorivich = 2*epsilon_NK


theta_ball = midrad(mid(thetaBar_intval), sup(r_NewtonKantorivich));
xn_ball = midrad(mid(xn_bar_intval), sup(r_NewtonKantorivich));
tau_ball = midrad(mid(tauBar_intval), sup(r_NewtonKantorivich));


theta_ballTest = sup(norm(theta_ball))
if ( theta_ballTest >= inf(testDom))
   crashNow = crashHere 
end

tau_ballTest = sup(abs(tau_ball))
if (tau_ballTest >= inf(testDom))
   crashNow = crashHere 
end



sigma_s_r = -log((sup(abs(tau_ball)))/(domRad0s))
sigma_u_r = -log((sup(norm(theta_ball)))/(domRad0u))

if (sup(sigma_s_r) > 1)
   sigma_s_r = intval(1) 
end

if (sup(sigma_u_r) > 1)
    sigma_u_r = intval(1)
end


C_Beta = max([sup(2*abs(a)), sup(2*abs(c)), sup(abs(b))])


CP_1 = parm_2ndDiff_1Dbound(tau, P0s1, P0s2, P0s3) + ...
      4*intval('pi')^2*N_dim^2*sigma_s_r^(-2)*domRad0s^(-2)*delta_0s

  
%Again: The 16 below is becaus the manifold is two dimensional
%with complex congugate eigenvalues.
CP_2 = parm_2ndDiff_2Dbound(theta_ball, P0u1, P0u2, P0u3) + ...
      16*intval('pi')^2*N_dim^2*sigma_u_r^(-2)*domRad0u^(-2)*delta_0u

CP_maxes = [CP_1, CP_2];
[maxCP_float, maxCPIndex] = max([sup(CP_1), sup(CP_2)]);
CP = CP_maxes(maxCPIndex)
  


kappa_NK = N_f*C_Beta + CP
NK_Test = (4*kappa_NK*Ainv_norm*epsilon_NK)/(1-M_N)


if (sup(NK_Test) <= 1)
   orbitValidated = 1
else
    orbitValidated = 0
end



%Transversallity Test:

chaosValidation = 0;

if (sup(NK_Test) <1)
    chaosValidation = 1;
end
chaosValidation




  
varargout(1,1)={orbitValidated};
varargout(2,1)={chaosValidation};
varargout(3,1)={r_NewtonKantorivich};

