function varargout = validateOrbit_6D(...
    theta_hat, xn_hat, tau_hat, n_u, n_s, ...
    P0u1, P0u2, P0u3, P0u4, P0u5, P0u6, ...
    P0s1, P0s2, P0s3, P0s4, P0s5, P0s6, ...
    domRad0s, domRad0u, delta_0u, delta_0s, ...
    Nu, Ns, ...
    alpha1, tau1, a1, b1, c1, ...
    alpha2, tau2, a2, b2, c2, ...
    epsilon);


N_dim = 6;
N_f = 10;

  

%Interal version of approximate orbit:  (It is not necessary that the 
%                                        interval guess include the
%                                        floating point guess.  We only
%                                        need some guess.  So we opt for a
%                                        guess with zero raduis i.e. a
%                                        degenerate interval.

thetaBar_intval = intval(theta_hat);
xn_bar_intval = intval(xn_hat);
tauBar_intval = intval(tau_hat);



%check parmeter domain conditions:
theta_norm = norm([sqrt(thetaBar_intval(1)^2 + thetaBar_intval(2)^2),
                   sqrt(thetaBar_intval(3)^2 + thetaBar_intval(4)^2)], ...
                   inf);

              
thetaTest = sup(theta_norm)
testDom = inf(domRad0u)
if ( thetaTest >= inf(testDom))
   crashNow = crashHere 
end

tauTest = max(sup(abs(tauBar_intval(1))), sup(abs(tauBar_intval(2))))
testDom = domRad0s
if (tauTest >= inf(testDom))
   crashNow = crashHere 
end


sigma_s_hat = -log(tauTest/domRad0s)
sigma_u_hat = -log(sup(theta_norm)/domRad0u)


if (sup(sigma_s_hat) > 1)
    sigma_s_hat = intval(1)
end

if (sup(sigma_u_hat) > 1)
    sigma_u_hat = intval(1)
end


%The Newton Vector:
[Fx1_intval, ...
 Fx2_intval, ...
 Fx3_intval] = F_prod_intval_6D(...
                     thetaBar_intval, ...
                     xn_bar_intval, ...
                     tauBar_intval, ...
                     n_u, n_s, ...                          
                     P0u1, P0u2, P0u3, P0u4, P0u5, P0u6, ...
                     P0s1, P0s2, P0s3, P0s4, P0s5, P0s6, ...
                     alpha1, tau1, a1, b1, c1, ...
                     alpha2, tau2, a2, b2, c2, ...
                     epsilon);

                 
Fx_intval = [Fx1_intval, Fx2_intval, Fx3_intval];


errorTest = sup(norm(Fx_intval, inf))


%Interval evaluation of Df(x_0):
A = DF_prod_intval_6D(thetaBar_intval, ...
                      xn_bar_intval, ...
                      tauBar_intval, ...
                      n_u, n_s, ...
                      P0u1, P0u2, P0u3, P0u4, P0u5, P0u6, ...
                      P0s1, P0s2, P0s3, P0s4, P0s5, P0s6, ...
                      alpha1, tau1, a1, b1, c1, ...
                      alpha2, tau2, a2, b2, c2, ...
                      epsilon);



Ainv = inv(A);
Ainv_norm = norm(Ainv, inf)
Ainv_1   = Ainv(:, 1);
Ainv_2   = Ainv(:, 2);
Ainv_3   = Ainv(:, 3);
Ainv_4   = Ainv(:, 4);
Ainv_nkm = Ainv(:, N_dim*(n_s + n_u) - 1);
Ainv_nk  = Ainv(:, N_dim*(n_s + n_u));

%Step 3: Validate the Newton Computation.                           

%In this case there are m_u = 2 pair of complex conjugate eigenvalues for
%the four dimensional manifold and l_s = 2 real distince eigenvalues for 
%the two dimensional manifold.  This accounts for the constants below.

l_s = 2
m_u = 2
M_N = intval('pi')*...
      (norm(Ainv_1 + Ainv_2 + Ainv_3 + Ainv_4, inf)*...
       (4*m_u*delta_0u/(domRad0u*sigma_u_hat)) + ...
      norm(Ainv_nk + Ainv_nkm, inf)*(2*l_s*delta_0s/(domRad0s*sigma_s_hat)))


  
if (sup(M_N) >=1)  
  crashNow = crashHere
  M_N
end
 

epsilon_hat = sup(norm( Ainv*Fx_intval', inf))

delta_hat = sup(norm(Ainv_1 + Ainv_2 + Ainv_3 + Ainv_4, inf)*delta_0u + ...
            norm(Ainv_nk + Ainv_nkm, inf)*delta_0s)
epsilon_NK = (1/(1 - M_N))*(epsilon_hat + delta_hat)

%Note that since r_NewtonKantorivich = 2*epsilon_NK we always
%have r_NewtonKantorivich < 4*epsilon_NK as needed for the 
%transversality argument.
r_NewtonKantorivich = 2*epsilon_NK


theta_ball = midrad(mid(thetaBar_intval), sup(r_NewtonKantorivich));
xn_ball = midrad(mid(xn_bar_intval), sup(r_NewtonKantorivich));
tau_ball = midrad(mid(tauBar_intval), sup(r_NewtonKantorivich));


theta_ballTest = sup(norm([norm(theta_ball(1:2)), ...
                           norm(theta_ball(3:4))], inf))

if ( theta_ballTest >= inf(domRad0u))
   crashNow = crashHere 
end

tau_ballTest = sup(norm(tau_ball, inf))
if (tau_ballTest >= inf(domRad0s))
   crashNow = crashHere 
end

norm_theta_ball = norm([norm(theta_ball(1:2)), ...
                        norm(theta_ball(3:4))], inf);

sigma_s_r = -log((sup(norm(tau_ball, inf)))/(domRad0s))
sigma_u_r = -log(sup(norm_theta_ball)/(domRad0u))

if (sup(sigma_s_r) > 1)
   sigma_s_r = intval(1) 
end

if (sup(sigma_u_r) > 1)
    sigma_u_r = intval(1)
end


C_Beta = max([sup(2*abs(a1)), ...
              sup(2*abs(c1)), ...
              sup(abs(b1)), ...
              sup(2*abs(a2)), ...
              sup(2*abs(c2)), ...
              sup(abs(b2))]) + ...
              2*sup(epsilon)

%TO HERE.............................................................
         

%Again the constants below account for the fact that the stable manifold
%is real and the unstable is 4D with 2 pair of complex conjugate eigs
          
%Use l_s count:
CP_1 = parm_2ndDiff_2Dbound_6D(tau_ball, P0s1, P0s2, P0s3, ...
                                         P0s4, P0s5, P0s6) + ...
      intval('pi')^2*(2*l_s)^2*sigma_s_r^(-2)*domRad0s^(-2)*delta_0s


delta_0u
sigma_u_r
  
%Use m_u count:  
CP_2 = parm_2ndDiff_4Dbound_6D(theta_ball, P0u1, P0u2, P0u3, ...
                                           P0u4, P0u5, P0u6) + ...
      intval('pi')^2*(4*m_u)^2*sigma_u_r^(-2)*domRad0u^(-2)*delta_0u

CP_maxes = [CP_1, CP_2];
[maxCP_float, maxCPIndex] = max([sup(CP_1), sup(CP_2)]);
CP = CP_maxes(maxCPIndex)
  
kappa_NK = N_f*C_Beta + CP
NK_Test = sup((4*kappa_NK*Ainv_norm*epsilon_NK)/(1-M_N))



if (NK_Test <= 1)
   orbitValidated = 1
else
    orbitValidated = 0
end


%Transversallity Test:

chaosValidation = 0;
if (NK_Test <1)
    chaosValidation = 1;
end
chaosValidation

varargout(1,1)={orbitValidated};
varargout(2,1)={chaosValidation};
varargout(3,1)={r_NewtonKantorivich};












