function varargout = validate_coDimOne_stable_henonLike(...
    Lambda, A, Q, Q_inv, s, epsilon_tol, a,b, numDim)



goodComputation = 0;

%==========================================================================
%Validation Algorithm:
%==========================================================================

%STEP 1: ------------------------------------------------------------------
% Pick the size of the neighborhood to look at
R = intval(1)      %intval('2')/intval('1')      
                   %In the notes this is rho instead of R
                   %I'm using R here as rho is a system 
                   %parameter for Lorenz


%STEP 2: ------------------------------------------------------------------
%Compute a bound on the size of the vector field in an R neighborhood
%of the equilibria:

M = abs(2*a);
N_f = 1


%STEP 3: ------------------------------------------------------------------
%Compute the matrix norm bounds:

a_norm = norm(A, inf)
Q1 = norm(Q, inf)
Q2 = norm(Q_inv, inf)


%STEP 6: ------------------------------------------------------------------
%Choose s and compute the combinatorial constants:
%Do step 6 first so we can see how bad consts are...

dim = numDim    %Dimension of the manifolds
C_n = linearApproxCombConst_C(s, dim)
K_n = linearApproxCombConst_K(s, dim)

%STEP 4: ------------------------------------------------------------------

%Find the resonance bound N_0: How many conditions to check by hand in order
%to rule out resonances?

mu_subStar = intval(1);
mu_supStar = intval(0);


for k = 1:(numDim-1)
    this_up = sup(abs(Lambda(k,k)));
    this_down = inf(abs(Lambda(k,k)));
    if this_up > sup(mu_supStar)
       mu_supStar = abs(Lambda(k,k)); 
    end
    if inf(mu_subStar) > this_down
       mu_subStar = abs(Lambda(k,k));
    end
end



mu_subStar
mu_supStar



checkNum = inf(mu_subStar) - sup(mu_supStar); 
N_0 = 1;
while checkNum <= 0
    N_0 = N_0 + 1;
   checkNum = inf(mu_subStar) - sup(mu_supStar)^(N_0);
end

%Answer:
N_0

nZeroTooBig = 0;
if N_0 > 2
   nZeroTooBig = 1 
end

%Now find the bound on the nearest resonance in order to bound the 
%linear operator L.

maxRes = 1/abs(mu_subStar - mu_supStar^N_0)


loopNum = numDim - 1
max_res = intval(0)
counter = 1


for j = 1:loopNum
   for k = 1:loopNum
      this_c = ...
          1/abs(Lambda(k,k)^2 - Lambda(j,j));
      %resBounds(counter) = this_c;
      counter = counter + 1;
      if inf(this_c) > sup(max_res)
        max_res = this_c; 
       end
   end
end


for j = 1:loopNum-1
    lambda_j = Lambda(j,j);
    j
    for m_1 = 1:(loopNum - 1)
        for m_2 = (m_1+1):(loopNum)
            m_2;
            %indices(counter, 1:2) = [m_1, m_2];
            this_c = ...
                1/abs(Lambda(m_1, m_1)*Lambda(m_2, m_2) - lambda_j);
            %resBounds(counter) = this_c;
            counter = counter+1;
             
            if inf(this_c) > sup(max_res)
               max_res = this_c; 
            end
        end
    end
end

maxRes



%STEP 5: ------------------------------------------------------------------
%    Define the bound on the linear operator:
C_tilde = Q1*Q2*maxRes


%Step 7: ------------------------------------------------------------------
%Fix the tolerance and compute a guess for nu:

nu = sqrt(epsilon_tol);
%nu = intval(1);


%Step 8: ------------------------------------------------------------------
%find delta_min and check non-degenerate:

B = C_tilde*N_f*M*C_n


if inf((2*a_norm*B*nu - 1)^2 - 4*B^2*a_norm^2*nu^2) <= 0
    discriminant = inf((2*a_norm*B*nu - 1)^2 - 4*B^2*a_norm^2*nu^2)
    failCondition = negativeDiscriminant 
end



if sup(2*a_norm*B*nu) >= 1
   failCondition = b_not_negative 
end

%Use the quadratic formula to find the min and max delta:
delta_min  = ((1 - 2*a_norm*B*nu) - ...
    sqrt((1 - 2*a_norm*B*nu)^2 - 4*B*a_norm^2*B*nu^2))/(2*B) 




%STEP 9: ------------------------------------------------------------------
%Check that s was small enough


step_9_number = (a_norm*nu + delta_min)/R
if sup((a_norm*nu + delta_min)/R) >= s
   failCondition = s_not_small_enough 
end

%STEP 10: -----------------------------------------------------------------

step_10_number = C_tilde*N_f*M*K_n*(a_norm*nu + delta_min)

if sup(step_10_number) >= 1
   failCondition = lost_Z_bound 
elseif nZeroTooBig == 1
        failCondition = N_0_TooBig
else
    goodComputation = 1
end


%STEP 11: -----------------------------------------------------------------
%Return the values: if we made it here then:

dimensionIs = numDim

%validated domain radius:
nu_is = sup(nu) 

s_is = s
%validated error bound:
delta_is = sup(delta_min)

C_tilde


C_n


K_n


C_T = delta_is/nu_is^2


varargout(1,1)={delta_min};
varargout(2,1)={nu};
varargout(3,1)= {C_tilde};
varargout(4,1)={C_n};
varargout(5,1)={K_n};
varargout(6,1)={goodComputation};



