
clear
format long 


%intvalinit('FastIVmult')
intvalinit('SharpIVmult')


%=================================================================
%   PROGRAM COMPUTES BOTH ONE AND TWO DIMENSIONAL STABLE AND 
%   UNSTABLE MANIFOLDS AT BOTH FIXED POINTS, BY PARAMETERIZATION
%
%   COMPUTE HOMOCLINIC ORBIT:
%
%
%
%
%================================================================



%==========================================================================
%Non-Rigorous Computation of Approximate Solution:
%==========================================================================



%==============================================================
%Computation Controls:
%==============================================================



%Map Parameter Set: Dullin to Lomeli Meiss.
   %Dullin Meiss Parms:
      
   %Subsystem 1 Parameters:
      a_dm1 = 1
      b_dm1 = 0.5
      c_dm1 = 0.5
            
      
      eps_dm1 = 5.5    %boundary near 6.182001
     %eps_dm = 0.32      %worked at 0.4
      mu_dm1 = -2.4
    
   %Subsystem 2 Parameters:   
      a_dm2 = 1
      b_dm2 = 0.5
      c_dm2 = 0.5
            
      
      eps_dm2 = 6.1    %boundary near 6.182001
     %eps_dm = 0.32      %worked at 0.4
      mu_dm2 = -2.4
      
      
      
      
   %Convert Subsystem 1 to Lomeli:   
      a1 = c_dm1
      b1 = b_dm1 - 2*c_dm1
      c1 = c_dm1 + a_dm1 - b_dm1
      
      tau1 =    2*a_dm1*(3+mu_dm1)/(2*a_dm1 - b_dm1)
      alpha1 =  ((9+6*mu_dm1+mu_dm1^2)*a_dm1 - 4*eps_dm1*a_dm1^2 + ...
               4*eps_dm1*a_dm1*b_dm1 - eps_dm1*b_dm1^2)/(2*a_dm1 - b_dm1)^2

%Convert Subsystem 2 to Lomeli:   
      a2 = c_dm2
      b2 = b_dm2 - 2*c_dm2
      c2 = c_dm2 + a_dm2 - b_dm2
      
      tau2 =    2*a_dm2*(3+mu_dm2)/(2*a_dm2 - b_dm2)
      alpha2 =  ((9+6*mu_dm2+mu_dm2^2)*a_dm2 - 4*eps_dm2*a_dm2^2 + ...
               4*eps_dm2*a_dm2*b_dm2 - eps_dm2*b_dm2^2)/(2*a_dm2 - b_dm2)^2




%Order of manifold computation:
  %Order of 4D manifolds:
  Nu = 7;
  %Order of 2D manifold computations 
  Ns = 12;
   
  

  
  %Coupling Strength:
  epsilon_coupling = 5*10^(-7);
  %epsilon_coupling = -2.2*10^(-11) 
  
%==========================================================================
%Generate zeroth and first order problem data:
%==========================================================================

[p0, p1, lambda0_s1, lambda0_s2, ...
         lambda0_u1, lambda0_u2, lambda0_u3, lambda0_u4, ...
         lambda1_u1, lambda1_u2, ...
         lambda1_s1, lambda1_s2, lambda1_s3, lambda1_s4, ...
         xi0_s1, xi0_s2, xi0_u1, xi0_u2, xi0_u3, xi0_u4, ...
         xi1_u1, xi1_u2, xi1_s1, xi1_s2, xi1_s3, xi1_s4] ...
     = firstOrderData_6DHenon(a1, b1, c1, alpha1, tau1, ...
                              a2, b2, c2, alpha2, tau2, ...
                              epsilon_coupling); 
  
 
                          
                          
                          
%=====================================================================
%Low-D Manifolds 
%=====================================================================

%Step 1: Compute 2D Coefficients:

scale0_s1 = 1.5;
scale0_s2 = 1.5;

[A0s, B0s, C0s, D0s, E0s, F0s] = twoDimManifold_6DHenon(...
      a1, b1, c1, alpha1, tau1, ...
      a2, b2, c2, alpha2, tau2, ...
      epsilon_coupling, Ns, p0, ...
      lambda0_s1, lambda0_s2, xi0_s1, xi0_s2, ...
      scale0_s1, scale0_s2);

  
domRad0s = 0.001 
 

 
%=====================================================================
%High-D Manifolds 
%=====================================================================

%Step 1: Compute 4D Coefficients:

scale0_u1 = 0.4; 
scale0_u2 = 0.4;
scale0_u3 = 0.4;
scale0_u4 = 0.4;


floatingPoint4DManifolds = 1

tic

[A0u, B0u, C0u, D0u, E0u, F0u] = fourDimManifold_6DHenon(...
      a1, b1, c1, alpha1, tau1, ...
      a2, b2, c2, alpha2, tau2, ...
      epsilon_coupling, Nu, p0, ...
      lambda0_u1, lambda0_u2, lambda0_u3, lambda0_u4, ...
      xi0_u1, xi0_u2, xi0_u3, xi0_u4, ...
      scale0_u1, scale0_u2, scale0_u3, scale0_u4);

  
fourDim_floatTime = toc  
  
domRad0u = 0.001

n_s = 10
n_u = 10

load orbitData_6D_1;
load orbitData_6D_2;



%Note: theta_0(1) and (2) should be the variables
%      associated with the strongest eigendirections,
%      in this case system 2.  The other variables
%      are assigned accordingly.


theta_0 = [intersectionParameters2(1);
           intersectionParameters2(2);
           intersectionParameters1(1);
           intersectionParameters1(2)];
       
 
%Note: similarly, tau_0(1) should be the variable associated
%      with the system with largest stable eigenvalue.
%
%Note: In both cases the right answer is clear from the eigenvectors
%      when epsilon is small...
       
tau_0 = [intersectionParameters1(60);
         intersectionParameters2(60)];       

[theta_n, xn, tau_n, thisError] = ...
 homoclinicPoint6D_p0_prodSpace(...
    theta_0, tau_0, n_u, n_s, ...
    A0u, B0u, C0u, D0u, E0u, F0u, ...
    A0s, B0s, C0s, D0s, E0s, F0s, ...
    Nu, Ns, ...
    alpha1, tau1, a1, b1, c1, ...
    alpha2, tau2, a2, b2, c2, ...
    epsilon_coupling);
 
thisError


validationParameters = [theta_n; xn; tau_n];



    N = Nu;
    order_1 = Ns;
    epsilon = epsilon_coupling;

    scale_0s1 = scale0_s1;
    scale_0s2 = scale0_s2; 
    scale_0u1 = scale0_u1;
    scale_0u2 = scale0_u2; 
    scale_0u3 = scale0_u3;
    scale_0u4 = scale0_u4;
    

         
   %Convert to Lomeli:   
      a1 = intval(a1)      %c_dm
      b1 = intval(b1)      %b_dm - 2*c_dm
      c1 = intval(c1)      %c_dm + a_dm - b_dm
      
      tau1 =    intval(tau1)  %2*a_dm*(3+mu_dm)/(2*a_dm - b_dm)
      alpha1 =  intval(alpha1) %((9+6*mu_dm+mu_dm^2)*a_dm - 4*eps_dm*a_dm^2 + ...
                           %4*eps_dm*a_dm*b_dm - eps_dm*b_dm^2)/(2*a_dm - b_dm)^2
                    
      a2 = intval(a2)      %c_dm
      b2 = intval(b2)      %b_dm - 2*c_dm
      c2 = intval(c2)      %c_dm + a_dm - b_dm
      
      tau2 =    intval(tau2)  %2*a_dm*(3+mu_dm)/(2*a_dm - b_dm)
      alpha2 =  intval(alpha2) %((9+6*mu_dm+mu_dm^2)*a_dm - 4*eps_dm*a_dm^2 + ...
                           %4*eps_dm*a_dm*b_dm - eps_dm*b_dm^2)/(2*a_dm - b_dm)^2
                    
      epsilon = intval(epsilon)         
                           
                                                    
  [p0, p1, lambda0_s1, lambda0_s2, ...
         lambda0_u1, lambda0_u2, lambda0_u3, lambda0_u4, ...
         lambda1_u1, lambda1_u2, ...
         lambda1_s1, lambda1_s2, lambda1_s3, lambda1_s4, ...
         xi0_s1, xi0_s2, xi0_u1, xi0_u2, xi0_u3, xi0_u4, ...
         xi1_u1, xi1_u2, xi1_s1, xi1_s2, xi1_s3, xi1_s4] ...
     = firstOrderData_intval_6DHenon(a1, b1, c1, alpha1, tau1, ...
                                     a2, b2, c2, alpha2, tau2, ...
                                     epsilon); 
  
          
                                
%=====================================================================
%Manifolds 
%=====================================================================



scale0_s1 = intval(scale_0s1);
scale0_s2 = intval(scale_0s2);

tic

[A0s, B0s, C0s, D0s, E0s, F0s] = twoDimManifold_intval_6DHenon(...
      a1, b1, c1, alpha1, tau1, ...
      a2, b2, c2, alpha2, tau2, ...
      epsilon, order_1, p0, ...
      lambda0_s1, lambda0_s2, xi0_s1, xi0_s2, ...
      scale0_s1, scale0_s2);

toc  
twoDimTime = toc/60  
  

domRad0s = intval(domRad0s);
 

 
%=====================================================================
%High-D Manifolds 
%=====================================================================

%Step 1: Compute 4D Coefficients:

scale0_u1 = intval(scale_0u1); 
scale0_u2 = intval(scale_0u2);
scale0_u3 = intval(scale_0u3);
scale0_u4 = intval(scale_0u4);


interval4DManifoldComputation = 1

tic

[A0u, B0u, C0u, D0u, E0u, F0u] = fourDimManifold_6DHenon_intval(...
      a1, b1, c1, alpha1, tau1, ...
      a2, b2, c2, alpha2, tau2, ...
      epsilon, N, p0, ...
      lambda0_u1, lambda0_u2, lambda0_u3, lambda0_u4, ...
      xi0_u1, xi0_u2, xi0_u3, xi0_u4, ...
      scale0_u1, scale0_u2, scale0_u3, scale0_u4);

  
fourDim_intvalTime = toc  
  
domRad0u = intval(domRad0u)

testCoef = F0u(3,3,3,3)
testRad = rad(F0u(3,3,3,3))

%----------------------Put in Polynomial Data Structure--------------------


%[P0u1, P0u2, P0u3] = convertToPoly_2D(A0u, B0u, C0u, N);

[P0s1, P0s2, P0s3, P0s4, P0s5, P0s6] = ...
    convertToPoly_2D_6D(A0s, B0s, C0s, D0s, E0s, F0s, Ns);


[P0u1, P0u2, P0u3, P0u4, P0u5, P0u6] = ...
    convertToPoly_4D_6D(A0u, B0u, C0u, D0u, E0u, F0u, Nu);



N_f = 10
N_dim = 6

tic

[delta_0s, delta_max_0s, P0s_norm, s0_delta_good, OrderTest_s]...
        = validateStableManifold_2D_6DHenon(...
                P0s1, P0s2, P0s3, P0s4, P0s5, P0s6, ...
                p0, xi0_s1, xi0_s2, xi0_u1, xi0_u2, xi0_u3, xi0_u4, ...
                lambda0_s1, lambda0_s2, N_f, N_dim, domRad0s, Ns, ...
                a1, b1, c1, alpha1, tau1, ...
                a2, b2, c2, alpha2, tau2, epsilon);


 toc
 time_2D_proof = toc/60

s0_delta_good 

OrderTest_s
 
P0s_norm

delta_0s





tic


[delta_0u, delta_max_0u, P0u_norm, u0_delta_good, OrderTest_u]...
        = validateUnstableManifold_4D_6DHenon(...
                P0u1, P0u2, P0u3, P0u4, P0u5, P0u6, ...
                p0, xi0_s1, xi0_s2, xi0_u1, xi0_u2, xi0_u3, xi0_u4, ...
                lambda0_u1, lambda0_u2, lambda0_u3, lambda0_u4, ...
                N_f, N_dim, domRad0u, Nu, ...
                a1, b1, c1, alpha1, tau1, ...
                a2, b2, c2, alpha2, tau2, epsilon);


toc
time_4D_proof = toc/60



OrderTest_u

u0_delta_good

P0u_norm

delta_0u




%stopNow = stopHere

temp_1 = size(validationParameters);
numOrbits = temp_1(2);


tic

theta_hat = validationParameters(1:4);
xn_hat = validationParameters(5:(6*(n_s + n_u)-2))';
tau_hat = validationParameters(...
               (6*(n_s + n_u) - 1):(6*(n_s + n_u)));

                                                   
                          
[goodOrbit, goodChaos, r_NK] = ...
    validateOrbit_6D(...
        theta_hat, xn_hat, tau_hat, n_u, n_s, ...
        P0u1, P0u2, P0u3, P0u4, P0u5, P0u6, ...
        P0s1, P0s2, P0s3, P0s4, P0s5, P0s6, ...
        domRad0s, domRad0u, delta_0u, delta_0s, ...
        Nu, Ns, ...
        alpha1, tau1, a1, b1, c1, ...
        alpha2, tau2, a2, b2, c2, ...
        epsilon)

proofCount = 1;    
  
orbitsValidated(proofCount) = goodOrbit;
chaosValidation(proofCount) = goodChaos;
radii(proofCount) = r_NK;


toc
time_orbitsProof = toc/60




