
clear
format long 


%intvalinit('FastIVmult')
intvalinit('SharpIVmult')


%=================================================================
%   PROGRAM COMPUTES BOTH ONE AND TWO DIMENSIONAL STABLE AND 
%   UNSTABLE MANIFOLDS AT BOTH FIXED POINTS, BY PARAMETERIZATION
%
%   COMPUTE HOMOCLINIC ORBIT:
%
%
%
%
%================================================================



%==========================================================================
%Non-Rigorous Computation of Approximate Solution:
%==========================================================================



%==============================================================
%Computation Controls:
%==============================================================



%Map Parameter Set: Dullin to Lomeli Meiss.
   %Dullin Meiss Parms:
      a_dm = 1
      b_dm = 0.5
      c_dm = 0.5
            
      
      eps_dm = 6.1    %boundary near 6.182001
     %eps_dm = 0.32      %worked at 0.4
      mu_dm = -2.4
    
      
   %Convert to Lomeli:   
      a = c_dm
      b = b_dm - 2*c_dm
      c = c_dm + a_dm - b_dm
      
      tau =    2*a_dm*(3+mu_dm)/(2*a_dm - b_dm)
      alpha =  ((9+6*mu_dm+mu_dm^2)*a_dm - 4*eps_dm*a_dm^2 + ...
               4*eps_dm*a_dm*b_dm - eps_dm*b_dm^2)/(2*a_dm - b_dm)^2



%---for epsilon > 3.5-ish guess_tol ~ 10^(-1) is good.
%---for epsilon ~ 0.2 - 2 neew  guess_tol ~ 2.5 X 10^(-2)
%---what to do close to eps ~ 0.1?
guess_tol = 2.5*10^(-2) 

l = 20

numRad = 10;
numTheta = 250;

%Order of manifold computation:
  %Order of 2D manifolds:
  N = 7;
  %Order of 1D manifold computations 
  order_1 = 9;
   
   
%==========================================================================
%Generate zeroth and first order problem data:
%==========================================================================

[p0, p1, lambda0_s, lambda0_u1, lambda0_u2, ...
         lambda1_u, lambda1_s1, lambda1_s2, ...
         xi0_s, xi0_u1, xi0_u2, ...
         xi1_u, xi1_s1, xi1_s2] ...
     = firstOrderData_3DHenon(a, b, c, alpha, tau) 
  
 
 
%=====================================================================
%One-D Manifolds 
%=====================================================================

%Step 1: Compute 1D Coefficients:

scale_0s = 1.5;

[A0s, B0s, C0s] = oneDimManifold_3DHenon(a, b, c, alpha, tau, ...
     order_1, p0, lambda0_s, xi0_s, scale_0s);


domRad0s = 0.001

               
                           
%stopNow = stopHere                           



%==========================================================================
%Two-D Manifolds:
%==========================================================================

%Step 1: Compute Coefficients:
scale_0u = 0.4;

[A0u, B0u, C0u] = twoDimManifold_3DHenon(a, b, c, alpha, tau, ...
    N, p0, lambda0_u1, lambda0_u2, xi0_u1, xi0_u2, scale_0u, scale_0u);

 
domRad0u = 0.001

 



[delta_0u, delta_0s, P0u_norm, P0s_norm] = ...
    validateManifoldsProof_p0(N, order_1, ...
        scale_0s, scale_0u, domRad0s, ...
        domRad0u, a, b, c, alpha, tau);


delta_0u
delta_0s


%stopNow = stopHere                


%Step 3: Compute fundamental domain point clouds



numPoints = 500
[I0s_left, I0s_right, W0s_fd_left, W0s_fd_right, ...
    I0s, W0s_loc] = ...
    oneDim_fd(A0s, B0s, C0s, order_1, lambda0_s, 0.9*domRad0s, numPoints);



%Step 3: Compute 2D fundamental domains:

[D0u_fd, W0u_fd] = twoDim_fd(A0u, B0u, C0u, N, ...
                             lambda0_u1, 0.95*domRad0u, numRad, numTheta);
                                                                                                   
                         
%==========================================================================
%Step 5: Compute Homoclinic Orbit/Tangle.
%        (Here we will just compute at p0)
%==========================================================================


%----------------Compute Homoclinic Equation Parameters:-------------------




  %Iterate the 2D fundamental domain:
  %n_s_guess = 74 Worked
  n_s_guess = 8  %121 Proof failed...
  n_u = 12
   
  n_s = n_s_guess
  
  W0u_global = ...
        k_image_set(W0u_fd, n_u, alpha, tau, a, b, c);
 
   globalize_fd = inverse_iterate_set_k(W0s_fd_right, n_s_guess, ...
                                        alpha, tau, a, b, c); 
    
   %globalize_fd = k_preImage_set(...
   %    W0s_fd_right, n_s_guess, alpha, tau, a, b, c);
    
   l = 5.5
   
   figure
   hold on
   %plot3(W0u_fd(:,1), W0u_fd(:,2), W0u_fd(:,3), 'g.')
   plot3(W0u_global(:,1), W0u_global(:,2), W0u_global(:,3), 'b.'); 
   %plot3(W0s_fd_right(:,1), W0s_fd_right(:,2), W0s_fd_right(:,3), 'r.'); 
   plot3(globalize_fd(:,1), globalize_fd(:,2), globalize_fd(:,3), 'r.');
   plot3(p0(1), p0(2), p0(3), 'k*'); 
   axis([-l l -l l -l l]);
   
   
   
  %stopNow = stopHere 
  
  
  
  
 
  %Find an intersection Guess:
  bestDist = inf;
  
    
  this_fd_right = k_preImage_set(W0s_fd_right, ...
                  n_s_guess, ...
                  alpha, tau, a, b, c);

                      
  %Cull the points far form p0:              
  [this_fd_right, fd_indices] = cullSetII(this_fd_right, 8, p0);
  
  
  size(this_fd_right)
  
 
  tol_keep = 1.5
  [this_fd_right, fd_indices] = setDistCull(...
      W0u_global, this_fd_right, fd_indices, tol_keep);
  
  
  
    figure
    hold on
   
    plot3(this_fd_right(:, 1), this_fd_right(:, 2), this_fd_right(:, 3), '.g')
    plot3(W0u_global(:, 1), W0u_global(:, 2), W0u_global(:, 3), '.b')
    
    %stopHere = stopNow
     
%%%Check to here...
   

temp1 = size(this_fd_right)
numPoints1 = temp1(1,1)

temp2 = size(this_fd_right)
numPoints2 = temp2(1,1)

pointsFound = 0;



for k = 1:numPoints1
    
  k
  numPoints1
  
  thisIndex = fd_indices(k)      
  tau_0 = I0s_right(thisIndex);

  temp = size(D0u_fd);
  numDiskPoints = temp(1,1);

  thisPS_point = this_fd_right(k);

  distances = zeros(1, numDiskPoints);
  
  newPoint = 0;
  
  for n = 1:numDiskPoints
    distances(n) = norm(thisPS_point - W0u_global(n));
  end    
  [theMin, minIndex] = min(distances);
  
  theta_0 = D0u_fd(minIndex, :);
  [thetaBar, tauBar, error] = homoclinicPoint_p0(...
       theta_0, tau_0, n_u, n_s, A0u, B0u, C0u, ...
       A0s, B0s, C0s, N, order_1, alpha, tau, a, b, c);

  
   
   
   
   %stopNow = stopHere
   
  if (exist('intersectionParameters') == 0 && ...
          (error <= 10^(-6)) && ...
          (norm([thetaBar(1), thetaBar(2), tauBar]) > 10^(-8)))
      
      [thetaBar_prod, xn_bar, tauBar_prod, errorProd] = ...
        homoclinicPoint_p0_prodSpace(thetaBar, tauBar, n_u, n_s, ...
        A0u, B0u, C0u, A0s, B0s, C0s, N, order_1, alpha, tau, a, b, c);
  
      intersectionParameters = ...
        [thetaBar_prod(1), thetaBar_prod(2), ...
         xn_bar', tauBar_prod];
  elseif ((error <= 10^(-6)) && (isnan(error) == 0) && ...
                   (norm([thetaBar(1), thetaBar(2), tauBar]) > 10^(-8)))
    temp = size(intersectionParameters);
    thisSize = temp(1,1);
    newPoint = 0;
    newPointTest = 0;
    for (n = 1:thisSize)
      thisTheta_1 = intersectionParameters(n, 1);
      thisTheta_2 = intersectionParameters(n, 2);
      thisTau = intersectionParameters(n, 3*(n_s + n_u));
     
      dist_1 = abs(thisTheta_1 - thetaBar(1));
      dist_2 = abs(thisTheta_2 - thetaBar(2));
      dist_3 = abs(thisTau - tauBar);
     
      testDist = norm([dist_1 dist_2 dist_3], inf);
      
      if (testDist >= 5*10^(-5))
         newPointTest = newPointTest + 1;
      end
    end
    
    
    
    if (newPointTest == thisSize)
      pointsFound = pointsFound + 1;
      
      [thetaBar_prod, xn_bar, tauBar_prod, errorProd] = ...
        homoclinicPoint_p0_prodSpace(thetaBar, tauBar, n_u, n_s, ...
        A0u, B0u, C0u, A0s, B0s, C0s, N, order_1, alpha, tau, a, b, c);
  
      intersectionParameters(pointsFound, 1:(3*(n_s + n_u))) = ...
        [thetaBar_prod(1), thetaBar_prod(2), xn_bar', tauBar_prod];              
    end
      
  end
  
  
end



temp = size(intersectionParameters)
numPossParm = temp(1,1)


goodParms = 0;
r_u1 = 0.95*domRad0u
r_u0 = norm(lambda0_u1^(-1))*0.95*domRad0u
r_s1 = 0.9*domRad0s
r_s0 = 0.9*domRad0s*norm(lambda0_s)

for n = 1:numPossParm
   
    n
    parmNorm_u = norm(intersectionParameters(n, 1:2))
    parmNorm_s = norm(intersectionParameters(n, 3*(n_s + n_u)))

    
    
    if ((parmNorm_u <= r_u1) && (r_u0 <= parmNorm_u) && ...
        (parmNorm_s <= r_s1) && (r_s0 <= parmNorm_s))
       goodParms = goodParms + 1
    
       tempParms(goodParms, 1:(3*(n_s + n_u))) = ...
           intersectionParameters(n, :);
    end
end

intersectionParameters = tempParms;

  
temp1 = size(intersectionParameters)
numIntersections = temp1(1,1) - 1
  

for k = 1:(numIntersections + 1)

    
%tauBar = intersectionParameters(k, 3*(n_s + n_u));
%thetaBar = intersectionParameters(k, 1:2)
    
    
 %homPoint_s = [horner(tauBar, A0s, order_1), ...
 %             horner(tauBar, B0s, order_1), ...
 %             horner(tauBar, C0s, order_1)];
  
 homPoint_s = ...
   henon3D(intersectionParameters(...
            k, (3*(n_s+n_u)-3):(3*(n_s+n_u)-1)), alpha, tau, a, b, c)';
 
 
        %homPoint_u = evaluate_complexCase(thetaBar(1), thetaBar(2), ...
        %                          A0u, B0u, C0u, N);
       
        
 homPoint_u = ...
     inv_henon3D(intersectionParameters(k, 3:5), alpha, tau, a, b, c)';
        
        timesToIterate = 5                              
        homOrbit_s1 = inverse_iterate_set_k(...
            homPoint_s, n_s, alpha, tau, a, b, c);      
        homOrbit_s2 = iterate_set_k(...
            homPoint_s, timesToIterate, alpha, tau, a, b, c); 
        homOrbit_u1 = iterate_set_k(...
            homPoint_u, n_u, alpha, tau, a, b, c); 
        homOrbit_u2 = inverse_iterate_set_k(...
            homPoint_u, timesToIterate, alpha, tau, a, b, c); 

        
        time1 = linspace(-timesToIterate, 0, timesToIterate+1);
        time2 = linspace(0, n_u, n_u+1);
        time3 = linspace(n_u, n_s + n_u, n_s + 1);
        time4 = linspace(n_s+n_u, ...
                         n_s+n_u+timesToIterate, timesToIterate+1);
        figure
        hold on
        plot(time1(:), flipud(homOrbit_u2(:, 1)), 'g');
        plot(time2(:), homOrbit_u1(:,1), 'g');
        plot(time3(:), flipud(homOrbit_s1(:,1)), 'g');
        plot(time4(:), homOrbit_s2(:,1), 'g');
        plot(time1(:), flipud(homOrbit_u2(:, 1)), 'r.');
        plot(time2(:), homOrbit_u1(:,1), 'k.');
        plot(time3(:), flipud(homOrbit_s1(:,1)), 'k.');
        plot(time4(:), homOrbit_s2(:,1), 'r.');
       
        
end


figure 
hold on
for k = 1:(numIntersections + 1)

    
%tauBar = intersectionParameters(k, 3*(n_s + n_u));
%thetaBar = intersectionParameters(k, 1:2)
    
    
 %homPoint_s = [horner(tauBar, A0s, order_1), ...
 %             horner(tauBar, B0s, order_1), ...
 %             horner(tauBar, C0s, order_1)];
        
 %       homPoint_u = evaluate_complexCase(thetaBar(1), thetaBar(2), ...
 %                                 A0u, B0u, C0u, N);
  
 
  homPoint_s = ...
   henon3D(intersectionParameters(...
            k, (3*(n_s+n_u)-3):(3*(n_s+n_u)-1)), alpha, tau, a, b, c)';
 
 
       
 homPoint_u = ...
     inv_henon3D(intersectionParameters(k, 3:5), alpha, tau, a, b, c)';
        
 
 
        timesToIterate = 5                              
        homOrbit_s1 = inverse_iterate_set_k(...
            homPoint_s, n_s, alpha, tau, a, b, c);      
        homOrbit_s2 = iterate_set_k(...
            homPoint_s, timesToIterate, alpha, tau, a, b, c); 
        homOrbit_u1 = iterate_set_k(...
            homPoint_u, n_u, alpha, tau, a, b, c); 
        homOrbit_u2 = inverse_iterate_set_k(...
            homPoint_u, timesToIterate, alpha, tau, a, b, c); 

        
     
            
        
        plot3(homOrbit_s1(:,1), homOrbit_s1(:,2), homOrbit_s1(:,3), 'k.');
        plot3(homOrbit_s2(:,1), homOrbit_s2(:,2), homOrbit_s2(:,3), 'g.');
        plot3(homOrbit_u1(:,1), homOrbit_u1(:,2), homOrbit_u1(:,3), 'k.');
        plot3(homOrbit_u2(:,1), homOrbit_u2(:,2), homOrbit_u2(:,3), 'g.');

        
        
end

  
figure 
hold on

temp = size(intersectionParameters)
numIntersections = temp(1,1)

t = linspace(0,2*pi,100);
h=0;
k=0;
r=r_u1;
x = r*cos(t)+h;
y = r*sin(t)+k;
plot(x,y);

r=r_u0;
x = r*cos(t)+h;
y = r*sin(t)+k;
plot(x,y);


for n = 1:numIntersections
   plot(intersectionParameters(n, 1), intersectionParameters(n, 2), 'ro')
   
end
  
  
  
  
  
  
  