clear
format long 

%intvalinit('FastIVmult')
intvalinit('SharpIVmult')

%=================================================================
% This program executes the first 34 computer assisted proofs of     
% the existence of homoclinic orbits in section 6.  The important
% part begins at line 117 where the user must decide which 
% data set to load. Only load one data set per run of the 
% program.
%================================================================


%==========================================================================
%Non-Rigorous Computation of Approximate Solution:
%==========================================================================



%==============================================================
%Computation Controls:
%==============================================================



%Map Parameter Set: Dullin to Lomeli Meiss.
   %Dullin Meiss Parms:
      a_dm = 1
      b_dm = 0.5
      c_dm = 0.5
            
      
      eps_dm = 5.5    %boundary near 6.182001
     %eps_dm = 0.32      %worked at 0.4
      mu_dm = -2.4
    
      
   %Convert to Lomeli:   
      a = c_dm
      b = b_dm - 2*c_dm
      c = c_dm + a_dm - b_dm
      
      tau =    2*a_dm*(3+mu_dm)/(2*a_dm - b_dm)
      alpha =  ((9+6*mu_dm+mu_dm^2)*a_dm - 4*eps_dm*a_dm^2 + ...
               4*eps_dm*a_dm*b_dm - eps_dm*b_dm^2)/(2*a_dm - b_dm)^2



%---for epsilon > 3.5-ish guess_tol ~ 10^(-1) is good.
%---for epsilon ~ 0.2 - 2 neew  guess_tol ~ 2.5 X 10^(-2)
%---what to do close to eps ~ 0.1?
guess_tol = 2.5*10^(-2) 



%Order of manifold computation:
  %Order of 2D manifolds:
  N = 25;
  %Order of 1D manifold computations 
  order_1 = 50;
   
   
%==========================================================================
%Generate zeroth and first order problem data:
%==========================================================================

[p0, p1, lambda0_s, lambda0_u1, lambda0_u2, ...
         lambda1_u, lambda1_s1, lambda1_s2, ...
         xi0_s, xi0_u1, xi0_u2, ...
         xi1_u, xi1_s1, xi1_s2] ...
     = firstOrderData_3DHenon(a, b, c, alpha, tau) 
   
%=====================================================================
%One-D Manifolds 
%=====================================================================

%Step 1: Compute 1D Coefficients:

scale_0s = 2;

[A0s, B0s, C0s] = oneDimManifold_3DHenon(a, b, c, alpha, tau, ...
     order_1, p0, lambda0_s, xi0_s, scale_0s);



domRad0s = 0.9
                      

%Step 3: Compute fundamental domain point clouds


%==========================================================================
%Two-D Manifolds:
%==========================================================================

%Step 1: Compute Coefficients:
scale_0u = 1.5;

[A0u, B0u, C0u] = twoDimManifold_3DHenon(a, b, c, alpha, tau, ...
    N, p0, lambda0_u1, lambda0_u2, xi0_u1, xi0_u2, scale_0u, scale_0u);

 
domRad0u = 0.4
                                                                                                
  
%----------------Homoclinic Parameters From Memory:------------------------

  %LOAD GUESSES: comment out all load commands except the one you want
  %              to run. 
 
  %Proof 1:
  %     n_s = 3
  %     n_u = 3  
  %     load homData_eps5p5_ns_3_nu_3

  
  %Proof 2:
  %      n_s = 5
  %      n_u = 3
  %      load homData_eps5p5_ns_5_nu_3
  

  %Proof 3:
  %      n_s = 6
  %      n_u = 3
  %      load homData_eps5p5_ns_6_nu_3

  %Proof 4:
  %      n_s = 8
  %      n_u = 3
  %      load homData_eps5p5_ns_8_nu_3
  

  %Proof 5: 
  %     n_s = 9
  %     n_u = 3
  %     load homData_eps5p5_ns_9_nu_3
  
  %Proof 6:
  %     n_s = 10
  %     n_u = 3
  %     load homData_eps5p5_ns_10_nu_3
   
  
  %Proof 7:
  %    n_s = 11
  %    n_u = 3
  %    load homData_eps5p5_ns_11_nu_3
  
  
  %Proof 8:
  %  n_s = 12
  %  n_u = 3
  %  load homData_eps5p5_ns_12_nu_3
  
  
  %Proof 9:
  %  n_s = 13
  %  n_u = 3
  %  load homData_eps5p5_ns_13_nu_3
  
  %Proof 10:
  %  n_s = 14
  %  n_u = 3
  %  load homData_eps5p5_ns_14_nu_3
  
 
  %Proof 11:
  %  n_s = 15
  %  n_u = 3
  %  load homData_eps5p5_ns_15_nu_3
  
  %Proof 12:
  %  n_s = 16
  %  n_u = 3
  %  load homData_eps5p5_ns_16_nu_3 
    
   
  %Proof :
  %      n_s = 17
  %      n_u = 3
  %      load homData_eps5p5_ns_17_nu_3
  
  
  
  %Proof:
    n_s = 18
    n_u = 3
    load homData_eps5p5_ns_18_nu_3     
  


tic        

[orbitsValidated, chaosValidations, validatedRadii] = ...
     validateChaosProof(intersectionParameters, ...
            n_s, n_u, N, order_1, scale_0s, scale_0u, ...
            domRad0s, domRad0u, a, b, c, alpha, tau);
   

proofTime = toc   

             
orbitsValidated
chaosValidations
validatedRadii
   
%=====================================================================
  % Plotting:
%====================================================================   
  
temp1 = size(intersectionParameters)
numIntersections = temp1(1,1) - 1
  

for k = 1:(numIntersections + 1)

    
%tauBar = intersectionParameters(k, 3*(n_s + n_u));
%thetaBar = intersectionParameters(k, 1:2)
    
    
 %homPoint_s = [horner(tauBar, A0s, order_1), ...
 %             horner(tauBar, B0s, order_1), ...
 %             horner(tauBar, C0s, order_1)];
  
 homPoint_s = ...
   henon3D(intersectionParameters(...
            k, (3*(n_s+n_u)-3):(3*(n_s+n_u)-1)), alpha, tau, a, b, c)';
 
 
        %homPoint_u = evaluate_complexCase(thetaBar(1), thetaBar(2), ...
        %                          A0u, B0u, C0u, N);
       
        
 homPoint_u = ...
     inv_henon3D(intersectionParameters(k, 3:5), alpha, tau, a, b, c)';
        
        timesToIterate = 10                              
        homOrbit_s1 = inverse_iterate_set_k(...
            homPoint_s, n_s, alpha, tau, a, b, c);      
        homOrbit_s2 = iterate_set_k(...
            homPoint_s, timesToIterate, alpha, tau, a, b, c); 
        homOrbit_u1 = iterate_set_k(...
            homPoint_u, n_u, alpha, tau, a, b, c); 
        homOrbit_u2 = inverse_iterate_set_k(...
            homPoint_u, timesToIterate, alpha, tau, a, b, c); 

        
        time1 = linspace(-timesToIterate, 0, timesToIterate+1);
        time2 = linspace(0, n_u, n_u+1);
        time3 = linspace(n_u, n_s + n_u, n_s + 1);
        time4 = linspace(n_s+n_u, ...
                         n_s+n_u+timesToIterate, timesToIterate+1);
        figure
        hold on
        plot(time1(:), flipud(homOrbit_u2(:, 1)), 'g');
        plot(time2(:), homOrbit_u1(:,1), 'g');
        plot(time3(:), flipud(homOrbit_s1(:,1)), 'g');
        plot(time4(:), homOrbit_s2(:,1), 'g');
        plot(time1(:), flipud(homOrbit_u2(:, 1)), 'r.');
        plot(time2(:), homOrbit_u1(:,1), 'k.');
        plot(time3(:), flipud(homOrbit_s1(:,1)), 'k.');
        plot(time4(:), homOrbit_s2(:,1), 'r.');
       
        
end


figure 
hold on
for k = 1:(numIntersections + 1)

    
%tauBar = intersectionParameters(k, 3*(n_s + n_u));
%thetaBar = intersectionParameters(k, 1:2)
    
    
 %homPoint_s = [horner(tauBar, A0s, order_1), ...
 %             horner(tauBar, B0s, order_1), ...
 %             horner(tauBar, C0s, order_1)];
        
 %       homPoint_u = evaluate_complexCase(thetaBar(1), thetaBar(2), ...
 %                                 A0u, B0u, C0u, N);
  
 
  homPoint_s = ...
   henon3D(intersectionParameters(...
            k, (3*(n_s+n_u)-3):(3*(n_s+n_u)-1)), alpha, tau, a, b, c)';
 
 
       
 homPoint_u = ...
     inv_henon3D(intersectionParameters(k, 3:5), alpha, tau, a, b, c)';
        
 
 
        timesToIterate = 10                              
        homOrbit_s1 = inverse_iterate_set_k(...
            homPoint_s, n_s, alpha, tau, a, b, c);      
        homOrbit_s2 = iterate_set_k(...
            homPoint_s, timesToIterate, alpha, tau, a, b, c); 
        homOrbit_u1 = iterate_set_k(...
            homPoint_u, n_u, alpha, tau, a, b, c); 
        homOrbit_u2 = inverse_iterate_set_k(...
            homPoint_u, timesToIterate, alpha, tau, a, b, c); 

        
     
            
        
        plot3(homOrbit_s1(:,1), homOrbit_s1(:,2), homOrbit_s1(:,3), 'k.');
        plot3(homOrbit_s2(:,1), homOrbit_s2(:,2), homOrbit_s2(:,3), 'g.');
        plot3(homOrbit_u1(:,1), homOrbit_u1(:,2), homOrbit_u1(:,3), 'k.');
        plot3(homOrbit_u2(:,1), homOrbit_u2(:,2), homOrbit_u2(:,3), 'g.');

        
        
end

  
  
  
  
  
  
  
  
  
  
  
