clear 
format long



%intvalinit('FastIVmult')
intvalinit('SharpIVmult')


%==========================================================================
%---------------Rigorous Parameterization Computations---------------------
%
%This program uses the MatLab package ``IntLab''.
%Before using run from the command prompt:
%
%>> startintlab
%
%
%   Compute rigorous Coefficients and rigorous defects.
%
%
%
%Default evaluation of univariate interval polynomials (always matlab/polyval for no interval data):
%  'EvaluateUPolyHorner'     Evaluation by Horner's scheme, slow for large degree
%                               e.g. (x-3).*x+5
%  'EvaluateUPolyPower'      Evaluation by sums of scaled powers for degree>=20
%                               e.g. sum([1 -3 5].*[1 x x^2])
%  'EvaluateUPoly'           res = 'EvaluateUPolyHorner'
%                                  'EvaluateUPolyPower'
%
%Default evaluation of multivariate interval polynomials:
%  'EvaluateMPolyHorner'     Evaluation by recursive Horner's scheme
%  'EvaluateMPolyPower'      Evaluation by sums of scaled powers
%  'EvaluateMPoly'           res = 'EvaluateMPolyHorner'
%                                  'EvaluateMPolyPower'
%
%--------------------------------------------------------------------------
%==========================================================================






%parm controls:
order_1 = 25


%Map Parameter Set: Dullin to Lomeli Meiss.
   %Dullin Meiss Parms:
      a_dm = intval('1')
      b_dm = intval('0.5')
      c_dm = intval('0.5')
            
      
      
      eps_dm = intval('2.11')    %boundary near 6.182001
      mu_dm = intval('-2.4')
      
      
   %Convert to Lomeli:   
      a = c_dm
      b = b_dm - 2*c_dm
      c = c_dm + a_dm - b_dm
      
      tau =    2*a_dm*(3+mu_dm)/(2*a_dm - b_dm)
      alpha =  ((9+6*mu_dm+mu_dm^2)*a_dm - 4*eps_dm*a_dm^2 + ...
               4*eps_dm*a_dm*b_dm - eps_dm*b_dm^2)/(2*a_dm - b_dm)^2

                      
           
[p0, p1, lambda0_s, lambda0_u1, lambda0_u2, ...
         lambda1_u, lambda1_s1, lambda1_s2, ...
         xi0_s, xi0_u1, xi0_u2, ...
         xi1_u, xi1_s1, xi1_s2] ...
       = firstOrderData_intval_3DHenon(a, b, c, alpha, tau);
     
   
   

           
%=====================================================================
%One-D Manifolds 
%=====================================================================

%Step 1: Compute 1D Coefficients:


scale = intval('20');


[A0s, B0s, C0s] = oneDimManifold_intval_3DHenon(a, b, c, alpha, tau, ...
     order_1, p0, lambda0_s, xi0_s, scale);

 
scale = intval('50') 
 
[A1u, B1u, C1u] = oneDimManifold_intval_3DHenon(a, b, c, alpha, tau, ...
    order_1, p1, lambda1_u, xi1_u, scale);           
           
           

firstOrder = 1

           
%==========================================================================
%Two-D Manifolds:
%==========================================================================

%Step 1: Compute Coefficients:
 scale1 = intval('25');
 scale2 = intval('25');
 
 N = 10;
 

 
 tic
 
 [A0u, B0u, C0u] = twoDimManifold_intval_3DHenon(a, b, c, alpha, tau, ...
     N, p0, lambda0_u1, lambda0_u2, xi0_u1, xi0_u2, scale1, scale2);
 
 
 [A1s, B1s, C1s] = twoDimManifold_intval_3DHenon(a, b, c, alpha, tau, ...
     N, p1, lambda1_s1, lambda1_s2, xi1_s1, xi1_s2, scale1, scale2);           
            
   
 time_1 = toc/60


%----------------------Put in Polynomial Data Structure--------------------



[P0u1, P0u2, P0u3, P1s1, P1s2, P1s3] = convertToPoly_2D(A0u, B0u, C0u, ...
                                                     A1s, B1s, C1s, N);


[P0s1, P0s2, P0s3, P1u1, P1u2, P1u3] = convertToPoly_1D(A1u, B1u, C1u, ...
                                                   A0s, B0s, C0s, order_1);                                                 



%Phase Space Bounds: 1D  
domRad0s = intval('0.2');
domRad0u = intval('0.025')


%Bounds.
max1 = polyval(mag(P0s1-p0(1)), domRad0s);
max2 = polyval(mag(P0s2-p0(2)), domRad0s);
max3 = polyval(mag(P0s3-p0(3)), domRad0s);
P0s_norm = max([sup(max1), sup(max2), sup(max3)])


max1 = polyval(mag(P1u1-p1(1)), domRad0u);
max2 = polyval(mag(P1u2-p1(2)), domRad0u);
max3 = polyval(mag(P1u3-p1(3)), domRad0u);
P1u_norm = max([sup(max1), sup(max2), sup(max3)])


%A Posteriori Errors: 1D                                               
                                               
                                               
                                               
epsilon_s1D = Error_intval_1D(P0s1, P0s2, P0s3, alpha, tau, ...
                               a, b, c, lambda0_s, order_1, domRad0s)

                          
                          
epsilon_u1D = Error_intval_1D(P1u1, P1u2, P1u3, alpha, tau, ...
                               a, b, c, lambda1_u, order_1, domRad0u)


%Phase Space Bounds: 2D

domRad1s = intval('0.025');

max1 = polyval(mag(P1s1 - p1(1)), domRad1s, domRad1s);
max2 = polyval(mag(P1s2 - p1(2)), domRad1s, domRad1s);
max3 = polyval(mag(P1s3 - p1(3)), domRad1s, domRad1s);
P1s_norm = max([sup(max1), sup(max2), sup(max3)])
              
domRad1u = intval('0.025');

max1 = polyval(mag(P0u1- p0(1)), domRad1u, domRad1u);
max2 = polyval(mag(P0u2- p0(2)), domRad1u, domRad1u);
max3 = polyval(mag(P0u3- p0(3)), domRad1u, domRad1u);
P0u_norm = max([sup(max1), sup(max2), sup(max3)])                          
                     
%A-Posteriori Errors: 2D                           


epsilon_u2D = Error_intval_2D(P0u1, P0u2, P0u3, alpha, tau, ...
                               a, b, c, lambda0_u1, lambda0_u2, N, domRad1u)

                           
                                               
epsilon_s2D = Error_intval_2D(P1s1, P1s2, P1s3, alpha, tau, ...
                               a, b, c, lambda1_s1, lambda1_s2, N, domRad1s)


                           

%%END OF COMPUTATIONS.                

                           










