

clear
format long 


%intvalinit('FastIVmult')
intvalinit('SharpIVmult')


%=================================================================
%   PROGRAM COMPUTES BOTH ONE AND TWO DIMENSIONAL STABLE AND 
%   UNSTABLE MANIFOLDS AT BOTH FIXED POINTS, BY PARAMETERIZATION
%
%   COMPUTE HOMOCLINIC ORBIT:
%
%
%
%
%================================================================



%==========================================================================
%Non-Rigorous Computation of Approximate Solution:
%==========================================================================



%==============================================================
%Computation Controls:
%==============================================================



%Map Parameter Set: Dullin to Lomeli Meiss.
   %Dullin Meiss Parms:
      a_dm = 1
      b_dm = 0.5
      c_dm = 0.5
            
      
      eps_dm = 4.5    %boundary near 6.182001
     %eps_dm = 0.32      %worked at 0.4
      mu_dm = -2.4
      
      
   %Convert to Lomeli:   
      a = c_dm
      b = b_dm - 2*c_dm
      c = c_dm + a_dm - b_dm
      
      tau =    2*a_dm*(3+mu_dm)/(2*a_dm - b_dm)
      alpha =  ((9+6*mu_dm+mu_dm^2)*a_dm - 4*eps_dm*a_dm^2 + ...
               4*eps_dm*a_dm*b_dm - eps_dm*b_dm^2)/(2*a_dm - b_dm)^2



%---for epsilon > 3.5-ish guess_tol ~ 10^(-1) is good.
%---for epsilon ~ 0.2 - 2 neew  guess_tol ~ 2.5 X 10^(-2)
%---what to do close to eps ~ 0.1?
guess_tol = 2.5*10^(-2) 

l = 20

numRad = 25;
numTheta = 100;

%Order of manifold computation:
  %Order of 2D manifolds:
  N = 9;
  %Order of 1D manifold computations 
  order_1 = 7;
   
   
%==========================================================================
%Generate zeroth and first order problem data:
%==========================================================================

[p0, p1, lambda0_s, lambda0_u1, lambda0_u2, ...
         lambda1_u, lambda1_s1, lambda1_s2, ...
         xi0_s, xi0_u1, xi0_u2, ...
         xi1_u, xi1_s1, xi1_s2] ...
     = firstOrderData_3DHenon(a, b, c, alpha, tau) 
  
 
 
%=====================================================================
%One-D Manifolds 
%=====================================================================

%Step 1: Compute 1D Coefficients:

scale_0s = 1.0;

[A0s, B0s, C0s] = oneDimManifold_3DHenon(a, b, c, alpha, tau, ...
     order_1, p0, lambda0_s, xi0_s, scale_0s);



domRad0s = 10^(-2)

error_0s = Error_1D(A0s, B0s, C0s, alpha, tau, ...
                               a, b, c, lambda0_s, order_1, domRad0s)


                         



%Step 3: Compute fundamental domain point clouds


numPoints = 500
[I0s_left, I0s_right, W0s_fd_left, W0s_fd_right, ...
    I0s, W0s_loc] = ...
    oneDim_fd(A0s, B0s, C0s, order_1, lambda0_s, domRad0s, numPoints);



%==========================================================================
%Two-D Manifolds:
%==========================================================================

%Step 1: Compute Coefficients:
scale_0u = 1.25;

[A0u, B0u, C0u] = twoDimManifold_3DHenon(a, b, c, alpha, tau, ...
    N, p0, lambda0_u1, lambda0_u2, xi0_u1, xi0_u2, scale_0u, scale_0u);

 
domRad0u = 10^(-3)


error_0u = Error_2D(A0u, B0u, C0u, alpha, tau, ...
                    a, b, c, lambda0_u1, lambda0_u2, N, domRad0u)






%Step 3: Compute 2D fundamental domains:

[D0u_fd, W0u_fd] = twoDim_fd(A0u, B0u, C0u, N, ...
                             lambda0_u1, domRad0u, numRad, numTheta);
                                                                                                   
                         
%==========================================================================
%Step 5: Compute Homoclinic Orbit/Tangle.
%        (Here we will just compute at p0)
%==========================================================================


%----------------Compute Homoclinic Equation Parameters:-------------------


  %Iterate the 2D fundamental domain:
  n_s_guess = 9
  n_u = 12
  
  
  n_s = n_s_guess
  
  W0u_global = ...
        k_image_set(W0u_fd, n_u, alpha, tau, a, b, c);
 
   globalize_fd = inverse_iterate_set_k(W0s_fd_right, n_s_guess, ...
                                        alpha, tau, a, b, c); 
    
   
   %globalize_fd = k_preImage_set(W0s_fd_right, n_s_guess, alpha, tau, a, b, c);
   
   figure
   hold on
   %plot3(W0u_fd(:,1), W0u_fd(:,2), W0u_fd(:,3), 'b.')
   plot3(W0u_global(:,1), W0u_global(:,2), W0u_global(:,3), 'b.'); 
   %plot3(W0s_fd_right(:,1), W0s_fd_right(:,2), W0s_fd_right(:,3), 'r.'); 
   plot3(globalize_fd(:,1), globalize_fd(:,2), globalize_fd(:,3), 'r.');
   plot3(p0(1), p0(2), p0(3), 'k*'); 
   axis([-l l -l l -l l]);

   
  %stopNow = stopHere 
   
  
 
   
  %[theta_0, tau_0, n_s, stableIndex] = homoclinicInitialGuessII(...
  %    D0u_fd, I0s_left, I0s_right, ...
  %    W0u_global, W0s_fd_left, W0s_fd_right, ...
  %    alpha, tau, a, b, c, p0, n_s_guess)
  

  %Find an intersection Guess:
  bestDist = inf;
  
    
  this_fd_right = k_preImage_set(W0s_fd_right, ...
                  n_s_guess, ...
                  alpha, tau, a, b, c);

                      
  %Cull the points far form p0:              
  [this_fd_right, fd_indices] = cullSetII(this_fd_right, 8, p0);
  
  size(this_fd_right)
  
  tol_keep = 5*10^(-1)
  [this_fd_right, fd_indices] = setDistCull(...
      W0u_global, this_fd_right, fd_indices, tol_keep);
  
  
  size(this_fd_right)
  
  
  figure
  hold on
   
  plot3(this_fd_right(:, 1), this_fd_right(:, 2), this_fd_right(:, 3), '.g')
  plot3(W0u_global(:, 1), W0u_global(:, 2), W0u_global(:, 3), '.b')
    
    %stopHere = stopNow
     
%%%Check to here...
    
    
[thisDist_right, pointIndex_theta2, pointIndex_right, global_index] = ...
            setDistII(W0u_global, this_fd_right, fd_indices, p0);
  
   
bestDist = thisDist_right
theta_0 = D0u_fd(pointIndex_theta2, :)
tau_0 = I0s_right(pointIndex_right)
  
 
   
[thetaBar, tauBar, error] = homoclinicPoint_p0(theta_0, tau_0, n_u, n_s, ...
    A0u, B0u, C0u, A0s, B0s, C0s, N, order_1, alpha, tau, a, b, c)



[thetaBar_prod, xn_bar, tauBar_prod, errorProd] = ...
    homoclinicPoint_p0_prodSpace(thetaBar, tauBar, n_u, n_s, ...
    A0u, B0u, C0u, A0s, B0s, C0s, N, order_1, alpha, tau, a, b, c);
  
   
intersectionParameters = [thetaBar_prod(1), thetaBar_prod(2), ...
                          xn_bar', tauBar_prod];


intersectionParameters



[orbitValidated, chaosValidation, validatedRadius] = ...
      validateChaosProof(intersectionParameters, ...
            n_s, n_u, N, order_1, scale_0s, scale_0u, ...
            domRad0s, domRad0u, a, b, c, alpha, tau);

        
numContSteps = 113
       
orbitsValidated = zeros(numContSteps+1, 1);
chaosValidations = zeros(numContSteps+1, 1);
validatedRadii = intval(zeros(numContSteps+1, 1));        
                
orbitsValidated(1) = orbitValidated;
chaosValidations(1) = chaosValidation;
validatedRadii(1) = validatedRadius;
   

normalLoop = 1;
numFailuers = 0;
lastWorkingEps = eps_dm
failLoopFirred = 0
rescaled_s = 0
rescaled_u = 0

for m = 1:numContSteps
         
      a_dm = 1
      b_dm = 0.5
      c_dm = 0.5
       
      mu_dm = -2.4 
      
      
      if (normalLoop == 1)
       eps_dm = eps_dm - 0.05    %boundary near 6.182001
      end 
      
      temp_1 = size(intersectionParameters)
      m = temp_1(1,1)
      k = temp_1(1,2)
      
      theta_0 = intersectionParameters(m, 1:2);
      tau_0 = intersectionParameters(m, k);
      
      
   %Convert to Lomeli:   
      a = c_dm
      b = b_dm - 2*c_dm
      c = c_dm + a_dm - b_dm
      
      tau =    2*a_dm*(3+mu_dm)/(2*a_dm - b_dm)
      alpha =  ((9+6*mu_dm+mu_dm^2)*a_dm - 4*eps_dm*a_dm^2 + ...
               4*eps_dm*a_dm*b_dm - eps_dm*b_dm^2)/(2*a_dm - b_dm)^2



      [p0, p1, lambda0_s, lambda0_u1, lambda0_u2, ...
             lambda1_u, lambda1_s1, lambda1_s2, ...
             xi0_s, xi0_u1, xi0_u2, ...
             xi1_u, xi1_s1, xi1_s2] ...
        = firstOrderData_3DHenon(a, b, c, alpha, tau) 
      
      
       
%=====================================================================
%One-D Manifolds 
%=====================================================================
         
    %Step 1: Compute 1D Coefficients:
    
      scale_0s = 1.0;

      [A0s, B0s, C0s] = oneDimManifold_3DHenon(a, b, c, alpha, tau, ...
         order_1, p0, lambda0_s, xi0_s, scale_0s);


      domRad0s = 10^(-2)

%==========================================================================
%Two-D Manifolds:
%==========================================================================

    %Step 1: Compute Coefficients:
      scale_0u = 1.25;

      [A0u, B0u, C0u] = twoDimManifold_3DHenon(a, b, c, alpha, tau, ...
          N, p0, lambda0_u1, lambda0_u2, xi0_u1, xi0_u2, scale_0u, scale_0u);

 
      domRad0u = 10^(-3)
 
     
    
    [thetaBar, tauBar, error] = ...
        homoclinicPoint_p0(theta_0, tau_0, n_u, n_s, ...
        A0u, B0u, C0u, A0s, B0s, C0s, N, order_1, alpha, tau, a, b, c)
    
    
    error
         
    
    [thetaBar_prod, xn_bar, tauBar_prod, errorProd] = ...
          homoclinicPoint_p0_prodSpace(thetaBar, tauBar, n_u, n_s, ...
          A0u, B0u, C0u, A0s, B0s, C0s, N, order_1, alpha, tau, a, b, c);
      
      
    errorProd
      
      
    converged = 1;
    
    normThisTheta = norm(theta_0)
    normThisTau = abs(tau_0)

    if ((normThisTheta < 10^(-6)) || (normThisTau < 10^(-6)))
      crashNow = crashHere
    end
        
    
     if ((normThisTheta >= 0.7*domRad0u) ...
             && (rescaled_u == 0) && (converged == 1))
          while (normThisTheta >= 0.7*domRad0u)
            disp('   Rescaling: theta       ')
            theta_0
            scaleTheta = abs(lambda0_u1^(-1))
            newAngle = atan2(imag(lambda0_u1), real(lambda0_u1))
            rotMatrix = [cos(newAngle), -sin(newAngle);
                 sin(newAngle), cos(newAngle)]
            transMatrix = inv(rotMatrix)
            theta_0 = (scaleTheta*transMatrix*[theta_0(1);
                                      theta_0(2)])'
            n_u = n_u + 1                   
            
            rescaled_u = 1
            normThisTheta = norm(theta_0)
          end
      end

    if ((normThisTau >= 0.7*domRad0s) ...
             && (rescaled_s == 0) && (converged == 1))
        while (normThisTau >= 0.7*domRad0s)
         disp('    Rescaling: tau     ') 
          tau_0
          tau_0 = tau_0*lambda0_s
          n_s = n_s + 1
          
          rescaled_s = 1
          normThisTau = abs(tau_0)       
        end
    end
    
   
    if ((rescaled_s == 1) || (rescaled_u == 1))
    [thetaBar, tauBar, error] = ...
        homoclinicPoint_p0(theta_0, tau_0, n_u, n_s, ...
        A0u, B0u, C0u, A0s, B0s, C0s, N, order_1, alpha, tau, a, b, c)
    
    
    error
         
    
    [thetaBar_prod, xn_bar, tauBar_prod, errorProd] = ...
          homoclinicPoint_p0_prodSpace(thetaBar, tauBar, n_u, n_s, ...
          A0u, B0u, C0u, A0s, B0s, C0s, N, order_1, alpha, tau, a, b, c);
      
      
    errorProd
    end    
    
    
    if ((errorProd > 10^(-9)) || isnan(errorProd))
        converged = 0 
        numFailuers = numFailuers + 1
    end

    m
    eps_dm
    n_s
    n_u
    
    if (converged == 1)
        
        rescaled_u = 0
        rescaled_s = 0
        normalLoop = 1
        
        lastWorkingEps = eps_dm
        
        converged
        numFailuers = 0;
        
        
        intersectionParameters(m+1, 1:(3*(n_s+n_u))) = ...
            [thetaBar_prod(1), thetaBar_prod(2), ...
                               xn_bar', tauBar_prod];
                     
        assignedParmMatrix = 1                  
        size(intersectionParameters)                  
         
        [orbitValidated, chaosValidation, validatedRadius] = ...
              validateChaosProof(intersectionParameters(m+1, :), ...
                    n_s, n_u, N, order_1, scale_0s, scale_0u, ...
                    domRad0s, domRad0u, a, b, c, alpha, tau);
        
        if orbitValidated == 0
           crashNow = crashHere 
        end
        
        homPoint_s = [horner(tauBar, A0s, order_1), ...
              horner(tauBar, B0s, order_1), ...
              horner(tauBar, C0s, order_1)];
        
        homPoint_u = evaluate_complexCase(thetaBar(1), thetaBar(2), ...
                                  A0u, B0u, C0u, N);
        
        timesToIterate = 10                              
        homOrbit_s1 = inverse_iterate_set_k(...
            homPoint_s, n_s, alpha, tau, a, b, c);      
        homOrbit_s2 = iterate_set_k(...
            homPoint_s, timesToIterate, alpha, tau, a, b, c); 
        homOrbit_u1 = iterate_set_k(...
            homPoint_u, n_u, alpha, tau, a, b, c); 
        homOrbit_u2 = inverse_iterate_set_k(...
            homPoint_u, timesToIterate, alpha, tau, a, b, c); 

        
        if (mod(m, 20) == 1)
        m
            
        figure
        hold on
        plot3(homOrbit_s1(:,1), homOrbit_s1(:,2), homOrbit_s1(:,3), 'k.');
        plot3(homOrbit_s2(:,1), homOrbit_s2(:,2), homOrbit_s2(:,3), 'g.');
        plot3(homOrbit_u1(:,1), homOrbit_u1(:,2), homOrbit_u1(:,3), 'k.');
        plot3(homOrbit_u2(:,1), homOrbit_u2(:,2), homOrbit_u2(:,3), 'g.');

        time1 = linspace(-timesToIterate, 0, timesToIterate+1);
        time2 = linspace(0, n_u, n_u+1);
        time3 = linspace(n_u, n_s + n_u, n_s + 1);
        time4 = linspace(n_s+n_u, ...
                         n_s+n_u+timesToIterate, timesToIterate+1);
        figure
        hold on
        plot(time1(:), flipud(homOrbit_u2(:, 1)), 'g');
        plot(time2(:), homOrbit_u1(:,1), 'g');
        plot(time3(:), flipud(homOrbit_s1(:,1)), 'g');
        plot(time4(:), homOrbit_s2(:,1), 'g');
        plot(time1(:), flipud(homOrbit_u2(:, 1)), 'r.');
        plot(time2(:), homOrbit_u1(:,1), 'k.');
        plot(time3(:), flipud(homOrbit_s1(:,1)), 'k.');
        plot(time4(:), homOrbit_s2(:,1), 'r.');
        end
        
        
        
        orbitsValidated(m+1) = orbitValidated;
        chaosValidations(m+1) = chaosValidation;
        validatedRadii(m+1) = validatedRadius;
        
    else
       eps_dm = lastWorkingEps*(1-0.1^(numFailuers))
       failLoopFirred = failLoopFirred + 1
       
       normalLoop = 0
    end       
end           
           
       
orbitsValidated;
chaosValidations;
validatedRadii;