
clear
format long 


%intvalinit('FastIVmult')
intvalinit('SharpIVmult')


%=================================================================
%   PROGRAM COMPUTES BOTH ONE AND TWO DIMENSIONAL STABLE AND 
%   UNSTABLE MANIFOLDS AT BOTH FIXED POINTS, BY PARAMETERIZATION
%
%   COMPUTE RIGOROUS HOMOCLINIC ORBIT:
%
%   Plot 1 and 2 D manifolds for Dullin-Meiss parameters.
%       Steps 1, 2, and 3 of paper codes.
%
%
%  Development Nots:  The code for globalizing the stable and 
%  unstable manifolds are in aaa_barycenterWorkII.m and 
%  aaa_barycenterWu.m respectively.
%
%
%
%================================================================



%==========================================================================
%Non-Rigorous Computation of Approximate Solution:
%==========================================================================



%==============================================================
%Computation Controls:
%==============================================================



%Map Parameter Set: Dullin to Lomeli Meiss.
   %Dullin Meiss Parms:
      a_dm = 1
      b_dm = 0.5
      c_dm = 0.5
            
      
      
      eps_dm = 2.11    %boundary near 6.182001
      mu_dm = -2.4
      
      
   %Convert to Lomeli:   
      a = c_dm
      b = b_dm - 2*c_dm
      c = c_dm + a_dm - b_dm
      
      tau =    2*a_dm*(3+mu_dm)/(2*a_dm - b_dm)
      alpha =  ((9+6*mu_dm+mu_dm^2)*a_dm - 4*eps_dm*a_dm^2 + ...
               4*eps_dm*a_dm*b_dm - eps_dm*b_dm^2)/(2*a_dm - b_dm)^2



%---for epsilon > 3.5-ish guess_tol ~ 10^(-1) is good.
%---for epsilon ~ 0.2 - 2 neew  guess_tol ~ 2.5 X 10^(-2)
%---what to do close to eps ~ 0.1?
guess_tol = 5*10^(-2) 


l = 20

numRad = 30;
numTheta = 250;

%Order of manifold computation:
  %Order of 2D manifolds:
  N = 25;
  %Order of 1D manifold computations 
  order_1 = 50;
   
  n_u = 3;
  ns_0 = 1
  
%==========================================================================
%Generate zeroth and first order problem data:
%==========================================================================

[p0, p1, lambda0_s, lambda0_u1, lambda0_u2, ...
         lambda1_u, lambda1_s1, lambda1_s2, ...
         xi0_s, xi0_u1, xi0_u2, ...
         xi1_u, xi1_s1, xi1_s2] ...
     = firstOrderData_3DHenon(a, b, c, alpha, tau) 
  
%=====================================================================
%One-D Manifolds 
%=====================================================================

%Step 1: Compute 1D Coefficients:

scale = 20;

[A0s, B0s, C0s] = oneDimManifold_3DHenon(a, b, c, alpha, tau, ...
     order_1, p0, lambda0_s, xi0_s, scale);

scale = 50 
 
[A1u, B1u, C1u] = oneDimManifold_3DHenon(a, b, c, alpha, tau, ...
    order_1, p1, lambda1_u, xi1_u, scale);

%Step 2: Choose Parameter domain radius:
r0 = 0.25;
step = 0.001;
eps_tol = 5*10^(-12);

r1_u = oneD_parmInterval(A1u, B1u, C1u, order_1, lambda1_u, r0, step, eps_tol, ...
                         alpha, tau, a, b, c);

r0 = 0.025
step = 0.0001

                     
r0_s = oneD_parmInterval(A0s, B0s, C0s, order_1, lambda0_s, r0, step, eps_tol, ...
                         alpha, tau, a, b, c);

r0_s
r1_u

%Step 3: Compute fundamental domain point clouds:
numPoints = 5000;

[I1u_left, I1u_right, W1u_fd_left, W1u_fd_right, ...
    I1u, W1u_loc] = ...
    oneDim_fd(A1u, B1u, C1u, order_1, lambda1_u, r1_u, numPoints);

[I0s_left, I0s_right, W0s_fd_left, W0s_fd_right, ...
    I0s, W0s_loc] = ...
    oneDim_fd(A0s, B0s, C0s, order_1, lambda0_s, r0_s, numPoints);


[I1u_left, I1u_right, W1u_fd_left, W1u_fd_right] = ...
    oneDim_fd(A1u, B1u, C1u, order_1, lambda1_u, r1_u, numPoints);

[I0s_left, I0s_right, W0s_fd_left, W0s_fd_right] = ...
    oneDim_fd(A0s, B0s, C0s, order_1, lambda0_s, r0_s, numPoints);

%==========================================================================
%Two-D Manifolds:
%==========================================================================

%Step 1: Compute Coefficients:
scale1 = 25;
scale2 = 25;

[A0u, B0u, C0u] = twoDimManifold_3DHenon(a, b, c, alpha, tau, ...
    N, p0, lambda0_u1, lambda0_u2, xi0_u1, xi0_u2, scale1, scale2);

[A1s, B1s, C1s] = twoDimManifold_3DHenon(a, b, c, alpha, tau, ...
    N, p1, lambda1_s1, lambda1_s2, xi1_s1, xi1_s2, scale1, scale2);

%Step 2: Choose Parameter Domain Radius:
r0 = 0.025;
step = 0.001;
eps_tol = 5*10^(-5);

r0_u = twoD_parmDomain(A0u, B0u, C0u, N, lambda0_u1, r0, step, eps_tol, ...
                      alpha, tau, a, b, c);

r1_s = twoD_parmDomain(A1s, B1s, C1s, N, lambda1_s1, r0, step, eps_tol, ...
                      alpha, tau, a, b, c);                  

r0_u
r1_s    

%Step 3: Compute 2D fundamental domains:

[D0u_fd, W0u_fd] = twoDim_fd(A0u, B0u, C0u, N, ...
                             lambda0_u1, r0_u, numRad, numTheta);
[D1s_fd, W1s_fd] = twoDim_fd(A1s, B1s, C1s, N, ...
                             lambda1_s1, r1_s, numRad, numTheta);

%==========================================================================
%Step 5: Compute Homoclinic Orbit/Tangle.
%        (Here we will just compute at p0)
%==========================================================================


%----------------Compute Homoclinic Equation Parameters:-------------------

  %Iterate the 2D fundamental domain:
 
  W0u_global = ...
        k_image_set(W0u_fd, n_u, alpha, tau, a, b, c);
 
   globalize_fd = inverse_iterate_set_k(W0s_fd_right, 15, ...
                                        alpha, tau, a, b, c); 
    
   figure
   hold on
   %plot3(W0u_fd(:,1), W0u_fd(:,2), W0u_fd(:,3), 'b.')
   plot3(W0u_global(:,1), W0u_global(:,2), W0u_global(:,3), 'b.'); 
   %plot3(W0s_fd_right(:,1), W0s_fd_right(:,2), W0s_fd_right(:,3), 'r.'); 
   plot3(globalize_fd(:,1), globalize_fd(:,2), globalize_fd(:,3), 'r.');
   plot3(p0(1), p0(2), p0(3), 'k*'); 
   axis([-l l -l l -l l]);
   
   
  %Iterate the 1D fundamental domain until an approximate
  %intersection with the 2D domain is found.  Return the 
  %iterate numbers, and the approximate parameters.
  
  %tol = guess_tol;
  %tol = 2.5*10^(-2)
  tol = 5^(-1)
 
  
  [theta_0, tau_0, n_s] = homoclinicInitialGuess(...
      D0u_fd, I0s_left, I0s_right, ...
      W0u_global, W0s_fd_left, W0s_fd_right, ...
      alpha, tau, a, b, c, p0, tol, 9)
  


%---------------Solve Homoclinic Equation by Newton Method:----------------


[thetaBar, tauBar, error] = homoclinicPoint_p0(theta_0, tau_0, n_u, n_s, ...
    A0u, B0u, C0u, A0s, B0s, C0s, N, order_1, alpha, tau, a, b, c)


K = n_u + n_s;
Kmid = ceil(K/2);

xn = zeros(3*(K-1), 1);
this_x = evaluate_complexCase(theta_0(1), theta_0(2), A0u, B0u, C0u, N); 
this_y = [horner(tau_0, A0s, order_1);
          horner(tau_0, B0s, order_1);
          horner(tau_0, C0s, order_1)];
 
      
for j = 1:Kmid 
   this_orbit = henon3D(this_x, alpha, tau, a, b, c);
   xn((3*j-2):(3*j)) = this_orbit;
   this_x = this_orbit;
end

thisIndex = 0;
for j = (Kmid+1):(K-1)
   this_orbit = inv_henon3D(this_y, alpha, tau, a, b, c);
   xn((3*(K-1) - 3*thisIndex - 2):(3*(K-1) - 3*thisIndex)) = this_orbit;
   this_y = this_orbit;
   thisIndex = thisIndex + 1;
end

theta_n = [theta_0(1);
           theta_0(2)];
tau_n = tau_0;
x = zeros(3*K, 1);
Fx = x;
DFx = zeros(3*K, 3*K);

thisError = inf;
numLoops = 0;   

%Base error calculation




while ((thisError > 10^(-13)) & numLoops < 25)
 
    
 numLoops = numLoops + 1
 
 %Image of theta_n under Ps:
 Ps_xn = [horner(tau_n, A0s, order_1);
          horner(tau_n, B0s, order_1);
          horner(tau_n, C0s, order_1)];
 %Image of z_n = (s_n, t_n) Pu:
 Pu_zn = evaluate_complexCase(theta_n(1), theta_n(2), A0u, B0u, C0u, N);
 
 
 %Differential of Ps(theta_n):
 DPs_tau = evaluate_differential_1D(tau_n, A0s, B0s, C0s, order_1);
 %Differential of Pu(z_n):
 DPu_theta = evaluate_jacob_complexCase(theta_n(1), theta_n(2), A0u, B0u, C0u, N);
 
 
 x(1) = theta_n(1);
 x(2) = theta_n(2);
 x(3:(3*K-1)) = xn;
 x(3*K) = tau_n; 
 
 Fx(1:3) = inv_henon3D(xn(1:3), alpha, tau, a, b, c) - Pu_zn';
 
 DFx = zeros(3*K, 3*K);
 DFx(1:3, 1:2) = -DPu_theta;
 DFx(1:3, 3:5) = inv_henon3D_differential(xn(1:3), alpha, tau, a, b, c);
 
 
 Id = eye(3);
 for j = 2:Kmid
    Fx((3*j - 2):(3*j)) = ...
          inv_henon3D(xn((3*j-2):(3*j)), alpha, tau, a, b, c) - xn((3*j-5):(3*j-3)); 
    DFx((3*j - 2):(3*j), (3*j-3):(3*j-1)) = -Id;
    DFx((3*j - 2):(3*j), (3*j):(3*j+2)) = ...
        inv_henon3D_differential(xn((3*j-2):(3*j)), alpha, tau, a, b, c);
 end
 for j = (Kmid+1):(K-1)
    Fx((3*j - 2):(3*j)) = ...
        henon3D(xn((3*(j-1)-2):(3*(j-1))), alpha, tau, a, b, c) - xn((3*j - 2):(3*j)); ;
    DFx((3*j-2):(3*j), (3*j-3):(3*j-1)) = ...
        henon3D_differential(xn((3*(j-1)-2):(3*(j-1))) , alpha, tau, a, b, c);
    DFx((3*j-2):(3*j), (3*j):(3*j+2)) = -Id;
 end
 
 Fx((3*K - 2):(3*K)) = henon3D(xn((3*(K-1) - 2):(3*(K-1))), alpha, tau, a, b, c) -  Ps_xn;
 
 DFx((3*K - 2):(3*K), (3*K-3):(3*K-1)) = ...
     henon3D_differential(xn((3*(K-1) - 2):(3*(K-1))) , alpha, tau, a, b, c);
 DFx((3*K - 2):(3*K), 3*K) = -DPs_tau;       
 
 %Compute the Newton Step:
 xnew = x - inv(DFx)*Fx;
 
 theta_n = xnew(1:2);
 xn = xnew(3:(3*K-1));
 tau_n = xnew(3*K);
 
 
 %Check Error:
 Ps_xn = [horner(tau_n, A0s, order_1);
          horner(tau_n, B0s, order_1);
          horner(tau_n, C0s, order_1)];
 %Image of z_n = (s_n, t_n) Pu:
 Pu_zn = evaluate_complexCase(theta_n(1), theta_n(2), A0u, B0u, C0u, N);
 Fx_new(1:3) = inv_henon3D(xn(1:3), alpha, tau, a, b, c) - Pu_zn';
 for j = 2:Kmid
    Fx_new((3*j - 2):(3*j)) = ...
          inv_henon3D(xn((3*j-2):(3*j)), alpha, tau, a, b, c) - xn((3*j-5):(3*j-3)); 
 end
 for j = (Kmid+1):(K-1)
    Fx_new((3*j - 2):(3*j)) = ...
        henon3D(xn((3*(j-1)-2):(3*(j-1))), alpha, tau, a, b, c) - xn((3*j - 2):(3*j)); ;
 end
 Fx_new((3*K - 2):(3*K)) = henon3D(xn((3*(K-1) - 2):(3*(K-1))), alpha, tau, a, b, c) -  Ps_xn;
 
 
 %lastError = thisError
 thisError = max(abs(Fx_new))
end







