clear
format long 


%intvalinit('FastIVmult')
intvalinit('SharpIVmult')


%=================================================================
%   PROGRAM COMPUTES BOTH ONE AND TWO DIMENSIONAL STABLE AND 
%   UNSTABLE MANIFOLDS AT BOTH FIXED POINTS, BY PARAMETERIZATION
%
%   COMPUTE RIGOROUS HOMOCLINIC ORBIT:
%
%   Plot 1 and 2 D manifolds for Dullin-Meiss parameters.
%       Steps 1, 2, and 3 of paper codes.
%
%
%  Development Nots:  The code for globalizing the stable and 
%  unstable manifolds are in aaa_barycenterWorkII.m and 
%  aaa_barycenterWu.m respectively.
%
%
%
%================================================================



%==========================================================================
%Non-Rigorous Computation of Approximate Solution:
%==========================================================================



%==============================================================
%Computation Controls:
%==============================================================



%Map Parameter Set: Dullin to Lomeli Meiss.
   %Dullin Meiss Parms:
      a_dm = 1
      b_dm = 0.5
      c_dm = 0.5
            
      
      %eps_dm = 2.11    %boundary near 6.182001
      eps_dm = 0.32      %worked at 0.4
      mu_dm = -2.4
      
      
   %Convert to Lomeli:   
      a = c_dm
      b = b_dm - 2*c_dm
      c = c_dm + a_dm - b_dm
      
      tau =    2*a_dm*(3+mu_dm)/(2*a_dm - b_dm)
      alpha =  ((9+6*mu_dm+mu_dm^2)*a_dm - 4*eps_dm*a_dm^2 + ...
               4*eps_dm*a_dm*b_dm - eps_dm*b_dm^2)/(2*a_dm - b_dm)^2



%---for epsilon > 3.5-ish guess_tol ~ 10^(-1) is good.
%---for epsilon ~ 0.2 - 2 neew  guess_tol ~ 2.5 X 10^(-2)
%---what to do close to eps ~ 0.1?
guess_tol = 2.5*10^(-2) 


l = 20

numRad = 30;
numTheta = 250;

%Order of manifold computation:
  %Order of 2D manifolds:
  N = 25;
  %Order of 1D manifold computations 
  order_1 = 50;
   
  n_u = 3;
  ns_0 = 1
  
%==========================================================================
%Generate zeroth and first order problem data:
%==========================================================================

[p0, p1, lambda0_s, lambda0_u1, lambda0_u2, ...
         lambda1_u, lambda1_s1, lambda1_s2, ...
         xi0_s, xi0_u1, xi0_u2, ...
         xi1_u, xi1_s1, xi1_s2] ...
     = firstOrderData_3DHenon(a, b, c, alpha, tau) 
  
 
 
%=====================================================================
%One-D Manifolds 
%=====================================================================

%Step 1: Compute 1D Coefficients:

scale = 20;

[A0s, B0s, C0s] = oneDimManifold_3DHenon(a, b, c, alpha, tau, ...
     order_1, p0, lambda0_s, xi0_s, scale);

scale = 50 
 
[A1u, B1u, C1u] = oneDimManifold_3DHenon(a, b, c, alpha, tau, ...
    order_1, p1, lambda1_u, xi1_u, scale);

%Step 2: Choose Parameter domain radius:
r0 = 0.25;
step = 0.001;
eps_tol = 5*10^(-12);

r1_u = oneD_parmInterval(A1u, B1u, C1u, order_1, lambda1_u, r0, step, eps_tol, ...
                         alpha, tau, a, b, c);

r0 = 0.025
step = 0.0001

                     
r0_s = oneD_parmInterval(A0s, B0s, C0s, order_1, lambda0_s, r0, step, eps_tol, ...
                         alpha, tau, a, b, c);

r0_s
r1_u

%Step 3: Compute fundamental domain point clouds:
numPoints = 5000;

[I1u_left, I1u_right, W1u_fd_left, W1u_fd_right, ...
    I1u, W1u_loc] = ...
    oneDim_fd(A1u, B1u, C1u, order_1, lambda1_u, r1_u, numPoints);

[I0s_left, I0s_right, W0s_fd_left, W0s_fd_right, ...
    I0s, W0s_loc] = ...
    oneDim_fd(A0s, B0s, C0s, order_1, lambda0_s, r0_s, numPoints);


%[I1u_left, I1u_right, W1u_fd_left, W1u_fd_right] = ...
%    oneDim_fd(A1u, B1u, C1u, order_1, lambda1_u, r1_u, numPoints);

%[I0s_left, I0s_right, W0s_fd_left, W0s_fd_right] = ...
%    oneDim_fd(A0s, B0s, C0s, order_1, lambda0_s, r0_s, numPoints);

%==========================================================================
%Two-D Manifolds:
%==========================================================================

%Step 1: Compute Coefficients:
scale1 = 25;
scale2 = 25;

[A0u, B0u, C0u] = twoDimManifold_3DHenon(a, b, c, alpha, tau, ...
    N, p0, lambda0_u1, lambda0_u2, xi0_u1, xi0_u2, scale1, scale2);

[A1s, B1s, C1s] = twoDimManifold_3DHenon(a, b, c, alpha, tau, ...
    N, p1, lambda1_s1, lambda1_s2, xi1_s1, xi1_s2, scale1, scale2);

%Step 2: Choose Parameter Domain Radius:
r0 = 0.025;
step = 0.001;
eps_tol = 5*10^(-5);

r0_u = twoD_parmDomain(A0u, B0u, C0u, N, lambda0_u1, r0, step, eps_tol, ...
                      alpha, tau, a, b, c);

r1_s = twoD_parmDomain(A1s, B1s, C1s, N, lambda1_s1, r0, step, eps_tol, ...
                      alpha, tau, a, b, c);                  

r0_u
r1_s    

%Step 3: Compute 2D fundamental domains:

[D0u_fd, W0u_fd] = twoDim_fd(A0u, B0u, C0u, N, ...
                             lambda0_u1, r0_u, numRad, numTheta);
[D1s_fd, W1s_fd] = twoDim_fd(A1s, B1s, C1s, N, ...
                             lambda1_s1, r1_s, numRad, numTheta);

                         
%==========================================================================
%Step 5: Compute Homoclinic Orbit/Tangle.
%        (Here we will just compute at p0)
%==========================================================================


%----------------Compute Homoclinic Equation Parameters:-------------------

  %Iterate the 2D fundamental domain:
 
  W0u_global = ...
        k_image_set(W0u_fd, 3, alpha, tau, a, b, c);
 
   globalize_fd = inverse_iterate_set_k(W0s_fd_right, 46, ...
                                        alpha, tau, a, b, c); 
    
   figure
   hold on
   plot3(W0u_fd(:,1), W0u_fd(:,2), W0u_fd(:,3), 'b.')
   %plot3(W0u_global(:,1), W0u_global(:,2), W0u_global(:,3), 'b.'); 
   %plot3(W0s_fd_right(:,1), W0s_fd_right(:,2), W0s_fd_right(:,3), 'r.'); 
   plot3(globalize_fd(:,1), globalize_fd(:,2), globalize_fd(:,3), 'r.');
   plot3(p0(1), p0(2), p0(3), 'k*'); 
   axis([-l l -l l -l l]);

   
  %stopNow = stopHere
   
   
  %Iterate the 1D fundamental domain until an approximate
  %intersection with the 2D domain is found.  Return the 
  %iterate numbers, and the approximate parameters.
  
  %tol = guess_tol;
  %tol = 4*10^(-2)
  tol = 5^(-1)           %was working for largish eps_dm >= 1
 

  
  
  [theta_0, tau_0, n_s] = homoclinicInitialGuess(...
      D0u_fd, I0s_left, I0s_right, ...
      W0u_global, W0s_fd_left, W0s_fd_right, ...
      alpha, tau, a, b, c, p0, tol, 46)
  
  

  
  

%---------------Solve Homoclinic Equation by Newton Method:----------------


[thetaBar, tauBar, error] = homoclinicPoint_p0(theta_0, tau_0, n_u, n_s, ...
    A0u, B0u, C0u, A0s, B0s, C0s, N, order_1, alpha, tau, a, b, c)


[thetaBar_prod, xn_bar, tauBar_prod, errorProd] = ...
    homoclinicPoint_p0_prodSpace(thetaBar, tauBar, n_u, n_s, ...
    A0u, B0u, C0u, A0s, B0s, C0s, N, order_1, alpha, tau, a, b, c);






%----------------Plot Orbit:-----------------------------------------------

homPoint_s = [horner(tauBar, A0s, order_1), ...
              horner(tauBar, B0s, order_1), ...
              horner(tauBar, C0s, order_1)];

homPoint_u = evaluate_complexCase(thetaBar(1), thetaBar(2), ...
                                  A0u, B0u, C0u, N);
      
timesToIterate = 30                              
homOrbit_s1 = inverse_iterate_set_k(homPoint_s, n_s, alpha, tau, a, b, c);      
homOrbit_s2 = iterate_set_k(homPoint_s, timesToIterate, alpha, tau, a, b, c); 
homOrbit_u1 = iterate_set_k(homPoint_u, n_u, alpha, tau, a, b, c); 
homOrbit_u2 = inverse_iterate_set_k(homPoint_u, timesToIterate, alpha, tau, a, b, c); 

figure
hold on
plot3(homOrbit_s1(:,1), homOrbit_s1(:,2), homOrbit_s1(:,3), 'g.');
plot3(homOrbit_s2(:,1), homOrbit_s2(:,2), homOrbit_s2(:,3), 'g.');
plot3(homOrbit_u1(:,1), homOrbit_u1(:,2), homOrbit_u1(:,3), 'g.');
plot3(homOrbit_u2(:,1), homOrbit_u2(:,2), homOrbit_u2(:,3), 'g.');

time1 = linspace(-timesToIterate, 0, timesToIterate+1);
time2 = linspace(0, n_u, n_u+1);
time3 = linspace(n_u, n_s + n_u, n_s + 1);
time4 = linspace(n_s+n_u, n_s+n_u+timesToIterate, timesToIterate+1);
figure
hold on
plot(time1(:), flipud(homOrbit_u2(:, 1)), 'g');
plot(time2(:), homOrbit_u1(:,1), 'g');
plot(time3(:), flipud(homOrbit_s1(:,1)), 'g');
plot(time4(:), homOrbit_s2(:,1), 'g');
plot(time1(:), flipud(homOrbit_u2(:, 1)), 'k.');
plot(time2(:), homOrbit_u1(:,1), 'k.');
plot(time3(:), flipud(homOrbit_s1(:,1)), 'k.');
plot(time4(:), homOrbit_s2(:,1), 'k.');





%==========================================================================
%Rigorous Validation:
%==========================================================================



%Step 1: Rigorous Manifolds


%Map Parameter Set: Dullin to Lomeli Meiss.
   %Dullin Meiss Parms:
     
      
      
   %Convert to Lomeli:   
      a = intval(a)      %c_dm
      b = intval(b)      %b_dm - 2*c_dm
      c = intval(c)      %c_dm + a_dm - b_dm
      
      tau =  intval(tau)  %2*a_dm*(3+mu_dm)/(2*a_dm - b_dm)
      alpha =  intval(alpha) %((9+6*mu_dm+mu_dm^2)*a_dm - 4*eps_dm*a_dm^2 + ...
                           %4*eps_dm*a_dm*b_dm - eps_dm*b_dm^2)/(2*a_dm - b_dm)^2

     


                     
           
[p0, p1, lambda0_s, lambda0_u1, lambda0_u2, ...
         lambda1_u, lambda1_s1, lambda1_s2, ...
         xi0_s, xi0_u1, xi0_u2, ...
         xi1_u, xi1_s1, xi1_s2] ...
       = firstOrderData_intval_3DHenon(a, b, c, alpha, tau);
     
   
   

           
%=====================================================================
%One-D Manifolds 
%=====================================================================

%Step 1: Compute 1D Coefficients:


scale = intval('20');


[A0s, B0s, C0s] = oneDimManifold_intval_3DHenon(a, b, c, alpha, tau, ...
     order_1, p0, lambda0_s, xi0_s, scale);

 
scale = intval('50') 
 
[A1u, B1u, C1u] = oneDimManifold_intval_3DHenon(a, b, c, alpha, tau, ...
    order_1, p1, lambda1_u, xi1_u, scale);           
           
           

firstOrder = 1

           
%==========================================================================
%Two-D Manifolds:
%==========================================================================

%Step 1: Compute Coefficients:
 scale1 = intval('25');
 scale2 = intval('25');
 
 %N = 25;
 

 
 tic
 
 [A0u, B0u, C0u] = twoDimManifold_intval_3DHenon(a, b, c, alpha, tau, ...
     N, p0, lambda0_u1, lambda0_u2, xi0_u1, xi0_u2, scale1, scale2);
 
 
 [A1s, B1s, C1s] = twoDimManifold_intval_3DHenon(a, b, c, alpha, tau, ...
     N, p1, lambda1_s1, lambda1_s2, xi1_s1, xi1_s2, scale1, scale2);           
            
   
 time_1 = toc/60


%----------------------Put in Polynomial Data Structure--------------------



[P0u1, P0u2, P0u3, P1s1, P1s2, P1s3] = convertToPoly_2D(A0u, B0u, C0u, ...
                                                     A1s, B1s, C1s, N);


[P0s1, P0s2, P0s3, P1u1, P1u2, P1u3] = convertToPoly_1D(A1u, B1u, C1u, ...
                                                   A0s, B0s, C0s, order_1);                                                 



                                               
%Step 2: Validate the Manifolds                                               
                                               



%domRad0s = intval('0.02')
%domRad1u = intval('0.01')
%domRad1s = intval('0.01');              
%domRad0u = intval('0.01');
 

domRad0s = intval(r0_s)
domRad0u = intval(r0_u)
domRad1s = intval(r1_s)
domRad1u = intval(r1_u)


N_f = 4
N_dim = 3

[delta_0s, delta_1u, delta_max_0s, delta_max_1u, ...
    P0s_norm, P1u_norm, ...
    u1_delta_good, s0_delta_good, OrderTest]...
    = validateManifolds_1D_3DHenon(...
    P0s1, P0s2, P0s3, P1u1, P1u2, P1u3, ...
    p0, p1, xi0_u1, xi0_u2, xi0_s, xi1_u, xi1_s1, xi1_s2,...
    lambda0_s, lambda1_u, N_f, N_dim, ...
    domRad0s, domRad1u, order_1, a, b, c, alpha, tau);


P0s_norm
P1u_norm

delta_0s
delta_1u



                                                                                  
%Phase Space Bounds: 2D


                     
%A-Posteriori Errors: 2D                           




[delta_0u, delta_1s, delta_max_0u, delta_max_1s,...
 P0u_norm, P1s_norm, u0_delta_good, s1_delta_good, orderTest] ...
    = validateManifolds_2D_3DHenon(...
    P0u1, P0u2, P0u3, P1s1, P1s2, P1s3, ...
    p0, p1, xi0_u1, xi0_u2, xi0_s, xi1_u, xi1_s1, xi1_s2, ...
    lambda0_u1, lambda0_u2, lambda1_s1, lambda1_s2, N_f, N_dim, ...
    domRad0u, domRad1s, N, a, b, c, alpha, tau);



P0s_norm
P1u_norm

delta_0s
delta_1u




P0u_norm
P1s_norm

delta_0u
delta_1s



%Interal version of approximate orbit:  (It is not necessary that the 
%                                        interval guess include the
%                                        floating point guess.  We only
%                                        need some guess.  So we opt for a
%                                        guess with zero raduis i.e. a
%                                        degenerate interval.
thetaBar_intval = intval(thetaBar_prod);
xn_bar_intval = intval(xn_bar);
tauBar_intval = intval(tauBar_prod);


%The Newton Vector:
[Fx1_intval, Fx2_intval, Fx3_intval] = ...
    F_prod_intval(thetaBar_intval, xn_bar_intval, tauBar_intval, n_u, n_s, ...                          
    P0u1, P0u2, P0u3, P0s1, P0s2, P0s3, alpha, tau, a, b, c);


Fx_intval = [Fx1_intval, Fx2_intval, Fx3_intval]

%Rigorous Epsilon Error:
epsilon_hat = ...
    max([sup(abs(Fx1_intval)), sup(abs(Fx2_intval)), sup(abs(Fx3_intval))])


%Interval evaluation of Df(x_0):
A = DF_prod_intval(thetaBar_intval, xn_bar_intval, tauBar_intval, n_u, n_s, ...
    P0u1, P0u2, P0u3, P0s1, P0s2, P0s3, alpha, tau, a, b, c);


Ainv = inv(A);
Ainv_norm = norm(Ainv, inf)



%check parmeter domain conditions:
theta_norm = sqrt(thetaBar_intval(1)^2 + thetaBar_intval(2)^2);
thetaTest = sup(theta_norm)
if ( thetaTest >= inf(testDom))
   crashNow = crashHere 
end

tauTest = sup(abs(tauBar_intval))
testDom = domRad0s
if (testTau >= inf(testDom))
   crashNow = crashHere 
end





sigma_s = -(1/2)*log((domRad0s + abs(tauBar_intval))/(2*domRad0s))
sigma_u = -(1/2)*log((domRad0u + theta_norm)/(2*domRad0u))
delta_NK = max([sup(delta_0u), sup(delta_0s)])
nu_NK = min([inf(domRad0u), inf(domRad0s)])


%Step 3: Validate the Newton Computation.                           


M_N = intval('pi')*N_dim*Ainv_norm*delta_0s/(domRad0s*sigma_s) + ...
      intval('pi')*N_dim*Ainv_norm*delta_0u/(domRad0u*sigma_u);
     
C_Beta = max([sup(2*abs(a)), sup(2*abs(c)), sup(abs(b))])
C_P = parm_2ndDiff_1Dbound(domRad0s, P0s1, P0s2, P0s3) + ...
      parm_2ndDiff_2Dbound(domRad0u, P0u1, P0u2, P0u3) + ...
      4*intval('pi')^2*3^2*sigma_s^(-2)*domRad0s^(-2)*delta_0s+ ...
      4*intval('pi')^2*3^2*sigma_u^(-2)*domRad0u^(-2)*delta_0u
  
N_F = 15

epsilon_NK = (Ainv_norm/(1-M_N))*(epsilon_hat + delta_0s + delta_0u)
r_NewtonKantorivich = 2*epsilon_NK
kappa_NK = N_F*C_Beta + C_P
NK_Test = sup(4*kappa_NK*Ainv_norm*epsilon_NK)







%Transversallity Test:

theta_ball = midrad(mid(thetaBar_intval), r_NewtonKantorivich);
xn_ball = midrad(mid(xn_bar_intval), r_NewtonKantorivich);
tau_ball = midrad(mid(tauBar_intval), r_NewtonKantorivich);


DFx_ball =  DF_prod_intval(theta_ball, xn_ball, tau_ball, n_u, n_s, ...
     P0u1, P0u2, P0u3, P0s1, P0s2, P0s3, alpha, tau, a, b, c);                         
      
%Determinant: From section 10.10 [Rump]
 [L, U, p] = lu(mid(DFx_ball), 'vector');
 Linv = inv(L);
 Uinv = inv(U);
 C = intval(Linv)*(DFx_ball(p, :)*intval(Uinv));
 D = prod(gershgorin(C));
 
transverse_condNumber = real(D)
chaosValidation = 0;
%If zero is not in the determinant interval then
%the matrix is non-singular and the intersection is 
%transverse.
if (in(0, transverse_condNumber) == 0)
    chaosValidation = 1;
else
  %Try to subdivide r and fix the computation.
     %HAVE TO WRITE
end
chaosValidation










