clear 
format long

%==========================================================================
%---------------Rigorous Parameterization Computations---------------------
%
%This program uses the MatLab package ``IntLab''.
%Before using run from the command prompt:
%
%>> startintlab
%
%
%
%
%
%
%
%--------------------------------------------------------------------------
%==========================================================================

%parm controls:
order_1 = 50


%Map Parameter Set: Dullin to Lomeli Meiss.
   %Dullin Meiss Parms:
      a_dm = intval('1')
      b_dm = intval('0.5')
      c_dm = intval('0.5')
            
      
      
      eps_dm = intval('2.11')    %boundary near 6.182001
      mu_dm = intval('-2.4')
      
      
   %Convert to Lomeli:   
      a = c_dm
      b = b_dm - 2*c_dm
      c = c_dm + a_dm - b_dm
      
      tau =    2*a_dm*(3+mu_dm)/(2*a_dm - b_dm)
      alpha =  ((9+6*mu_dm+mu_dm^2)*a_dm - 4*eps_dm*a_dm^2 + ...
               4*eps_dm*a_dm*b_dm - eps_dm*b_dm^2)/(2*a_dm - b_dm)^2

                      
           
[p0, p1, lambda0_s, lambda0_u1, lambda0_u2, ...
         lambda1_u, lambda1_s1, lambda1_s2, ...
         xi0_s, xi0_u1, xi0_u2, ...
         xi1_u, xi1_s1, xi1_s2] ...
       = firstOrderData_intval_3DHenon(a, b, c, alpha, tau);
     
   
   

           
%=====================================================================
%One-D Manifolds 
%=====================================================================

%Step 1: Compute 1D Coefficients:


scale = intval('20');


[A0s, B0s, C0s] = oneDimManifold_intval_3DHenon(a, b, c, alpha, tau, ...
     order_1, p0, lambda0_s, xi0_s, scale);

 
scale = intval('50') 
 
[A1u, B1u, C1u] = oneDimManifold_intval_3DHenon(a, b, c, alpha, tau, ...
    order_1, p1, lambda1_u, xi1_u, scale);           
           
           

firstOrder = 1

           
%==========================================================================
%Two-D Manifolds:
%==========================================================================

%Step 1: Compute Coefficients:
 scale1 = intval('25');
 scale2 = intval('25');
 
 N = 25;
 

 
 tic
 
 [A0u, B0u, C0u] = twoDimManifold_intval_3DHenon(a, b, c, alpha, tau, ...
     N, p0, lambda0_u1, lambda0_u2, xi0_u1, xi0_u2, scale1, scale2);
 
 
 [A1s, B1s, C1s] = twoDimManifold_intval_3DHenon(a, b, c, alpha, tau, ...
     N, p1, lambda1_s1, lambda1_s2, xi1_s1, xi1_s2, scale1, scale2);           
            
   
 time_1 = toc/60
% 
% 
% %[A0u_fp, B0u_fp, C0u_fp] = twoDimManifold_3DHenon(a.mid, b.mid, c.mid, alpha.mid, tau.mid, ...
% %    N, p0.mid, lambda0_u1.mid, lambda0_u2.mid, xi0_u1.mid, xi0_u2.mid, scale1.mid, scale2.mid);


%----------------------Put in Polynomial Data Structure--------------------



[P0u1, P0u2, P0u3, P1s1, P1s2, P1s3] = convertToPoly_2D(A0u, B0u, C0u, ...
                                                     A1s, B1s, C1s, N);


[P0s1, P0s2, P0s3, P1u1, P1u2, P1u3] = convertToPoly_1D(A1u, B1u, C1u, ...
                                                   A0s, B0s, C0s, order_1);                                                 



%Phase Space Bounds: 1D  
domRad0s = intval('0.2');
domRad0u = intval('0.025')


%For plotting.
dom_1D_s = linspace(-inf(domRad0s)*0, inf(domRad0s), 100);
imageP0s = [polyval(mid(P0s1), dom_1D_s'), ...
            polyval(mid(P0s2), dom_1D_s'), ...
            polyval(mid(P0s3), dom_1D_s')];
        
dom_1D_u = linspace(-inf(domRad0u), 0*inf(domRad0u), 100);
imageP1u = [polyval(mid(P1u1), dom_1D_u'), ...
            polyval(mid(P1u2), dom_1D_u'), ...
            polyval(mid(P1u3), dom_1D_u')];        
 
                
        
        
figure
hold on
plot3(imageP0s(:,1), imageP0s(:,2), imageP0s(:,3), 'r.')
plot3(mid(p0(1)), mid(p0(2)), mid(p0(3)), 'k*')
plot3(imageP1u(:,1), imageP1u(:,2), imageP1u(:,3), 'b.')
plot3(mid(p1(1)), mid(p1(2)), mid(p1(3)), 'k*')


%Bounds.
max1 = polyval(mag(P0s1-p0(1)), domRad0s);
max2 = polyval(mag(P0s2-p0(2)), domRad0s);
max3 = polyval(mag(P0s3-p0(3)), domRad0s);
P0s_norm = max([sup(max1), sup(max2), sup(max3)])


max1 = polyval(mag(P1u1-p1(1)), domRad0u);
max2 = polyval(mag(P1u2-p1(2)), domRad0u);
max3 = polyval(mag(P1u3-p1(3)), domRad0u);
P1u_norm = max([sup(max1), sup(max2), sup(max3)])


%A Posteriori Errors: 1D                                               
                                               
                                               
                                               
epsilon_s1D = Error_intval_1D(P0s1, P0s2, P0s3, alpha, tau, ...
                               a, b, c, lambda0_s, order_1, domRad0s)

                          
                          
epsilon_u1D = Error_intval_1D(P1u1, P1u2, P1u3, alpha, tau, ...
                               a, b, c, lambda1_u, order_1, domRad0u)


%Phase Space Bounds: 2D

domRad1s = intval('0.025');

max1 = polyval(mag(P1s1 - p1(1)), domRad1s, domRad1s);
max2 = polyval(mag(P1s2 - p1(2)), domRad1s, domRad1s);
max3 = polyval(mag(P1s3 - p1(3)), domRad1s, domRad1s);
P1s_norm = max([sup(max1), sup(max2), sup(max3)])
              
domRad1u = intval('0.025');

max1 = polyval(mag(P0u1- p0(1)), domRad1u, domRad1u);
max2 = polyval(mag(P0u2- p0(2)), domRad1u, domRad1u);
max3 = polyval(mag(P0u3- p0(3)), domRad1u, domRad1u);
P0u_norm = max([sup(max1), sup(max2), sup(max3)])                          
                     
%A-Posteriori Errors: 2D                           


epsilon_u2D = Error_intval_2D(P0u1, P0u2, P0u3, alpha, tau, ...
                               a, b, c, lambda0_u1, lambda0_u2, N, domRad1u)

                           
                                               
epsilon_s2D = Error_intval_2D(P1s1, P1s2, P1s3, alpha, tau, ...
                               a, b, c, lambda1_s1, lambda1_s2, N, domRad1s)


                           

                

                           
                           
                           

% 
% 
%   tic
%   testVal_11 = real(polyval(P0u1_lambda, intval('0.876')+i*intval('-0.3457'), ...
%                             intval('0.876')-i*intval('-0.3457')));
%   testVal_12 = real(polyval(P0u2_lambda, intval('0.876')+i*intval('-0.3457'), ...
%                             intval('0.876')-i*intval('-0.3457')));
%   testVal_13 = real(polyval(P0u3_lambda, intval('0.876')+i*intval('-0.3457'), ...
%                             intval('0.876')-i*intval('-0.3457')));
%  
%   eval_P = [testVal_11;
%             testVal_12;
%             testVal_13]
%                         
%  time1 = toc 
% 
%  this_x = intval('0.876');
%  this_y = intval('-0.3457');
%  this_z = this_x + i*this_y;
%  this_zbar = this_x - i*this_y;
%  
%  tic
%   testVal_11lam = real(polyval(P0u1, lambda0_u1*this_z, lambda0_u2*this_zbar));
%   testVal_12lam = real(polyval(P0u2, lambda0_u1*this_z, lambda0_u2*this_zbar));
%   testVal_13lam = real(polyval(P0u3, lambda0_u1*this_z, lambda0_u2*this_zbar));
%  
%   eval_Plam = [testVal_11lam;
%                testVal_12lam;
%                testVal_13lam]
%                         
%  time1 = toc 
% 
% 
%  
 
 
% tic
% testVal_2 = evaluate_complexCase(0.876, -0.3457, A0u_fp, B0u_fp, C0u_fp, N)'
% time2 = toc
% 
%  
%tic
%testVal_2 = evaluate_intval_complexCase(...
%    intval('0.876'), intval('-0.3457'), A0u, B0u, C0u, N)
%time2 = toc










%----------Rigorous A-Posteriori Validation--------------------------------














