
clear
format long 


%intvalinit('FastIVmult')
intvalinit('SharpIVmult')


%=================================================================
%   PROGRAM COMPUTES BOTH ONE AND TWO DIMENSIONAL STABLE AND 
%   UNSTABLE MANIFOLDS AT BOTH FIXED POINTS, BY PARAMETERIZATION
%
%   COMPUTE HOMOCLINIC ORBIT:
%
%
%
%
%================================================================



%==========================================================================
%Non-Rigorous Computation of Approximate Solution:
%==========================================================================



%==============================================================
%Computation Controls:
%==============================================================



%Map Parameter Set: Dullin to Lomeli Meiss.
   %Dullin Meiss Parms:
      
   %Subsystem 1 Parameters:
      a_dm1 = 1
      b_dm1 = 0.5
      c_dm1 = 0.5
            
      
      eps_dm1 = 5.5    %boundary near 6.182001
     %eps_dm = 0.32      %worked at 0.4
      mu_dm1 = -2.4
    
   %Subsystem 2 Parameters:   
      a_dm2 = 1
      b_dm2 = 0.5
      c_dm2 = 0.5
            
      
      eps_dm2 = 6.1    %boundary near 6.182001
     %eps_dm = 0.32      %worked at 0.4
      mu_dm2 = -2.4
      
      
      
      
   %Convert Subsystem 1 to Lomeli:   
      a1 = c_dm1
      b1 = b_dm1 - 2*c_dm1
      c1 = c_dm1 + a_dm1 - b_dm1
      
      tau1 =    2*a_dm1*(3+mu_dm1)/(2*a_dm1 - b_dm1)
      alpha1 =  ((9+6*mu_dm1+mu_dm1^2)*a_dm1 - 4*eps_dm1*a_dm1^2 + ...
               4*eps_dm1*a_dm1*b_dm1 - eps_dm1*b_dm1^2)/(2*a_dm1 - b_dm1)^2

%Convert Subsystem 2 to Lomeli:   
      a2 = c_dm2
      b2 = b_dm2 - 2*c_dm2
      c2 = c_dm2 + a_dm2 - b_dm2
      
      tau2 =    2*a_dm2*(3+mu_dm2)/(2*a_dm2 - b_dm2)
      alpha2 =  ((9+6*mu_dm2+mu_dm2^2)*a_dm2 - 4*eps_dm2*a_dm2^2 + ...
               4*eps_dm2*a_dm2*b_dm2 - eps_dm2*b_dm2^2)/(2*a_dm2 - b_dm2)^2



%Order of manifold computation:
  %Order of 2D manifolds:
  Nu = 5;
  %Order of 1D manifold computations 
  Ns = 9;
   
  
  
  %Coupling Strength:
  epsilon_coupling = 5*10^(-7);
  %epsilon_coupling = -2.2*10^(-11) 
  
%==========================================================================
%Generate zeroth and first order problem data:
%==========================================================================

[p0, p1, lambda0_s1, lambda0_s2, ...
         lambda0_u1, lambda0_u2, lambda0_u3, lambda0_u4, ...
         lambda1_u1, lambda1_u2, ...
         lambda1_s1, lambda1_s2, lambda1_s3, lambda1_s4, ...
         xi0_s1, xi0_s2, xi0_u1, xi0_u2, xi0_u3, xi0_u4, ...
         xi1_u1, xi1_u2, xi1_s1, xi1_s2, xi1_s3, xi1_s4] ...
     = firstOrderData_6DHenon(a1, b1, c1, alpha1, tau1, ...
                              a2, b2, c2, alpha2, tau2, ...
                              epsilon_coupling); 
  
 
                          
                          
                          
%=====================================================================
%Low-D Manifolds 
%=====================================================================

%Step 1: Compute 2D Coefficients:

scale0_s1 = 1.5;
scale0_s2 = 1.5;

[A0s, B0s, C0s, D0s, E0s, F0s] = twoDimManifold_6DHenon(...
      a1, b1, c1, alpha1, tau1, ...
      a2, b2, c2, alpha2, tau2, ...
      epsilon_coupling, Ns, p0, ...
      lambda0_s1, lambda0_s2, xi0_s1, xi0_s2, ...
      scale0_s1, scale0_s2);

  
domRad0s = 0.001 
 

 
%=====================================================================
%High-D Manifolds 
%=====================================================================

%Step 1: Compute 4D Coefficients:

scale0_u1 = 0.4; 
scale0_u2 = 0.4;
scale0_u3 = 0.4;
scale0_u4 = 0.4;


tic

[A0u, B0u, C0u, D0u, E0u, F0u] = fourDimManifold_6DHenon(...
      a1, b1, c1, alpha1, tau1, ...
      a2, b2, c2, alpha2, tau2, ...
      epsilon_coupling, Nu, p0, ...
      lambda0_u1, lambda0_u2, lambda0_u3, lambda0_u4, ...
      xi0_u1, xi0_u2, xi0_u3, xi0_u4, ...
      scale0_u1, scale0_u2, scale0_u3, scale0_u4);

  
fourDim_floatTime = toc  
  
domRad0u = 0.001

n_s = 10
n_u = 10

load orbitData_6D_1;
load orbitData_6D_2;



%Note: theta_0(1) and (2) should be the variables
%      associated with the strongest eigendirections,
%      in this case system 2.  The other variables
%      are assigned accordingly.


theta_0 = [intersectionParameters2(1);
           intersectionParameters2(2);
           intersectionParameters1(1);
           intersectionParameters1(2)];
       
 
%Note: similarly, tau_0(1) should be the variable associated
%      with the system with largest stable eigenvalue.
%
%Note: In both cases the right answer is clear from the eigenvectors
%      when epsilon is small...
       
tau_0 = [intersectionParameters1(60);
         intersectionParameters2(60)];       

[theta_n, xn, tau_n, thisError] = ...
 homoclinicPoint6D_p0_prodSpace(...
    theta_0, tau_0, n_u, n_s, ...
    A0u, B0u, C0u, D0u, E0u, F0u, ...
    A0s, B0s, C0s, D0s, E0s, F0s, ...
    Nu, Ns, ...
    alpha1, tau1, a1, b1, c1, ...
    alpha2, tau2, a2, b2, c2, ...
    epsilon_coupling);
 
thisError


validationParameters = [theta_n; xn; tau_n];





[orbitsValidated, chaosValidation, radii] = ...
    validateChaosProof6D(validationParameters, ...
        n_s, n_u, Nu, Ns, scale0_s1, scale0_s2, ...
        scale0_u1, scale0_u2, scale0_u3, scale0_u4, ...
        domRad0s, domRad0u, ...
        a1, b1, c1, alpha1, tau1, ...
        a2, b2, c2, alpha2, tau2, epsilon_coupling);







