clear
format long 


%intvalinit('FastIVmult')
intvalinit('SharpIVmult')


%=================================================================
%   PROGRAM COMPUTES BOTH ONE AND TWO DIMENSIONAL STABLE AND 
%   UNSTABLE MANIFOLDS AT BOTH FIXED POINTS, BY PARAMETERIZATION
%
%   COMPUTE HOMOCLINIC ORBIT:
%
%
%
%
%================================================================



%==========================================================================
%Non-Rigorous Computation of Approximate Solution:
%==========================================================================



%==============================================================
%Computation Controls:
%==============================================================



%Map Parameter Set: Dullin to Lomeli Meiss.
   %Dullin Meiss Parms:
      a_dm = 1
      b_dm = 0.5
      c_dm = 0.5
            
      
      eps_dm = 4.5    %boundary near 6.182001
     %eps_dm = 0.32      %worked at 0.4
      mu_dm = -2.4
      
      
   %Convert to Lomeli:   
      a = c_dm
      b = b_dm - 2*c_dm
      c = c_dm + a_dm - b_dm
      
      tau =    2*a_dm*(3+mu_dm)/(2*a_dm - b_dm)
      alpha =  ((9+6*mu_dm+mu_dm^2)*a_dm - 4*eps_dm*a_dm^2 + ...
               4*eps_dm*a_dm*b_dm - eps_dm*b_dm^2)/(2*a_dm - b_dm)^2



%---for epsilon > 3.5-ish guess_tol ~ 10^(-1) is good.
%---for epsilon ~ 0.2 - 2 neew  guess_tol ~ 2.5 X 10^(-2)
%---what to do close to eps ~ 0.1?
guess_tol = 2.5*10^(-2) 

l = 20

numRad = 30;
numTheta = 250;

%Order of manifold computation:
  %Order of 2D manifolds:
  N = 7;
  %Order of 1D manifold computations 
  order_1 = 7;
   
  n_u = 3;
  ns_0 = 1
  
%==========================================================================
%Generate zeroth and first order problem data:
%==========================================================================

[p0, p1, lambda0_s, lambda0_u1, lambda0_u2, ...
         lambda1_u, lambda1_s1, lambda1_s2, ...
         xi0_s, xi0_u1, xi0_u2, ...
         xi1_u, xi1_s1, xi1_s2] ...
     = firstOrderData_3DHenon(a, b, c, alpha, tau) 
  
 
 
%=====================================================================
%One-D Manifolds 
%=====================================================================

%Step 1: Compute 1D Coefficients:

scale = 15;

[A0s, B0s, C0s] = oneDimManifold_3DHenon(a, b, c, alpha, tau, ...
     order_1, p0, lambda0_s, xi0_s, scale);

scale = 15
 
[A1u, B1u, C1u] = oneDimManifold_3DHenon(a, b, c, alpha, tau, ...
    order_1, p1, lambda1_u, xi1_u, scale);


figure
hold on 
plot(log(abs(A0s))/log(10),'b')
plot(log(abs(B0s))/log(10),'g')
plot(log(abs(C0s))/log(10),'r')

plot(log(abs(A1u))/log(10), 'b.')
plot(log(abs(B1u))/log(10), 'g.')
plot(log(abs(C1u))/log(10), 'r.')

domRad0s = 1
domRad1u = 0.5

error_0s = Error_1D(A0s, B0s, C0s, alpha, tau, ...
                               a, b, c, lambda0_s, order_1, domRad0s)



error_1u = Error_1D(A1u, B1u, C1u, alpha, tau, ...
                               a, b, c, lambda1_u, order_1, domRad1u)
          
                           
                           

                           
                           


%Step 3: Compute fundamental domain point clouds:
numPoints = 15000;

[I1u_left, I1u_right, W1u_fd_left, W1u_fd_right, ...
    I1u, W1u_loc] = ...
    oneDim_fd(A1u, B1u, C1u, order_1, lambda1_u, domRad1u, numPoints);

[I0s_left, I0s_right, W0s_fd_left, W0s_fd_right, ...
    I0s, W0s_loc] = ...
    oneDim_fd(A0s, B0s, C0s, order_1, lambda0_s, domRad0s, numPoints);


%[I1u_left, I1u_right, W1u_fd_left, W1u_fd_right] = ...
%    oneDim_fd(A1u, B1u, C1u, order_1, lambda1_u, r1_u, numPoints);

%[I0s_left, I0s_right, W0s_fd_left, W0s_fd_right] = ...
%    oneDim_fd(A0s, B0s, C0s, order_1, lambda0_s, r0_s, numPoints);

%==========================================================================
%Two-D Manifolds:
%==========================================================================

%Step 1: Compute Coefficients:
scale1 = 10;
scale2 = 10;

[A0u, B0u, C0u] = twoDimManifold_3DHenon(a, b, c, alpha, tau, ...
    N, p0, lambda0_u1, lambda0_u2, xi0_u1, xi0_u2, scale1, scale2);

[A1s, B1s, C1s] = twoDimManifold_3DHenon(a, b, c, alpha, tau, ...
    N, p1, lambda1_s1, lambda1_s2, xi1_s1, xi1_s2, scale1, scale2);

 

domRad0u = 0.3
domRad1s = 0.2

error_1s = Error_2D(A1s, B1s, C1s, alpha, tau, ...
                    a, b, c, lambda1_s1, lambda1_s2, N, domRad1s)



error_0u = Error_2D(A0u, B0u, C0u, alpha, tau, ...
                    a, b, c, lambda0_u1, lambda0_u2, N, domRad0u)






%Step 3: Compute 2D fundamental domains:

[D0u_fd, W0u_fd] = twoDim_fd(A0u, B0u, C0u, N, ...
                             lambda0_u1, r0_u, numRad, numTheta);
[D1s_fd, W1s_fd] = twoDim_fd(A1s, B1s, C1s, N, ...
                             lambda1_s1, r1_s, numRad, numTheta);

                         
                         
                         
                         
                         
                         
                         
                         
                         
                         